@@ -1,8 +1,11 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * kernel/lvm.h
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * February-November 1997
- * May,June 1998
+ * May-July 1998
+ * January 1999
  *
  * lvm is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,16 +27,46 @@
 /*
  * Changelog
  *
- *     10/10/1997 - beginning of new structure creation
- *     05/12/1998 - incorporated structures from lvm_v1.h and deleted lvm_v1.h
- *     06/07/1998 - avoided LVM_KMALLOC_MAX define by using vmalloc/vfree
- *                  instead of kmalloc/kfree
+ *    10/10/1997 - beginning of new structure creation
+ *    05/12/1998 - incorporated structures from lvm_v1.h and deleted lvm_v1.h
+ *    06/07/1998 - avoided LVM_KMALLOC_MAX define by using vmalloc/vfree
+ *                 instead of kmalloc/kfree
+ *    07/01/1998 - fixed wrong LVM_MAX_SIZE
+ *    07/07/1998 - extended pe_t structure by ios member (for statistic)
+ *    08/02/1998 - changes for official char/block major numbers
+ *    08/07/1998 - avoided init_module() and cleanup_module() to be static
+ *    08/29/1998 - seprated core and disk structure type definitions
+ *    09/01/1998 - merged kernel integration version (mike)
+ *    01/20/1999 - added LVM_PE_DISK_OFFSET macro for use in
+ *                 vg_read_with_pv_and_lv(), pv_move_pe(), pv_show_pe_text() ...
  *
  */
 
+
 #ifndef _LVM_H_INCLUDE
 #define _LVM_H_INCLUDE
 
+
+/* preprocessor definitions */
+#define	LVM_TOTAL_RESET
+#define LVM_GET_INODE
+
+/* CONFIG_LVM_PROC_FS moved to config.h ( mike) */
+/* #define CONFIG_LVM_PROC_FS */
+/*
+#define DEBUG_LVM_GET_INFO
+#define DEBUG
+#define DEBUG_MAP
+#define DEBUG_MAP_SIZE
+#define DEBUG_IOCTL
+#define DEBUG_READ
+#define DEBUG_GENDISK
+#define DEBUG_VG_CREATE
+#define DEBUG_LVM_BLK_OPEN
+#define DEBUG_VFREE
+*/
+/* end of preprocessor definitions */
+
 #ifndef LINUX_VERSION_CODE
 #  include <linux/version.h>
    /* for 2.0.x series */
@@ -43,22 +76,44 @@
 #endif
 
 #include <linux/kdev_t.h>
+#include <linux/major.h>
+
+/* leave this for now until major.h is updated (mike) */
+#ifndef	LVM_BLOCK_MAJOR
+#  define	LVM_BLOCK_MAJOR	58
+#endif
+#ifndef	LVM_CHAR_MAJOR
+#  define	LVM_CHAR_MAJOR	109
+#endif
+
+#if !defined ( LVM_BLOCK_MAJOR) || !defined ( LVM_CHAR_MAJOR)
+   #error Bad major.h - LVM MAJOR undefined
+#endif
 
 #define LVM_STRUCT_VERSION	1  /* structure version */
 #define MIN_LVM_STRUCT_VERSION	1
 #define MAX_LVM_STRUCT_VERSION	2
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION ( 2, 1, 0)
-#  define uint8_t __u8
-#  define uint16_t __u16
-#  define uint32_t __u32
+#if LINUX_VERSION_CODE < KERNEL_VERSION ( 2, 1 ,0)
+#  ifndef	uint8_t
+#    define uint8_t __u8
+#  endif
+#  ifndef	uint16_t
+#    define uint16_t __u16
+#  endif
+#  ifndef	uint32_t
+#    define uint32_t __u32
+#  endif
 #endif
 
 /* set the default structure version */
 #if ( LVM_STRUCT_VERSION == 1)
 #  define pv_t pv_v1_t
+#  define pv_disk_t pv_disk_v1_t
 #  define lv_t lv_v1_t
+#  define lv_disk_t lv_disk_v1_t
 #  define vg_t vg_v1_t
+#  define vg_disk_t vg_disk_v1_t
 #else
 #  if ( LVM_STRUCT_VERSION == 2)
 #    define pv_t pv_v2_t
@@ -68,11 +123,10 @@
 #endif
 
 #if ( LVM_STRUCT_VERSION != 1 &&  LVM_STRUCT_VERSION != 2)
-   #pragme Please set LVM_STRUCT_VERSION above
+   #error LVM_STRUCT_VERSION is not defined
 #endif
 
 
-#define	LVM_MAJOR	42         /* my testing major number */
 #define	LVM_IOP_VERSION	        2  /* i/o protocoll version */
 #define LVM_NAME        "lvm"
 
@@ -87,11 +141,11 @@
  * this has to be enhanced!!!
  */
 #if ( MINORBITS >= 12)
-#	define LVM_MINORBITS	12
+#   define LVM_MINORBITS	12
 #   define MAX_VG_BITS 5 /* otherwise too much memory for hd_structs ... */
 #   define BITS 0xFFF
 #else
-#	define LVM_MINORBITS	8
+#   define LVM_MINORBITS	8
 #   define MAX_VG_BITS 3
 #   define BITS 0xFF
 #endif
@@ -124,33 +178,38 @@
 #define	LVM_LV_DISK_OFFSET(a, b)	( (a)->lv_on_disk.base + sizeof ( lv_t) * b)
 #define	LVM_DISK_SIZE(pv)		( (pv)->pe_on_disk.base + \
                                           (pv)->pe_on_disk.size)
+#define	LVM_PE_DISK_OFFSET(pe, pv)	( pe * pv->pe_size + \
+					  ( LVM_DISK_SIZE ( pv) / SECTOR_SIZE))
 
 
 /*
  * LVM_PE_T_MAX corresponds to:
  *
- * 8KB PE size can map a ~512 MB logical volume at the cost of 512 KB memory,
+ * 8KB PE size can map a ~512 MB logical volume at the cost of 1MB memory,
  *
- * 128MB PE size can map a 8TB logical volume at the sam cost memory.
+ * 128MB PE size can map a 8TB logical volume at the same cost of memory.
  *
  * Default PE size of 4 MB gives a maximum logical volume size of 256 GB.
  *
- * AFAIK, the kernel is able to handle 1 TB for one disk (array).
+ * Maximum PE size of 16GB gives a maximum logical volume size of 1024TB.
+ *
+ * AFAIK, the actual kernels limit this to 1 TB.
  *
  * Should be a sufficient spectrum ;*)
  */
 
-/* This is the size of disk_pe_t.le_num !!!          v     v */
+/* This is the usable size of disk_pe_t.le_num !!!          v     v */
 #define	LVM_PE_T_MAX		( ( 1 << ( sizeof ( uint16_t) * 8)) - 2)
+
 #define	LVM_LV_SIZE_MAX(a)	( ( long long) LVM_PE_T_MAX * (a)->pe_size)
 #define	LVM_MIN_PE_SIZE		( 8L * 2)           /* 8 KB in sectors */
-#define	LVM_MAX_PE_SIZE		( 10240L * 1024L * 2) /* 10GB in sectors */
+#define	LVM_MAX_PE_SIZE		( 16L * 1024L * 1024L * 2) /* 16GB in sectors */
 #define	LVM_DEFAULT_PE_SIZE	( 4096L * 2)        /* 4 MB in sectors */
 #define	LVM_DEFAULT_STRIPE_SIZE	16L                 /* 16 KB  */
 #define	LVM_MIN_STRIPE_SIZE	2L                  /* 1 KB in sectors */
-#define	LVM_MAX_STRIPE_SIZE	( 2 * 128L)         /* 128 KB in sectors */
+#define	LVM_MAX_STRIPE_SIZE	( 128L * 2)         /* 128 KB in sectors */
 #define	LVM_MAX_STRIPES		128					/* max # of stripes */
-#define	LVM_MAX_SIZE            ( 1024L * 1024L * 1024L) /* 1TB */
+#define	LVM_MAX_SIZE            ( 1024LU * 1024 * 1024 * 2) /* 1TB[sectors] */
 #define	LVM_MAX_MIRRORS    	2		/* future use */
 #define	LVM_MIN_READ_AHEAD	2		/* min read ahead sectors (future use) */
 #define	LVM_MAX_READ_AHEAD	120	/* max read ahead sectors (future use) */
@@ -168,49 +227,49 @@
  */
 
 /* volume group */
-#define	VG_CREATE               _IO ( 0xfe, 0x00)
-#define	VG_REMOVE               _IO ( 0xfe, 0x01)
+#define	VG_CREATE               _IOW ( 0xfe, 0x00, 1)
+#define	VG_REMOVE               _IOW ( 0xfe, 0x01, 1)
 
-#define	VG_EXTEND               _IO ( 0xfe, 0x03)
-#define	VG_REDUCE               _IO ( 0xfe, 0x04)
+#define	VG_EXTEND               _IOW ( 0xfe, 0x03, 1)
+#define	VG_REDUCE               _IOW ( 0xfe, 0x04, 1)
 
-#define	VG_STATUS               _IO ( 0xfe, 0x05)
-#define	VG_STATUS_GET_COUNT     _IO ( 0xfe, 0x06)
-#define	VG_STATUS_GET_NAMELIST  _IO ( 0xfe, 0x07)
+#define	VG_STATUS               _IOWR ( 0xfe, 0x05, 1)
+#define	VG_STATUS_GET_COUNT     _IOWR ( 0xfe, 0x06, 1)
+#define	VG_STATUS_GET_NAMELIST  _IOWR ( 0xfe, 0x07, 1)
 
-#define	VG_SET_EXTENDABLE       _IO ( 0xfe, 0x08)
+#define	VG_SET_EXTENDABLE       _IOW ( 0xfe, 0x08, 1)
 
 
 /* logical volume */
-#define	LV_CREATE               _IO ( 0xfe, 0x20)
-#define	LV_REMOVE               _IO ( 0xfe, 0x21)
+#define	LV_CREATE               _IOW ( 0xfe, 0x20, 1)
+#define	LV_REMOVE               _IOW ( 0xfe, 0x21, 1)
 
 #define	LV_ACTIVATE             _IO ( 0xfe, 0x22)
 #define	LV_DEACTIVATE           _IO ( 0xfe, 0x23)
 
-#define	LV_EXTEND               _IO ( 0xfe, 0x24)
-#define	LV_REDUCE               _IO ( 0xfe, 0x25)
+#define	LV_EXTEND               _IOW ( 0xfe, 0x24, 1)
+#define	LV_REDUCE               _IOW ( 0xfe, 0x25, 1)
 
-#define	LV_STATUS_BYNAME        _IO ( 0xfe, 0x26)
-#define	LV_STATUS_BYINDEX       _IO ( 0xfe, 0x27)
+#define	LV_STATUS_BYNAME        _IOWR ( 0xfe, 0x26, 1)
+#define	LV_STATUS_BYINDEX       _IOWR ( 0xfe, 0x27, 1)
 
-#define LV_SET_ACCESS           _IO ( 0xfe, 0x28)
-#define LV_SET_ALLOCATION       _IO ( 0xfe, 0x29)
-#define LV_SET_STATUS           _IO ( 0xfe, 0x2a)
+#define LV_SET_ACCESS           _IOW ( 0xfe, 0x28, 1)
+#define LV_SET_ALLOCATION       _IOW ( 0xfe, 0x29, 1)
+#define LV_SET_STATUS           _IOW ( 0xfe, 0x2a, 1)
 
-#define LE_REMAP                _IO ( 0xfe, 0x2b)
+#define LE_REMAP                _IOW ( 0xfe, 0x2b, 1)
 
 
 /* physical volume */
-#define	PV_STATUS               _IO ( 0xfe, 0x40)
-#define	PV_CHANGE               _IO ( 0xfe, 0x41)
-#define	PV_FLUSH                _IO ( 0xfe, 0x42)
+#define	PV_STATUS               _IOWR ( 0xfe, 0x40, 1)
+#define	PV_CHANGE               _IOWR ( 0xfe, 0x41, 1)
+#define	PV_FLUSH                _IOW ( 0xfe, 0x42, 1)
 
-/* physical extend */
-#define	PE_LOCK_UNLOCK          _IO ( 0xfe, 0x50)
+/* physical extent */
+#define	PE_LOCK_UNLOCK          _IOW ( 0xfe, 0x50, 1)
 
 /* i/o protocol version */
-#define	LVM_GET_IOP_VERSION     _IO ( 0xfe, 0x98)
+#define	LVM_GET_IOP_VERSION     _IOR ( 0xfe, 0x98, 1)
 
 /* special reset function for testing purposes */
 #define	LVM_RESET               _IO ( 0xfe, 0x99)
@@ -304,6 +363,31 @@ typedef struct {
    struct inode     *inode;       /* HM */
 } pv_v1_t;
 
+typedef struct {
+   uint8_t          id[2];                 /* Identifier */
+   uint16_t         version;               /* HM lvm version */
+   lvm_disk_data_t  pv_on_disk;
+   lvm_disk_data_t  vg_on_disk;
+   lvm_disk_data_t  pv_namelist_on_disk;
+   lvm_disk_data_t  lv_on_disk;
+   lvm_disk_data_t  pe_on_disk;
+   uint8_t          pv_name[NAME_LEN];
+   uint8_t          vg_name[NAME_LEN];
+   uint8_t          system_id[NAME_LEN]; /* for vgexport/vgimport */
+   uint32_t         pv_major;
+   uint32_t         pv_number;
+   uint32_t         pv_status;
+   uint32_t         pv_allocatable;
+   uint32_t         pv_size;		/* HM */
+   uint32_t         lv_cur;
+   uint32_t         pe_size;
+   uint32_t         pe_total;
+   uint32_t         pe_allocated;
+   uint32_t         dummy1;
+   uint32_t         dummy2;
+   uint32_t         dummy3;
+} pv_disk_v1_t;
+
 
 /*
  * Structure Physical Volume (PV) Version 2
@@ -345,8 +429,10 @@ typedef struct {
 
 /* core PE information */
 typedef struct {
-   kdev_t dev;
-   uint32_t      pe;
+   kdev_t   dev;
+   uint32_t pe;
+   uint32_t reads;
+   uint32_t writes;
 } pe_t;
 
 typedef struct {
@@ -367,11 +453,11 @@ typedef struct {
    uint8_t        vg_name[NAME_LEN];
    uint32_t       lv_access;
    uint32_t       lv_status;
-   uint32_t       lv_open;			/* HM */
-   kdev_t         lv_dev;			/* HM */
+   uint32_t       lv_open;		/* HM */
+   kdev_t         lv_dev;		/* HM */
    uint32_t       lv_number;		/* HM */
    uint32_t       lv_mirror_copies;	/* for future use */
-   uint32_t       lv_recovery;      /*       "        */
+   uint32_t       lv_recovery;          /*       "        */
    uint32_t       lv_schedule;		/*       "        */
    uint32_t       lv_size;
    pe_t           *lv_current_pe;	/* HM */
@@ -385,6 +471,29 @@ typedef struct {
    uint32_t       lv_read_ahead;	/* HM, for future use */
 } lv_v1_t;
 
+typedef struct {
+   uint8_t        lv_name[NAME_LEN];
+   uint8_t        vg_name[NAME_LEN];
+   uint32_t       lv_access;
+   uint32_t       lv_status;
+   uint32_t       lv_open;		/* HM */
+   uint32_t       lv_dev;		/* HM */
+   uint32_t       lv_number;		/* HM */
+   uint32_t       lv_mirror_copies;	/* for future use */
+   uint32_t       lv_recovery;          /*       "        */
+   uint32_t       lv_schedule;		/*       "        */
+   uint32_t       lv_size;
+   uint32_t       dummy;
+   uint32_t       lv_current_le;	/* for future use */
+   uint32_t       lv_allocated_le;
+   uint32_t       lv_stripes;
+   uint32_t       lv_stripesize;
+   uint32_t       lv_badblock;		/* for future use */
+   uint32_t       lv_allocation;
+   uint32_t       lv_io_timeout;	/* for future use */
+   uint32_t       lv_read_ahead;	/* HM, for future use */
+} lv_disk_v1_t;
+
 
 /*
  * Structure Logical Volume (LV) Version 2
@@ -395,11 +504,11 @@ typedef struct {
    uint8_t        vg_name[NAME_LEN];
    uint32_t       lv_access;
    uint32_t       lv_status;
-   uint32_t       lv_open;			/* HM */
-   kdev_t         lv_dev;			/* HM */
+   uint32_t       lv_open;		/* HM */
+   kdev_t         lv_dev;		/* HM */
    uint32_t       lv_number;		/* HM */
    uint32_t       lv_mirror_copies;	/* for future use */
-   uint32_t       lv_recovery;      /*       "        */
+   uint32_t       lv_recovery;          /*       "        */
    uint32_t       lv_schedule;		/*       "        */
    uint32_t       lv_size;
    pe_t           *lv_current_pe;	/* HM */
@@ -430,19 +539,37 @@ typedef struct {
    uint32_t       lv_cur;                /* current logical volumes */
    uint32_t       lv_open;               /* open    logical volumes */
    uint32_t       pv_max;        /* maximum physical volumes */
-   uint32_t       pv_cur;		 /* current physical volumes FU */
+   uint32_t       pv_cur;	 /* current physical volumes FU */
    uint32_t       pv_act;        /* active physical volumes */
-   uint32_t       max_pe_per_pv; /* maximum physical extends per physical vol */
-   uint32_t       vgda;			 /* volume group descriptor arrays FU */
-   uint32_t       pe_size;       /* physical extend size in sectors */
-   uint32_t       pe_total;      /* total of physical extends */
-   uint32_t       pe_allocated;  /* allocated physical extends */
-   uint32_t       pvg_total;		 /* physical volume groups FU */
+   uint32_t       dummy;         /* was obsolete max_pe_per_pv */
+   uint32_t       vgda;		 /* volume group descriptor arrays FU */
+   uint32_t       pe_size;       /* physical extent size in sectors */
+   uint32_t       pe_total;      /* total of physical extents */
+   uint32_t       pe_allocated;  /* allocated physical extents */
+   uint32_t       pvg_total;	 /* physical volume groups FU */
    struct proc_dir_entry *proc;
    pv_t           *pv[ABS_MAX_PV+1]; /* physical volume struct pointers */
    lv_t           *lv[ABS_MAX_LV+1]; /* logical  volume struct pointers */
 } vg_v1_t;
 
+typedef struct {
+   uint8_t        vg_name[NAME_LEN];     /* volume group name */
+   uint32_t       vg_number;             /* volume group number */
+   uint32_t       vg_access;             /* read/write */
+   uint32_t       vg_status;             /* active or not */
+   uint32_t       lv_max;                /* maximum logical volumes */
+   uint32_t       lv_cur;                /* current logical volumes */
+   uint32_t       lv_open;               /* open    logical volumes */
+   uint32_t       pv_max;        /* maximum physical volumes */
+   uint32_t       pv_cur;	 /* current physical volumes FU */
+   uint32_t       pv_act;        /* active physical volumes */
+   uint32_t       dummy;
+   uint32_t       vgda;		 /* volume group descriptor arrays FU */
+   uint32_t       pe_size;       /* physical extent size in sectors */
+   uint32_t       pe_total;      /* total of physical extents */
+   uint32_t       pe_allocated;  /* allocated physical extents */
+   uint32_t       pvg_total;	 /* physical volume groups FU */
+} vg_disk_v1_t;
 
 /*
  * Structure Volume Group (VG) Version 2
@@ -461,9 +588,9 @@ typedef struct {
    uint32_t       pv_act;                /* future: active physical volumes */
    uint32_t       max_pe_per_pv;         /* OBSOLETE maximum PE/PV */
    uint32_t       vgda;			 /* volume group descriptor arrays FU */
-   uint32_t       pe_size;               /* physical extend size in sectors */
-   uint32_t       pe_total;              /* total of physical extends */
-   uint32_t       pe_allocated;          /* allocated physical extends */
+   uint32_t       pe_size;               /* physical extent size in sectors */
+   uint32_t       pe_total;              /* total of physical extents */
+   uint32_t       pe_allocated;          /* allocated physical extents */
    uint32_t       pvg_total;		 /* physical volume groups FU */
    struct proc_dir_entry *proc;
    pv_t           *pv[ABS_MAX_PV+1]; /* physical volume struct pointers */
