@@ -1,25 +1,35 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_check_consistency.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * December 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *   12/20/1998 - checked for correct LV number lv lv_check_consistency_all_lv()
+ *
+ */
+
 #include <liblvm.h>
 
 int lv_check_consistency ( lv_t *lv) {
@@ -88,8 +98,13 @@ int lv_check_consistency_all_lv ( vg_t *vg) {
          debug ( "lv_check_consistency_all_lv -- vg->lv[%d]: %X  name: %s\n",
                   l, ( uint) vg->lv[l], vg->lv[l]->lv_name);
 #endif
-         if ( vg->lv[l] != NULL)
+         if ( vg->lv[l] != NULL) {
             if ( ( ret = lv_check_consistency ( vg->lv[l])) < 0) break;
+            if ( vg->lv[l]->lv_number != l) {
+               ret = -LVM_ELV_NUMBER;
+               break;
+            }
+         }
       }
    }
 #ifdef DEBUG
