@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_check_on_pv.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * April 1997
- * June 1998
+ * June,July 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -25,6 +27,7 @@
  * Changelog
  *
  *    06/12/1998 - reworked return output
+ *    07/05/1998 - fixed bug in PE loop if
  *
  */
 
@@ -44,8 +47,10 @@ int lv_check_on_pv ( pv_t *pv, int lv_num) {
    }
 
    for ( pe = 0; pe < pv->pe_total; pe++) {
-      if ( pv->pe[pe].lv_num == lv_num) ret = TRUE;
-      break;
+      if ( pv->pe[pe].lv_num == lv_num) {
+         ret = TRUE;
+         break;
+      }
    }
 
 lv_check_on_pv_end:
