@@ -1,20 +1,22 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_create_kdev_t.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -26,6 +28,7 @@
  *    01/11/1998 - changed to use number_from_name functions
  *                 for free logical volume names
  *    04/29/1998 - rewritten to avoid lv_name usage
+ *    08/02/1998 - used official LVM block major
  *
  */
 
@@ -44,6 +47,6 @@ kdev_t lv_create_kdev_t ( ulong vg_number, ulong lv_number) {
 #ifdef DEBUG
    debug ( "lv_create_kdev_t -- LEAVING\n");
 #endif
-   return ( LVM_MAJOR << LVM_MINORBITS) +
+   return ( LVM_BLOCK_MAJOR << LVM_MINORBITS) +
             ( vg_number << VG_BITSHIFT) + lv_number;
 }
