@@ -1,37 +1,47 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_create_remove.c
+ *
+ * Copyright (C) 1997, 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
+ *
+ */
+
 #include <liblvm.h>
 
 /* internal function */
 int lv_create_remove ( vg_t *, lv_t *, char *, int);
 
 
-int lv_create ( vg_t *vg, lv_t *lv, char *lv_name) {
+inline int lv_create ( vg_t *vg, lv_t *lv, char *lv_name) {
    return lv_create_remove ( vg, lv, lv_name, LV_CREATE);
 }
 
 
-int lv_remove ( vg_t *vg, lv_t *lv, char *lv_name) {
+inline int lv_remove ( vg_t *vg, lv_t *lv, char *lv_name) {
    return lv_create_remove ( vg, lv, lv_name, LV_REMOVE);
 }
 
@@ -56,7 +66,7 @@ int lv_create_remove ( vg_t *vg, lv_t *lv, char *lv_name, int cr) {
       return -LVM_EPARAM;
    }
 
-   sprintf ( group_file, "/dev/%s/group%c", vg->vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg->vg_name, 0);
    strcpy ( req.lv_name, lv_name);
    req.lv = lv;
    if ( ( group = open ( group_file, O_RDWR)) == -1)
