@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_read.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * May   1998
+ * May,August,December 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -29,6 +31,9 @@
  *    05/01/1998 - rewritten lv_read() and lv_read_all_lv_of_vg()
  *                 to avoid LV number lookup from LV name
  *               - implemented lv_read_byindex()
+ *    08/29/1998 - seperated disk and core lv structures in
+ *                 lv_read_all_lv_of_vg() by using lv_copy_from_disk()
+ *    12/16/1998 - fixed free() bug
  *
  */
 
@@ -189,7 +194,7 @@ int lv_read_all_lv_of_vg ( char *vg_name, lv_t ***lv, int reread) {
    vg_t vg_this_sav;
    static char vg_name_sav[NAME_LEN] = { 0, };
    static lv_t **lv_this_ptr = NULL;
-   static lv_t *lv_this = NULL;
+   static lv_disk_t *lv_this = NULL;
    pv_t **pv_tmp = NULL;
 
 #ifdef DEBUG
@@ -212,10 +217,6 @@ int lv_read_all_lv_of_vg ( char *vg_name, lv_t ***lv, int reread) {
          free ( lv_this_ptr);
          lv_this_ptr = NULL;
       }
-      if ( lv_this != NULL) {
-         free ( lv_this);
-         lv_this = NULL;
-      }
       first = 0;
    }
 
@@ -240,7 +241,7 @@ int lv_read_all_lv_of_vg ( char *vg_name, lv_t ***lv, int reread) {
       }
       memset ( lv_this_ptr, 0, size);
 
-      size = vg_this->lv_max * sizeof ( lv_t);
+      size = vg_this->lv_max * sizeof ( lv_disk_t);
       if ( ( lv_this = malloc ( size)) == NULL) {
          free ( lv_this_ptr);
          lv_this_ptr = NULL;
@@ -267,8 +268,10 @@ int lv_read_all_lv_of_vg ( char *vg_name, lv_t ***lv, int reread) {
          nl = 0;
          for ( l = 0; l < vg_this->lv_max; l++) {
             lv_this_ptr[l] = NULL;
-            if ( vg_this->lv[l] != NULL) {
-               lv_this_ptr[l] = &lv_this[l];
+            /* changed check to avoid pointer access to vg->lv[] */
+            if ( lv_this[l].lv_name[0] != 0) {
+               /* lv_copy_from_disk has its own malloc! */
+               lv_this_ptr[l] = lv_copy_from_disk ( &lv_this[l]);
                nl++;
             }
          }
@@ -286,6 +289,8 @@ int lv_read_all_lv_of_vg ( char *vg_name, lv_t ***lv, int reread) {
    }
 
    *lv = lv_this_ptr;
+   free ( lv_this);
+   lv_this = NULL;
    if ( pv_handle != -1) close ( pv_handle);
 
 #ifdef DEBUG
