@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_setup_for_create.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March-May,November 1997
- * April,May,June 1998
+ * April-July 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -29,6 +32,8 @@
  *    05/16/1998 - avoided lv_create_name()
  *    06/14/1998 - added allocation policy on specified PVs for
  *                 striped LVs
+ *    07/05/1998 - fixed endless loop bug
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -39,7 +44,6 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
                           uint stripesize, uint mirrors, uint allocation,
                           uint permission, char **pv_allowed) {
    int l = 0;
-   int lv_kdev = 0;
    disk_pe_t lv_pe = { 0, 0};
    int p = 0;
    int pa = 0;
@@ -94,13 +98,14 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
             l, ( uint) vg_this->lv[l]);
 #endif
    if ( strlen ( lv_name) == 0) {
-      sprintf ( lv_name, "/dev/%s/lvol%d%c", vg_name, l + 1, 0);
+      sprintf ( lv_name, LVM_DIR_PREFIX "%s/lvol%d%c", vg_name, l + 1, 0);
    } else {
       strncpy ( lv_name_this, lv_name,
-                sizeof ( lv_name_this) - strlen ( vg_name) - 7);
-      sprintf ( lv_name, "/dev/%s/%s%c", vg_name, lv_name_this, 0);
+                sizeof ( lv_name_this) -
+                strlen ( vg_name) -
+                strlen ( LVM_DIR_PREFIX) - 2);
+      sprintf ( lv_name, LVM_DIR_PREFIX "%s/%s%c", vg_name, lv_name_this, 0);
    }
-   lv_kdev = lv_create_kdev_t ( vg_this->vg_number, l + 1);
    lv_pe.lv_num = l + 1;
    lv_pe.le_num = 0;
 #ifdef DEBUG
@@ -110,7 +115,7 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
    strcpy ( vg_this->lv[l]->lv_name, lv_name);
    vg_this->lv[l]->lv_access = permission;
    vg_this->lv[l]->lv_status = LV_ACTIVE;
-   vg_this->lv[l]->lv_dev = lv_kdev;
+   vg_this->lv[l]->lv_dev = lv_create_kdev_t ( vg_this->vg_number, l + 1);
    vg_this->lv[l]->lv_number = l;
    vg_this->lv[l]->lv_open = 0;
    vg_this->lv[l]->lv_mirror_copies = 0;
@@ -139,7 +144,7 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
       return -LVM_ELV_SETUP_FOR_CREATE_MALLOC;
    }
 
-   /* walk through physical volumes searching for physical extends */
+   /* walk through physical volumes searching for physical extents */
    stripes = vg_this->lv[l]->lv_stripes;
    pe = vg_this->lv[l]->lv_allocated_le;
 
@@ -148,40 +153,39 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
    if ( stripes < 2) {
       if ( pe > vg_this->pe_total - vg_this->pe_allocated)
          return -LVM_ELV_SETUP_FOR_CREATE_PE;
-      while ( p < vg_this->pv_max && pe > 0) {
-         if ( vg_this->pv[p] != NULL) {
-            if ( pv_allowed != NULL) {
-               for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
-                  if ( strcmp ( vg_this->pv[p]->pv_name, pv_allowed[pa]) == 0)
-                     break;
-               }
-               if ( pv_allowed[pa] == NULL) {
-                  p++;
-                  continue;
-               }
+      while ( p < vg_this->pv_cur && pe > 0) {
+         if ( pv_allowed != NULL) {
+            for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
+               if ( strcmp ( vg_this->pv[p]->pv_name, pv_allowed[pa]) == 0)
+                  break;
             }
-            if ( ! ( vg_this->pv[p]->pv_allocatable & PV_ALLOCATABLE)) {
-#ifdef DEBUG
-               debug ( "lv_setup_for_create -- %s NOT allocatable\n",
-                         vg_this->pv[p]->pv_name);
-#endif
+            if ( pv_allowed[pa] == NULL) {
                p++;
                continue;
             }
-            pe_last = pe;
-            pe_p = &vg_this->lv[l]->lv_current_pe[vg_this->\
-                   lv[l]->lv_allocated_le-pe];
-            if ( ( ret = pv_reserve_pe ( vg_this->pv[p], &lv_pe, &pe, pe_p,
-                                         vg_this->lv[l]->lv_allocation,
-                                         TRUE)) < 0)
-               return -LVM_ESIZE;
-            if ( pe < pe_last) vg_this->pv[p]->lv_cur++;
+         }
+
+         if ( ! ( vg_this->pv[p]->pv_allocatable & PV_ALLOCATABLE)) {
 #ifdef DEBUG
-            debug ( "lv_setup_for_create -- pv_reserve_pe returned: %d   "
-                     "pe_last: %d  pe: %d\n", ret, pe_last, pe);
+            debug ( "lv_setup_for_create -- %s NOT allocatable\n",
+                      vg_this->pv[p]->pv_name);
 #endif
             p++;
+            continue;
          }
+         pe_last = pe;
+         pe_p = &vg_this->lv[l]->lv_current_pe[vg_this->\
+                lv[l]->lv_allocated_le-pe];
+         if ( ( ret = pv_reserve_pe ( vg_this->pv[p], &lv_pe, &pe, pe_p,
+                                      vg_this->lv[l]->lv_allocation,
+                                      TRUE)) < 0)
+            return -LVM_ESIZE;
+         if ( pe < pe_last) vg_this->pv[p]->lv_cur++;
+#ifdef DEBUG
+         debug ( "lv_setup_for_create -- pv_reserve_pe returned: %d   "
+                  "pe_last: %d  pe: %d\n", ret, pe_last, pe);
+#endif
+         p++;
       }
 #ifdef DEBUG
       debug ( "lv_setup_for_create -- pe: %d\n", pe);
@@ -195,49 +199,47 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
       pe_this = pe_last = pe / stripes;
       s = stripes;
       while ( p < vg_this->pv_cur && s > 0) {
-         if ( vg_this->pv[p] != NULL) {
-            if ( pv_allowed != NULL) {
-               for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
-                  if ( strcmp ( vg_this->pv[p]->pv_name, pv_allowed[pa]) == 0)
-                     break;
-               }
-               if ( pv_allowed[pa] == NULL) {
-                  p++;
-                  continue;
-               }
+         if ( pv_allowed != NULL) {
+            for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
+               if ( strcmp ( vg_this->pv[p]->pv_name, pv_allowed[pa]) == 0)
+                  break;
             }
-
-            if ( ! ( vg_this->pv[p]->pv_allocatable & PV_ALLOCATABLE)) {
-#ifdef DEBUG
-               debug ( "lv_setup_for_create -- %s NOT allocatable\n",
-                         vg_this->pv[p]->pv_name);
-#endif
+            if ( pv_allowed[pa] == NULL) {
                p++;
                continue;
             }
-            if ( pe_this <= vg_this->pv[p]->pe_total -
-                            vg_this->pv[p]->pe_allocated) {
-               pe_this = pe_last;
-               pe_p = &vg_this->lv[l]->lv_current_pe[vg_this->lv[l]->
-                      lv_allocated_le-pe];
-#ifdef DEBUG
-               debug ( "lv_setup_for_create -- pe_this: %d\n", pe_this);
-#endif
-               if ( ( ret = pv_reserve_pe ( vg_this->pv[p],
-                                            &lv_pe, &pe_this,
-                                            pe_p, allocation, TRUE)) < 0)
-                  return -LVM_ESIZE;
-#ifdef DEBUG
-               debug ( "lv_setup_for_create -- pe_this: %d\n", pe_this);
-#endif
-               if ( pe_this == 0) {
-                  vg_this->pv[p]->lv_cur++;
-                  pe -= pe_last;
-                  s--;
-               } else return -LVM_ESIZE;
-            }
-            p++;
          }
+
+         if ( ! ( vg_this->pv[p]->pv_allocatable & PV_ALLOCATABLE)) {
+#ifdef DEBUG
+            debug ( "lv_setup_for_create -- %s NOT allocatable\n",
+                      vg_this->pv[p]->pv_name);
+#endif
+            p++;
+            continue;
+         }
+         if ( pe_this <= vg_this->pv[p]->pe_total -
+                         vg_this->pv[p]->pe_allocated) {
+            pe_this = pe_last;
+            pe_p = &vg_this->lv[l]->lv_current_pe[vg_this->lv[l]->
+                   lv_allocated_le-pe];
+#ifdef DEBUG
+            debug ( "lv_setup_for_create -- pe_this: %d\n", pe_this);
+#endif
+            if ( ( ret = pv_reserve_pe ( vg_this->pv[p],
+                                         &lv_pe, &pe_this,
+                                         pe_p, allocation, TRUE)) < 0)
+               return -LVM_ESIZE;
+#ifdef DEBUG
+            debug ( "lv_setup_for_create -- pe_this: %d\n", pe_this);
+#endif
+            if ( pe_this == 0) {
+               vg_this->pv[p]->lv_cur++;
+               pe -= pe_last;
+               s--;
+            } else return -LVM_ESIZE;
+         }
+         p++;
       }
       if ( pe != 0) return -LVM_ESIZE;
    }
