@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_setup_for_extend.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March-May,November 1997
  * May 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -25,6 +28,8 @@
  * Changelog
  *
  *    05/17/1998 - fixed allocation bug with striped logical volumes
+ *    01/22/1999 - fixed contiguous allocation extension bug if logical
+ *                 volume not starting on first physical volume
  *
  */
 
@@ -32,10 +37,12 @@
 
 int lv_setup_for_extend ( char *vg_name, vg_t *vg, char *lv_name,
                           uint size, char **pv_allowed) {
+   int end = 0;
    int i = 0;
    int j = 0;
    int l = 0;
    int p = 0;
+   int p_last = 0;
    int pa = 0;
    int ret = 0;
    int s = 0;
@@ -105,12 +112,21 @@ int lv_setup_for_extend ( char *vg_name, vg_t *vg, char *lv_name,
    }
 #endif
 
-   p = 0;
+   /* figure out physical volume holding last logical extent */
+   p_last = pv_get_index_by_kdev_t ( vg,
+                                     vg->lv[l]->lv_current_pe[old_le-1].dev);
+   end = 0;
    /* linear mapping */
    if ( stripes < 2) {
+      p = p_last;
       stripes = 1;
-      /* walk through physical volumes searching for physical extends */
-      while ( p < vg->pv_cur && pe > 0) {
+      /* walk through physical volumes searching for physical extents */
+      while ( ( !end ? p <= vg->pv_cur : p < p_last) && pe > 0) {
+         if ( p == vg->pv_cur) {
+            end = 1;
+            p = 0;
+         }
+
          /* check for restricted physical volume list */
          if ( pv_allowed != NULL) {
             for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
@@ -141,7 +157,7 @@ int lv_setup_for_extend ( char *vg_name, vg_t *vg, char *lv_name,
 #endif
 
          /* check for new LV on this PV */
-         if ( lv_check_on_pv ( vg->pv[p], l + 1) != TRUE) vg->pv[p]->lv_cur++;
+         if ( lv_check_on_pv ( vg->pv[p], l + 1) == FALSE) vg->pv[p]->lv_cur++;
 
          lv_pe.lv_num = l + 1;
          lv_pe.le_num = vg->lv[l]->lv_allocated_le - pe;
@@ -180,7 +196,8 @@ int lv_setup_for_extend ( char *vg_name, vg_t *vg, char *lv_name,
          }
       }
 
-      /* walk through physical volumes searching for physical extends */
+      /* walk through physical volumes searching for physical extents */
+      p = 0;
       while ( p < vg->pv_cur && s > 0) {
          /* match PV of a stripe of this LV */
          if ( vg->lv[l]->lv_current_pe\
