@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_show.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * June  1998
+ * June,July,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -28,6 +30,11 @@
  *    05/22/1998 - added free for lvm_show_size malloced memory
  *    06/27/1998 - changed to new lvm_tab_vg_read_with_pv_and_lv()
  *                 calling convention in lv_show_current_pe_text()
+ *    07/05/1998 - output # of PVs in lv_show_current_pe_text()
+ *    07/07/1998 - output # of I/Os since LV activation in
+ *                 lv_show_current_pe_text()
+ *    07/29/1998 - added PV i/o statistics in lv_show_current_pe_text()
+ *    09/07/1998 - added vg_free() in lv_show_current_pe_text()
  *
  */
 
@@ -146,7 +153,12 @@ int lv_show_current_pe_text ( lv_t *lv) {
    int l = 0;
    int p = 0;
    int pe = 0;
+   int pv_count = 0;
    int ret = 0;
+   uint32_t sum_reads = 0;
+   uint32_t reads = 0;
+   uint32_t sum_writes = 0;
+   uint32_t writes = 0;
    char *pv_name = NULL;
    kdev_t dev = 0;
    vg_t *vg = NULL;
@@ -161,31 +173,55 @@ int lv_show_current_pe_text ( lv_t *lv) {
                                                  &vg)) < 0) return ret;
    if ( ( l = lv_get_index_by_name ( vg, lv->lv_name)) < 0)
       return -LVM_ELV_SHOW_CURRENT_PE_TEXT_LV_INDEX;
-   printf ( "   --- Distribution of logical volume ---\n"
-            "   PV Name                  PE on PV\n");
-   for ( p = 0; p < vg->pv_cur; p++)
-      if ( lv_check_on_pv ( vg->pv[p], l + 1) == TRUE)
-         printf ( "   %-25s%d\n",
+   for ( p = 0; p < vg->pv_cur; p++) {
+      if ( ( ret = lv_check_on_pv ( vg->pv[p], l + 1)) == TRUE) pv_count++;
+   }
+   printf ( "   --- Distribution of logical volume on %d physical "
+            "volume%s  ---\n"
+            "   PV Name                  PE on PV     reads      writes\n",
+            pv_count, pv_count > 1 ? "s": "");
+   sum_reads = sum_writes = 0;
+   for ( p = 0; p < vg->pv_cur; p++) {
+      if ( lv_check_on_pv ( vg->pv[p], l + 1) == TRUE) {
+         reads = writes = 0;
+         for ( pe = 0; pe < lv->lv_allocated_le; pe++) {
+            if ( vg->pv[p]->pv_dev == lv->lv_current_pe[pe].dev) {
+               reads  += lv->lv_current_pe[pe].reads;
+               writes += lv->lv_current_pe[pe].writes;
+            }
+         }
+         sum_reads  += reads;
+         sum_writes += writes;
+         printf ( "   %-24s %-10d   %-9d  %-9d\n",
                   vg->pv[p]->pv_name,
-                  lv_count_pe ( vg->pv[p], l + 1));
+                  lv_count_pe ( vg->pv[p], l + 1),
+                  reads, writes);
+      }
+   }
+
+   printf ( "\n   --- logical volume i/o statistic ---\n"
+            "   %d reads  %d writes\n", sum_reads, sum_writes);
 
    printf ( "\n   --- Logical extents ---\n"
-            "   LE    PV                        PE\n");
+            "   LE    PV                        PE     reads      writes\n");
    dev = 0;
    for ( pe = 0; pe < lv->lv_allocated_le; pe++) {
       if ( lv->lv_current_pe[pe].dev != dev) {
          pv_name = pv_create_name_from_kdev_t ( lv->lv_current_pe[pe].dev);
          dev = lv->lv_current_pe[pe].dev;
       }
-      printf ( "   %05d %-25s %05u\n",
+      printf ( "   %05d %-25s %05u  %-9d  %-9d\n",
                pe,
                pv_name,
                ( lv->lv_current_pe[pe].pe -
                  vg->pv[pv_get_index_by_kdev_t
                         ( vg, lv->lv_current_pe[pe].dev)]->pe_on_disk.base /
                  SECTOR_SIZE) /
-                 ( lv->lv_size / lv->lv_allocated_le));
+                 ( lv->lv_size / lv->lv_allocated_le),
+                 lv->lv_current_pe[pe].reads,
+                 lv->lv_current_pe[pe].writes);
    }
+   vg_free ( vg, FALSE);
 
 #ifdef DEBUG
    debug ( "lv_show_current_pe_text -- LEAVING\n");
