@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_status.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * May   1998
+ * May,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -25,6 +28,11 @@
  * Changelog
  *
  *    05/01/1998 - added lv_status_byindex()
+ *    09/06/1998 - removed obsolete lv_status_with_pe() after enhancing
+ *                 lv_status_byname() and lv_status_baindex to get lv_current_pe
+ *               - avoided setting lv_current_pe pointers to NULL
+ *                 in lv_status_all_lv_of_vg()
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -38,6 +46,7 @@ int lv_status_byindex_internal ( char *, ulong, lv_t *);
 int lv_status_byname ( char *vg_name, char *lv_name, lv_t **lv) {
    int ret = 0;
    static lv_t lv_this;
+   pe_t *lv_current_pe = NULL;
 
 #ifdef DEBUG
    debug ( "lv_status_byname-- CALLED\n");
@@ -46,8 +55,18 @@ int lv_status_byname ( char *vg_name, char *lv_name, lv_t **lv) {
    if ( lv == NULL) return -LVM_EPARAM;
 
    lv_this.lv_current_pe = NULL;
-   if ( ( ret = lv_status_byname_internal ( vg_name, lv_name, &lv_this)) == 0)
+   if ( ( ret = lv_status_byname_internal ( vg_name, lv_name, &lv_this)) == 0) {
       *lv = &lv_this;
+      if ( ( lv_current_pe = malloc ( ( lv_this.lv_allocated_le + 1) *
+                                      sizeof ( pe_t))) == NULL) {
+         fprintf ( stderr, "malloc error in %s [line %d]\n",
+                           __FILE__, __LINE__);
+         return -LVM_ELV_STATUS_WITH_PE_MALLOC;
+      }
+      lv_this.lv_current_pe = lv_current_pe;
+      ret = lv_status_byname_internal ( vg_name, lv_name, &lv_this);
+      lv_this.lv_current_pe = lv_current_pe;
+   }
 
 #ifdef DEBUG
    debug ( "lv_status_byname-- LEAVING\n");
@@ -59,6 +78,7 @@ int lv_status_byname ( char *vg_name, char *lv_name, lv_t **lv) {
 int lv_status_byindex ( char *vg_name, ulong lv_index, lv_t **lv) {
    int ret = -1;
    static lv_t lv_this;
+   pe_t *lv_current_pe = NULL;
 
 #ifdef DEBUG
    debug ( "lv_status_byindex -- CALLED\n");
@@ -66,9 +86,20 @@ int lv_status_byindex ( char *vg_name, ulong lv_index, lv_t **lv) {
 
    if ( lv == NULL) return -LVM_EPARAM;
 
-   *lv = &lv_this;
    lv_this.lv_current_pe = NULL;
-   ret = lv_status_byindex_internal ( vg_name, lv_index, &lv_this);
+   if ( ( ret = lv_status_byindex_internal ( vg_name, lv_index, &lv_this))
+         == 0) {
+      *lv = &lv_this;
+      if ( ( lv_current_pe = malloc ( ( lv_this.lv_allocated_le + 1) *
+                                      sizeof ( pe_t))) == NULL) {
+         fprintf ( stderr, "malloc error in \"%s\" [line %d]\n",
+                           __FILE__, __LINE__);
+         return -LVM_ELV_STATUS_WITH_PE_MALLOC;
+      }
+      lv_this.lv_current_pe = lv_current_pe;
+      ret = lv_status_byindex_internal ( vg_name, lv_index, &lv_this);
+      lv_this.lv_current_pe = lv_current_pe;
+   }
 
 #ifdef DEBUG
    debug ( "lv_status_byindex -- LEAVING\n");
@@ -77,47 +108,6 @@ int lv_status_byindex ( char *vg_name, ulong lv_index, lv_t **lv) {
 }
 
 
-int lv_status_with_pe ( char *vg_name, char *lv_name, lv_t **lv) {
-   int ret = 0;
-   static lv_t lv_this;
-   pe_t *lv_current_pe;
-
-#ifdef DEBUG
-   debug ( "lv_status_with_pe -- CALLED\n");
-#endif
-
-   if ( lv == NULL) return -LVM_EPARAM;
-
-   *lv = &lv_this;
-
-   if ( lv_this.lv_current_pe != NULL) free ( lv_this.lv_current_pe);
-   lv_this.lv_current_pe = NULL;
-   if ( ( ret = lv_status_byname_internal ( vg_name, lv_name, &lv_this)) < 0)
-      return ret;
-
-   if ( ( lv_current_pe = malloc ( ( lv_this.lv_allocated_le + 1) *
-                                   sizeof ( pe_t))) == NULL) {
-      fprintf ( stderr, "malloc error in %s [line %d]\n",
-                        __FILE__, __LINE__);
-      return -LVM_ELV_STATUS_WITH_PE_MALLOC;
-   }
-   lv_this.lv_current_pe = lv_current_pe;
-   ret = lv_status_byname_internal ( vg_name, lv_name, &lv_this);
-   lv_this.lv_current_pe = lv_current_pe;
-
-#ifdef DEBUG
-   debug ( "lv_status_with_pe -- lv_this.lv_name: %s  "
-            "lv_this.lv_allocated_le: %lu  ret: %d  errno: %d\n",
-            lv_this.lv_name, lv_this.lv_allocated_le, ret, errno);
-#endif
-
-#ifdef DEBUG
-   debug ( "lv_status_with_pe -- LEAVING\n");
-#endif
-   return ret;
-}
-
-
 int lv_status_all_lv_of_vg ( char *vg_name, vg_t *vg, lv_t ***lv, int reread) {
    int l = 0;
    int nl = 0;
@@ -140,7 +130,11 @@ int lv_status_all_lv_of_vg ( char *vg_name, vg_t *vg, lv_t ***lv, int reread) {
    if ( reread == TRUE || lv_this == NULL) {
       if ( lv_this != NULL) {
          for ( l = 0; l < last_lv_max; l++)
-            if ( lv_this[l] != NULL) free ( lv_this[l]);
+            if ( lv_this[l] != NULL) {
+               if ( lv_this[l]->lv_current_pe != NULL)
+                  free ( lv_this[l]->lv_current_pe);
+               free ( lv_this[l]);
+            }
          free ( lv_this);
       }
       first = 0;
@@ -160,15 +154,18 @@ int lv_status_all_lv_of_vg ( char *vg_name, vg_t *vg, lv_t ***lv, int reread) {
             if ( ( lv_this[l] = malloc ( sizeof ( lv_t))) == NULL) {
                fprintf ( stderr, "malloc error in %s [line %d]\n",
                                  __FILE__, __LINE__);
-               for ( l = 0; l < vg->lv_max; l++) 
-                  if ( lv_this[l] != 0) free ( lv_this[l]);
+               for ( l = 0; l < vg->lv_max; l++) {
+                  if ( lv_this[l]->lv_current_pe != NULL)
+                     free ( lv_this[l]->lv_current_pe);
+                  if ( lv_this[l] != NULL) free ( lv_this[l]);
+               }
                free ( lv_this);
                lv_this = NULL;
                return -LVM_ELV_STATUS_ALL_LV_OF_VG_MALLOC;
             }
             memcpy ( lv_this[l], lv_tmp, sizeof ( lv_t));
             nl++;
-         }
+         } else vg->lv[l] = NULL;
       }
       first = 1;
    }
@@ -200,7 +197,7 @@ int lv_status_byname_internal ( char *vg_name, char *lv_name, lv_t *lv) {
         vg_check_name ( vg_name) < 0 ||
         lv_check_name ( lv_name) < 0) return -LVM_EPARAM;
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    strcpy ( req.lv_name, lv_name);
    req.lv = lv;
    if ( ( group = open ( group_file, O_RDONLY)) == -1)
@@ -234,7 +231,7 @@ int lv_status_byindex_internal ( char *vg_name, ulong lv_index, lv_t *lv) {
         lv      == NULL ||
         vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    req.lv_index = lv_index;
    req.lv = lv;
    if ( ( group = open ( group_file, O_RDONLY)) == -1)
