@@ -1,30 +1,42 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lv_write.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * August 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ * 08/30/1998 - seperated disk and core lv structures in lv_write()
+ *              and lv_write_all_lv_of_vg() by using lv_copy_to_disk()
+ *
+ */
+
 #include <liblvm.h>
 
 int lv_write ( char *pv_name, vg_t *vg, lv_t *lv, int l) {
    int pvol = -1;
    int ret  = 0;
+   lv_disk_t *lv_disk = NULL;
 
 #ifdef DEBUG
    debug ( "lv_write -- CALLED with pv_name: %s  vg->vg_name: %s  "
@@ -36,7 +48,10 @@ int lv_write ( char *pv_name, vg_t *vg, lv_t *lv, int l) {
         pv_check_name ( pv_name) < 0 ||
         vg == NULL ||
         lv == NULL ||
-        vg->lv_max == 0) return -LVM_EPARAM;
+        vg->lv_max == 0 ||
+        lv_check_consistency ( lv) < 0) return -LVM_EPARAM;
+
+   lv_disk = lv_copy_to_disk ( lv);
 
    if ( ( pvol = open ( pv_name, O_WRONLY)) == -1)
       ret = -LVM_ELV_STORE_OPEN;
@@ -45,13 +60,14 @@ int lv_write ( char *pv_name, vg_t *vg, lv_t *lv, int l) {
                         vg->pv[pv_get_index_by_name ( vg, pv_name)], l),
                      SEEK_SET) != LVM_LV_DISK_OFFSET ( vg->pv[0], l))
       ret = -LVM_ELV_STORE_LSEEK;
-   else if ( write ( pvol, lv, sizeof ( lv_t)) != sizeof ( lv_t))
+   else if ( write ( pvol, lv_disk, sizeof ( lv_disk_t)) != sizeof ( lv_disk_t))
       ret = -LVM_ELV_STORE_WRITE;
 
    if ( pvol != -1) {
       fsync ( pvol);
       close ( pvol);
    }
+   free ( lv_disk);
 
 #ifdef DEBUG
    debug ( "lv_write -- LEAVING with ret: %d\n", ret);
@@ -71,7 +87,7 @@ int lv_write_all_pv ( vg_t *vg, int l) {
    if ( vg == NULL) return -LVM_EPARAM;
 
    for ( p = 0; p < vg->pv_cur; p++)
-      if ( ( ret = lv_write ( vg->pv[p]->pv_name, vg, vg->lv[l], l)) != 0)
+      if ( ( ret = lv_write ( vg->pv[p]->pv_name, vg, vg->lv[l], l)) < 0)
          break;
 
 #ifdef DEBUG
@@ -86,7 +102,8 @@ int lv_write_all_lv_of_vg ( char *pv_name, vg_t *vg) {
    int l = 0;
    int pvol = -1;
    uint size = 0;
-   lv_t *lv_this;
+   lv_disk_t *lv_this = NULL;
+   lv_disk_t *lv_disk_tmp = NULL;
 
 #ifdef DEBUG
    debug ( "lv_write_all_lv_of_vg -- CALLED  pv_name: %s\n", pv_name);
@@ -99,7 +116,7 @@ int lv_write_all_lv_of_vg ( char *pv_name, vg_t *vg) {
         vg->lv_max == 0 ||
         vg->lv_max > MAX_LV) return -LVM_EPARAM;
 
-   size = vg->lv_max * sizeof ( lv_t);
+   size = vg->lv_max * sizeof ( lv_disk_t);
    if ( ( lv_this = malloc ( size)) == NULL) {
       fprintf ( stderr, "malloc error in %s [line %d]\n",
                         __FILE__, __LINE__);
@@ -111,12 +128,15 @@ int lv_write_all_lv_of_vg ( char *pv_name, vg_t *vg) {
          debug ( "lv_write_all_lv_of_vg -- copying vg->lv[%d] \"%s\"\n",
                   l, vg->lv[l]->lv_name);
 #endif
-         memcpy ( &lv_this[l], vg->lv[l], sizeof ( lv_t));
+         memcpy ( &lv_this[l],
+                  ( lv_disk_tmp = lv_copy_to_disk ( vg->lv[l])),
+                  sizeof ( lv_disk_t));
+         free ( lv_disk_tmp);
       } else {
 #ifdef DEBUG
-         debug ( "lv_write_all_lv_of_vg -- copying EMPTY LV\n");
+         debug ( "lv_write_all_lv_of_vg -- copying EMPTY LV #%d\n", l);
 #endif
-         memset ( &lv_this[l], 0, sizeof ( lv_t));
+         memset ( &lv_this[l], 0, sizeof ( lv_disk_t));
       }
    }
 
