@@ -1,20 +1,22 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lvm_check_number.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -36,7 +38,17 @@ int lvm_check_number ( char *number_str) {
 
    ret = TRUE;
    while ( *number_str != 0) {
-      if ( ! isdigit ( *number_str)) {
+      if ( number_str[1] == 0 &&
+           ( *number_str == 'k' ||
+             *number_str == 'k' ||
+             *number_str == 'm' ||
+             *number_str == 'M' ||
+             *number_str == 'g' ||
+             *number_str == 'G' ||
+             *number_str == 't' ||
+             *number_str == 'T')) {
+         break;
+      } else if ( ! isdigit ( *number_str)) {
          ret = FALSE;
          break;
       }
