@@ -1,29 +1,40 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/lib/lvm_dir_cache.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * May 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *   01/24/1999 - corrected to new devfs name space
+ *
+ */
+
 #include <liblvm.h>
 
 char *dirname = NULL;
-int lvm_dir_cache_select ( const struct dirent*);
+int lvm_dir_cache_select ( struct dirent*);
+int lvm_dir_cache_hit ( dir_cache_t*, int, dev_t);
 
 
 int lvm_dir_cache ( dir_cache_t **dir_cache_ptr) {
@@ -33,7 +44,9 @@ int lvm_dir_cache ( dir_cache_t **dir_cache_ptr) {
    int n = 0;
    char name[NAME_LEN] = { 0, };
    static char *devdir[] = {
-      "/dev/dsk",
+      "/dev/ide/hd",
+      "/dev/sd",
+      "/dev/md",
       "/dev",
       NULL
    };
@@ -85,10 +98,13 @@ int lvm_dir_cache ( dir_cache_t **dir_cache_ptr) {
                   return 0;
                }
          
-               strcpy ( dir_cache[cache_size].dev_name, name);
-               dir_cache[cache_size].st_rdev = stat_b.st_rdev;
-               dir_cache[cache_size].st_mode = stat_b.st_mode;
-               cache_size++;
+               if ( lvm_dir_cache_hit ( dir_cache, cache_size,
+                                        stat_b.st_rdev) == FALSE) {
+                  strcpy ( dir_cache[cache_size].dev_name, name);
+                  dir_cache[cache_size].st_rdev = stat_b.st_rdev;
+                  dir_cache[cache_size].st_mode = stat_b.st_mode;
+                  cache_size++;
+               }
             }
             for ( n = 0; n < dirent_count; n++) free ( dirent[n]);
             free ( dirent);
@@ -110,13 +126,35 @@ int lvm_dir_cache ( dir_cache_t **dir_cache_ptr) {
 
 /* internal selection function for scandir() */
 /* gets all c*, sd*, hd* and md* with name adn st_rdev into cache */
-int lvm_dir_cache_select ( const struct dirent *dirent) {
-   if ( ( strcmp ( dirname, "/dev/dsk") == 0 &&
-          dirent->d_name[0] == 'c') ||     /* new /dev/dsk/c#t#d#s# special */
-        ( ( dirent->d_name[0] == 's' ||    /* old /dev/sd[a-z]* special */
-            dirent->d_name[0] == 'h' ||    /* old /dev/hd[a-z]* special */
-            dirent->d_name[0] == 'm') &&   /* old /dev/md[a-z]* special */
-            dirent->d_name[1] == 'd')) {   /* the 'd' of the 3 above */
+int lvm_dir_cache_select ( struct dirent *dirent) {
+   /* devfs scsiand ide /dev/{sd, ide/hd}/c#t#d#s# specials */
+   if ( ( strcmp ( dirname, "/dev/sd") == 0 ||
+          strcmp ( dirname, "/dev/ide/hd") == 0) &&
+          dirent->d_name[0] == 'c' &&
+          isalnum ( dirent->d_name[1]))
       return 1;
-   } else return 0;
+
+   /* devfs multiple devicees */
+   if ( strcmp ( dirname, "/dev/md") == 0 &&
+        isalnum ( dirent->d_name[0]))
+      return 1;
+
+   if ( strcmp ( dirname, "/dev") == 0 &&
+        strlen ( dirent->d_name) > 2 &&
+        ( dirent->d_name[0] == 's' ||    /* old /dev/sd[a-z]* special */
+          dirent->d_name[0] == 'h' ||    /* old /dev/hd[a-z]* special */
+          dirent->d_name[0] == 'm') &&   /* old /dev/md[a-z]* special */
+          dirent->d_name[1] == 'd')      /* the 'd' of the 3 above */
+      return 1;
+
+   return 0;
+}
+
+
+
+int lvm_dir_cache_hit ( dir_cache_t* dir_cache, int cache_size, dev_t rdev) {
+   int d = 0;
+
+   for ( ; d < cache_size; d++) if ( dir_cache[d].st_rdev == rdev) return TRUE;
+   return FALSE;
 }
