@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/lvm_tab.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * Oktober-November 1997
  * May,June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -24,8 +26,8 @@
 /*
  * structure of the lvmtab file (all names have NAME_LEN)
  *
- *  name of first VG\n
- *  name of second VG\n
+ *  name of first VG\0
+ *  name of second VG\0
  *     ....
  *
  */
@@ -349,13 +351,44 @@ int lvm_tab_vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
 }
 
 
+int lvm_tab_lv_read_by_name ( char *vg_name, char *lv_name, lv_t **lv) {
+   int ret = 0;
+   vg_t *vg = NULL;
+
+#ifdef DEBUG
+   debug ( "lvm_tab_lv_read_by_name -- CALLED lv_name: %s\n", lv_name);
+#endif
+
+   if ( vg_name == NULL || lv_name == NULL || lv == NULL ||
+        vg_check_name ( vg_name) < 0 ||
+        lv_check_name ( lv_name) < 0) return -LVM_EPARAM;
+
+   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) == 0) {
+      ret = lv_get_index_by_name ( vg, lv_name);
+      if ( ret > -1) {
+         *lv = vg->lv[ret];
+         ret = 0;
+      } else {
+         *lv = NULL;
+         ret = -LVM_ELVM_TAB_LV_READ_BY_NAME_LV_GET_INDEX_BY_NAME;
+      }
+   } else ret = -LVM_ELVM_TAB_LV_READ_BY_NAME_LVM_TAB_VG_READ_WITH_PV_AND_LV;
+
+#ifdef DEBUG
+   debug ( "lvm_tab_lv_read_by_name -- LEAVING with ret: %d\n", ret);
+#endif
+
+   return ret;
+}
+
+
 int lvm_tab_lv_check_exist ( char *lv_name) {
    int l = 0;
    int ret = 0;
    vg_t *vg = NULL;
 
 #ifdef DEBUG
-   debug ( "lvm_tab_lv_check_exit -- CALLED\n");
+   debug ( "lvm_tab_lv_check_exist -- CALLED with \"%s\"\n", lv_name);
 #endif
 
    if ( lv_name == NULL || lv_check_name ( lv_name) < 0) return -LVM_EPARAM;
@@ -372,7 +405,7 @@ int lvm_tab_lv_check_exist ( char *lv_name) {
    }
 
 #ifdef DEBUG
-   debug ( "lvm_tab_lv_check_exit -- LEAVING with ret: %d\n", ret);
+   debug ( "lvm_tab_lv_check_exist -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
