@@ -1,20 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pe_lock.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * June 1997
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -25,6 +28,7 @@
  *
  *    10/18/1997 - fixed open of group file (always opened LVM_DEV before)
  *    29/04/1998 - changed to new lv_create_kdev_t()
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -58,7 +62,7 @@ int pe_lock_internal ( char *vg_name, kdev_t dev, ulong pe,
       if ( dev == 0 || pe == 0 || lv_num == 0) return -LVM_EPARAM;
    }
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    if ( ( group = open ( group_file, O_RDONLY)) == -1) return -LVM_EPE_LOCK;
 
    pe_lock_req.lock = lock;
