@@ -1,20 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_change.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -26,6 +29,7 @@
  *    04/28/1998 - used lv_number_from_name_in_vg instead of
  *                 lv_number_from_name to avoid getting integer
  *                 from name string
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -47,7 +51,7 @@ int pv_change ( char *vg_name, pv_t *pv) {
         vg_check_name ( vg_name) < 0 ||
         pv_check_name ( pv->pv_name) < 0) return -LVM_EPARAM;
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    strcpy ( req.pv_name, pv->pv_name);
    req.pv = pv;
    pv->pv_status = PV_ACTIVE;
