@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_check_consistency.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * May 1998
+ * May,September,December 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -26,6 +28,8 @@
  *
  *    05/20/1998 - included check of LVM_ID
  *    05/26/1998 - used lvm_check_dev()
+ *    09/06/1998 - added PE consistency check in pv_check_consistency_all_pv()
+ *    12/22/1998 - enhanced PE consistency check in           "
  *
  */
 
@@ -86,6 +90,8 @@ int pv_check_consistency ( pv_t *pv) {
 
 int pv_check_consistency_all_pv ( vg_t *vg) {
    int p = 0;
+   int pe = 0;
+   int pe_count = 0;
    int ret = 0;
 
 #ifdef DEBUG
@@ -102,6 +108,19 @@ int pv_check_consistency_all_pv ( vg_t *vg) {
 #endif
             return ret;
          }
+         
+         for ( pe = pe_count = 0; pe < vg->pv[p]->pe_total; pe++) {
+            if ( vg->pv[p]->pe[pe].lv_num > 0) pe_count++;
+            if ( vg->pv[p]->pe[pe].lv_num > vg->lv_max ||
+                 vg->pv[p]->pe[pe].le_num > vg->pe_total)
+               return -LVM_EPV_CHECK_CONSISTENCY_ALL_PV_PE;
+         }
+
+         /* vg->pv[p]->pe_allocated != 0 because vgmerge has to set it to 0
+            befaure doing lv_create() for the merged LVs */
+         if ( pe_count != vg->pv[p]->pe_allocated &&
+              vg->pv[p]->pe_allocated != 0)
+            return -LVM_EPV_CHECK_CONSISTENCY_ALL_PV_PE;
       }
    }
 
