@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_get_size.c
+ *
+ * Copyright (C) 1997 - 1998  Heinz Mauelshagen, Germany
  *
  * March,October 1997
  * May 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -27,6 +30,7 @@
  *    10/07/1997 - added extended partiton sizing
  *    11/07/1997 - corrected deadlock looping through partition tables
  *    05/04/1998 - added support for non partitioned disks
+ *    01/23/1999 - added devfs support
  *
  */
 
@@ -45,7 +49,7 @@ int pv_get_size ( char *dev_name, struct partition *part_ptr) {
    loff_t extended_offset = 0;
    struct partition *part = ( struct partition *) ( buffer + 0x1be);
    unsigned short *s_buffer = ( unsigned short *) buffer;
-   char disk_dev_name[NAME_LEN];
+   char disk_dev_name[NAME_LEN] = { 0, };
 
 #ifdef DEBUG
    debug ( "pv_get_size -- CALLED with %s and %X\n",
@@ -55,8 +59,14 @@ int pv_get_size ( char *dev_name, struct partition *part_ptr) {
    if ( dev_name == NULL ||
         pv_check_name ( dev_name) < 0) return -LVM_EPARAM;
 
-   strcpy ( disk_dev_name, dev_name);
-   disk_dev_name[DISK_NAME_LEN] = 0;
+   memset ( disk_dev_name, 0, sizeof ( disk_dev_name));
+
+   strncpy ( disk_dev_name, dev_name, sizeof ( disk_dev_name) - 1);
+   for ( i = strlen ( disk_dev_name); i > 0; i--)
+      if ( disk_dev_name[i] == 'p') break;
+   if ( i > 0) disk_dev_name[i] = 0;
+   else        disk_dev_name[DISK_NAME_LEN] = 0;
+
    memset ( buffer, 0, SECTOR_SIZE);
 
    part_i = pv_check_part ( dev_name);
@@ -75,12 +85,13 @@ int pv_get_size ( char *dev_name, struct partition *part_ptr) {
          break;
       }
 
-      if ( read ( pv_handle, buffer, SECTOR_SIZE) != SECTOR_SIZE) {
+      if ( ( ret = read ( pv_handle, buffer, SECTOR_SIZE)) != SECTOR_SIZE) {
          ret = -LVM_EPV_GET_SIZE_READ;
          break;
-      }
+      } else ret = 0;
 
       /* no partition table present --> use whole disk */
+
       if ( s_buffer[255] != 0xAA55) {
          if ( ioctl ( pv_handle, BLKGETSIZE, &ret) == -1) {
             ret = -LVM_EPV_GET_SIZE_IOCTL;
