@@ -1,20 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/lib/pv_move.c
  *
- * May 1998
+ * Copyright (C) 1997, 1998, 1999  Heinz Mauelshagen, Germany
+ *
+ * May  1998
+ * July 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -27,6 +31,11 @@
  *    05/14/1998 - implemented handling of striped logical volumes
  *    05/16/1998 - implemented handling of contiguous (optional striped)
  *                 logical volumes
+ *    07/05/1998 - added move of list of les in pv_move_pes()
+ *               - allow source PV == destination PV in pv_move_pe()
+ *    07/06/1998 - added move of lists of physical extents from one source
+ *                 to one destination physical volume
+ *    01/20/1999 - used LVM_PE_DISK_OFFSET macro in pv_move_pe()
  *
  */
 
@@ -53,25 +62,34 @@ void pv_move_interrupt ( int);
 static int pv_move_int = FALSE;
 static lv_t *lv_this = NULL;
 
-/* perform the move of all physical extends / those of a logical volume */
+/* perform the move of all physical extents / those of a logical volume */
 int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
                   int src_pv_index, int lv_index,
+                  int *les, int *source_pes, int *dest_pes,
                   int opt_v, int opt_t) {
-   int i = 0;
-   int p = 0;
-   int pa = 0;
    int add_it = FALSE;
+   int dest_pe_count = 0;
+   int dst_pv_index = 0;
+   int source_pe_count = 0;
+   int ext = 0;
+   int i = 0;
+   int l = 0;
+   int pa = 0;
    int ret = 0;
+   long le = 0;
    long pe = 0;
    long pe_for_dest = 0;
+   long pe_last = 0;
    long pe_to_move = 0;
    long pe_sav = 0;
    long pe_start = 0;
    ushort p_sav = 0;
    ushort le_num_sav = 0;
+   ushort lv_num_sav = 0;
    struct pe_struct {
       ushort p;
       long pe;
+      ushort lv_num;
       ushort le_num;
    } *pe_src = NULL, *pe_dest = NULL, *pe_src_sav, *pe_dest_sav;
 
@@ -84,26 +102,88 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
         buffer == NULL || pv_allowed == NULL ||
         src_pv_index < 0 || src_pv_index >= vg->pv_cur ||
         lv_index < -1 || lv_index >= ( int) vg->lv_max ||
-        opt_v < 0 || opt_t < 0) return -LVM_EPARAM;
+        opt_v < 0 || opt_t < 0) {
+      ret = -LVM_EPARAM;
+      goto pv_move_pes_end;
+   }
+
+   /* avoid LE list for striped LV */
+   if ( lv_index > -1 && lv_index < ( int) vg->lv_max &&
+        les != NULL && vg->lv[lv_index]->lv_stripes > 1) {
+      ret = -LVM_EPARAM;
+      goto pv_move_pes_end;
+   }
+
+   if ( source_pes != NULL) {
+      if ( les != NULL) {
+         ret = -LVM_EPARAM;
+         goto pv_move_pes_end;
+      }
+      source_pe_count = 0;
+      for ( ext = 0; source_pes[ext] != -1; ext++) {
+         if ( source_pes[ext] < 0 ||
+              source_pes[ext] >= vg->pv[src_pv_index]->pe_total)
+            return -LVM_EPARAM;
+         else source_pe_count++;
+      }
+   }
+
+   if ( dest_pes != NULL) {
+      dst_pv_index = pv_get_index_by_name ( vg, pv_allowed[0]);
+      dest_pe_count = 0;
+      for ( ext = 0; dest_pes[ext] != -1; ext++) {
+         if ( dest_pes[ext] < 0 ||
+              dest_pes[ext] >= vg->pv[dst_pv_index]->pe_total) {
+            return -LVM_EPARAM;
+         } else dest_pe_count++;
+      }
+   }
+
+   if ( source_pes != NULL && dest_pes != NULL &&
+        source_pe_count != dest_pe_count) return -LVM_EPARAM;
+   /* end of parameter check */
 
    pv_move_int = FALSE;
 
-   /* space check */
-   if ( opt_v > 0) printf ( "%s -- checking for enough free physical "
-                            "extends in %s\n", cmd, vg->vg_name);
+
    /* get all source PEs */
+   if ( opt_v > 0) printf ( "%s -- checking for enough free physical "
+                            "extents in %s\n", cmd, vg->vg_name);
    pe_to_move = 0;
    for ( pe = 0; pe < vg->pv[src_pv_index]->pe_total; pe++) {
       add_it = FALSE;
 
-      /* have to move a command line given logical volume */
+      /* have to move a given logical volume */
       if ( lv_index != -1) {
-         if ( vg->pv[src_pv_index]->pe[pe].lv_num == lv_index + 1)
-            add_it = TRUE;
+         if ( vg->pv[src_pv_index]->pe[pe].lv_num == lv_index + 1) {
+            /* have to move command line given list of logical extents */
+            if ( les != NULL) {
+               for ( le = 0; les[le] != -1; le++)
+                  if ( les[le] == vg->pv[src_pv_index]->pe[pe].le_num) {
+                     add_it = TRUE;
+                     break;
+                  }
+            } else add_it = TRUE;
+         }
+      /* have to move given physical extents */
+      } else if ( source_pes != NULL) {
+         for ( ext = 0; source_pes[ext] != -1; ext++) {
+            if ( pe == source_pes[ext]) {
+               if ( ( vg->lv[vg->pv[src_pv_index]->pe[pe].lv_num-1]\
+                      ->lv_allocation & LV_CONTIGUOUS) ||
+                    vg->lv[vg->pv[src_pv_index]->pe[pe].lv_num-1]\
+                    ->lv_stripes > 1) {
+                  ret = -LVM_EPV_MOVE_PES_ALLOC_STRIPES;
+                  goto pv_move_pes_end;
+               }
+               add_it = TRUE;
+               break;
+            }
+         }
+      /* have to move all logical extents */
       } else if ( vg->pv[src_pv_index]->pe[pe].lv_num != 0) add_it = TRUE;
 
       if ( add_it == TRUE) {
-         lv_index = vg->pv[src_pv_index]->pe[pe].lv_num - 1;
          pe_src_sav = pe_src;
          if ( ( pe_src = realloc ( pe_src, ( pe_to_move + 1) *
                                    sizeof ( struct pe_struct))) == NULL) {
@@ -113,9 +193,11 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
             ret = -LVM_EPV_MOVE_PES_REALLOC;
             goto pv_move_pes_end;
          }
+
          pe_src[pe_to_move].p  = src_pv_index;
          pe_src[pe_to_move].pe = pe;
          pe_src[pe_to_move].le_num = vg->pv[src_pv_index]->pe[pe].le_num;
+         pe_src[pe_to_move].lv_num = vg->pv[src_pv_index]->pe[pe].lv_num;
          pe_to_move++;
       }
    }
@@ -126,9 +208,9 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
    }
 
    /* sort source PEs in ascending LE order */
-   for ( i = 0; i < pe_to_move; i++) {
-      for ( pe = 0; pe < pe_to_move-1; pe++) {
-         if ( pe_src[pe].le_num > pe_src[pe+1].le_num) {
+   for ( l = 0; l < vg->pv[src_pv_index]->lv_cur; l++) {
+      for ( pe = pe_last ; pe < pe_to_move - 1; pe++) {
+         if ( pe_src[pe].lv_num > pe_src[pe+1].lv_num) {
             p_sav = pe_src[pe].p;
             pe_src[pe].p = pe_src[pe+1].p;
             pe_src[pe+1].p = p_sav;
@@ -137,6 +219,10 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
             pe_src[pe].pe = pe_src[pe+1].pe;
             pe_src[pe+1].pe = pe_sav;
 
+            lv_num_sav = pe_src[pe].lv_num;
+            pe_src[pe].lv_num = pe_src[pe+1].lv_num;
+            pe_src[pe+1].lv_num = lv_num_sav;
+
             le_num_sav = pe_src[pe].le_num;
             pe_src[pe].le_num = pe_src[pe+1].le_num;
             pe_src[pe+1].le_num = le_num_sav;
@@ -144,33 +230,82 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
       }
    }
 
-   /* save for access in interrupt routine */
-   lv_this = vg->lv[lv_index];
+   pe_last = 0;
+   for ( i = 0; i < pe_to_move; i++) {
+      for ( pe = pe_last ; pe < pe_to_move - 1; pe++) {
+         if ( pe_src[pe].lv_num == pe_src[pe+1].lv_num) {
+            if ( pe_src[pe].le_num > pe_src[pe+1].le_num) {
+               p_sav = pe_src[pe].p;
+               pe_src[pe].p = pe_src[pe+1].p;
+               pe_src[pe+1].p = p_sav;
    
+               pe_sav = pe_src[pe].pe;
+               pe_src[pe].pe = pe_src[pe+1].pe;
+               pe_src[pe+1].pe = pe_sav;
+   
+               lv_num_sav = pe_src[pe].lv_num;
+               pe_src[pe].lv_num = pe_src[pe+1].lv_num;
+               pe_src[pe+1].lv_num = lv_num_sav;
+   
+               le_num_sav = pe_src[pe].le_num;
+               pe_src[pe].le_num = pe_src[pe+1].le_num;
+               pe_src[pe+1].le_num = le_num_sav;
+            }
+         } else {
+            pe_last = pe;
+            break;
+         }
+      }
+   }
+   /* END sort source PEs in ascending LE order */
+
    /* get all destination PEs */
    pe_for_dest = 0;
    for ( pa = 0; pv_allowed[pa] != NULL && pe_for_dest < pe_to_move; pa++) {
-      /* PV full */
-      p = pv_get_index_by_name ( vg, pv_allowed[pa]);
-
-      /* striped LV on this PV */
-      if ( lv_check_on_pv ( vg->pv[p], lv_index + 1) == TRUE &&
-           vg->lv[lv_index]->lv_stripes > 1) continue;
+      dst_pv_index = pv_get_index_by_name ( vg, pv_allowed[pa]);
 
       /* physical volume is full */
-      if ( vg->pv[p]->pe_allocated == vg->pv[p]->pe_total) continue;
+      if ( vg->pv[dst_pv_index]->pe_allocated ==
+           vg->pv[dst_pv_index]->pe_total) continue;
 
       /* physical volume is not allocatable */
-      if ( ! ( vg->pv[p]->pv_allocatable & PV_ALLOCATABLE)) continue;
+      if ( ! ( vg->pv[dst_pv_index]->pv_allocatable & PV_ALLOCATABLE)) continue;
+
+      /* given logical volume */
+      if ( lv_index > -1) {
+         /* striped LV on this PV ? */
+         if ( lv_check_on_pv ( vg->pv[dst_pv_index], lv_index + 1) == TRUE &&
+              vg->lv[lv_index]->lv_stripes > 1) continue;
+   
+         pe_start = 0;
+         /* Enough contiguous free available in case of contiguous LV? */
+         if ( ( vg->lv[lv_index]->lv_allocation & LV_CONTIGUOUS) &&
+              pv_check_free_contiguous ( vg->pv[dst_pv_index],
+                                         pe_to_move, &pe_start) ==
+                                         FALSE) continue;
+      }
+
+      for ( pe = 0; pe < pe_to_move; pe++) {
+         if ( lv_check_on_pv ( vg->pv[dst_pv_index],
+                               pe_src[pe].lv_num - 1) == TRUE &&
+              vg->lv[pe_src[pe].lv_num-1]->lv_stripes > 1) break;
+         if ( ( vg->lv[pe_src[pe].lv_num-1]->lv_allocation & LV_CONTIGUOUS) &&
+              pv_check_free_contiguous ( vg->pv[dst_pv_index],
+                                         lv_count_pe ( vg->pv[src_pv_index],
+                                         pe_src[pe].lv_num), &pe_start) ==
+                                         FALSE) break;
+      }
+      if ( pe < pe_to_move) continue;
+
+      for ( pe = pe_start; pe < vg->pv[dst_pv_index]->pe_total; pe++) {
+         if ( vg->pv[dst_pv_index]->pe[pe].lv_num != 0) continue;
+         if ( dest_pes != NULL) {
+            for ( ext = 0; dest_pes[ext] != -1; ext++) {
+               if ( pe == dest_pes[ext]) break;
+            }
+            if ( dest_pes[ext] == -1) continue;
+         }
 
-      pe_start = 0;
-      /* Enough contiguous free available in case of contiguous LV? */
-      if ( ( vg->lv[lv_index]->lv_allocation & LV_CONTIGUOUS) &&
-           pv_check_free_contiguous ( vg->pv[p],
-                                      pe_to_move, &pe_start) == FALSE) continue;
-      
-      for ( pe = pe_start; pe < vg->pv[p]->pe_total; pe++) {
-         if ( vg->pv[p]->pe[pe].lv_num != 0) continue;
          pe_dest_sav = pe_dest;
          if ( ( pe_dest = realloc ( pe_dest, ( pe_for_dest + 1) *
                                     sizeof ( struct pe_struct))) == NULL) {
@@ -180,7 +315,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
             ret = -LVM_EPV_MOVE_PES_REALLOC;
             goto pv_move_pes_end;
          }
-         pe_dest[pe_for_dest].p = p;
+         pe_dest[pe_for_dest].p = dst_pv_index;
          pe_dest[pe_for_dest].pe = pe;
          pe_for_dest++;
          if ( pe_for_dest == pe_to_move) break;
@@ -205,6 +340,8 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
 
    /* Move PE by PE */
    for ( pe = 0; pv_move_int == FALSE && pe < pe_to_move; pe++) {
+      lv_index = pe_src[pe].lv_num - 1;
+      lv_this = vg->lv[lv_index];
       if ( ( ret = pv_move_pe ( vg, buffer,
                                 pe_src[pe].p,  pe_dest[pe].p,
                                 pe_src[pe].pe, pe_dest[pe].pe,
@@ -245,7 +382,6 @@ int pv_move_pe ( vg_t *vg, char *buffer,
 
    if ( src_pv_index < 0 || src_pv_index >= vg->pv_cur ||
         dst_pv_index < 0 || dst_pv_index >= vg->pv_cur ||
-        src_pv_index == dst_pv_index ||
         pe_source < 0 || pe_source >= vg->pv[src_pv_index]->pe_total ||
         pe_dest   < 0 || pe_dest   >= vg->pv[dst_pv_index]->pe_total ||
         opt_v < 0 || opt_t < 0 || act_pe < 0 || off_pe < 0 ||
@@ -288,10 +424,8 @@ int pv_move_pe ( vg_t *vg, char *buffer,
    strcpy ( le_remap_req.lv_name, lv_name_this);
    le_remap_req.old_dev = vg->pv[src_pv_index]->pv_dev;
    le_remap_req.new_dev = vg->pv[dst_pv_index]->pv_dev;
-   le_remap_req.old_pe  = ( LVM_DISK_SIZE ( vg->pv[src_pv_index]) /
-                          SECTOR_SIZE) + pe_source * vg->pe_size;
-   le_remap_req.new_pe  = ( LVM_DISK_SIZE ( vg->pv[dst_pv_index]) /
-                          SECTOR_SIZE) + pe_dest * vg->pe_size;
+   le_remap_req.old_pe  = LVM_PE_DISK_OFFSET( pe_source, vg->pv[src_pv_index]);
+   le_remap_req.new_pe  = LVM_PE_DISK_OFFSET( pe_dest, vg->pv[dst_pv_index]);
 
 
    if ( opt_v > 1) {
@@ -380,6 +514,8 @@ int pv_move_pe ( vg_t *vg, char *buffer,
       }
    }
 
+   /* Future HM extension: write time stamp for pe_lock o.k. */
+
    if ( opt_v > 1) printf ( "%s -- reading input physical volume %s\n",
                             cmd, vg->pv[src_pv_index]->pv_name);
    size = vg->pe_size * SECTOR_SIZE;
@@ -405,9 +541,18 @@ int pv_move_pe ( vg_t *vg, char *buffer,
          ret = -LVM_EPE_MOVE_PE_WRITE_OUT;
          goto pv_move_pe_end;
       }
-      fsync ( out);
+      if ( fsync ( out) < 0) {
+         fprintf ( stderr, "%s -- ERROR syncing output to "
+                           "physical volume %s\n\n",
+                   cmd, vg->pv[dst_pv_index]->pv_name);
+         pe_unlock ( vg->vg_name);
+         ret = -LVM_EPE_MOVE_PE_WRITE_OUT;
+         goto pv_move_pe_end;
+      }
    }
 
+   /* Future HM extension: write time stamp for lv_le_remap */
+
    if ( opt_v > 1) printf ( "%s -- remapping physical extend "
                             "in VGDA of kernel\n", cmd);
    if ( opt_t == 0) {
