@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_read.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March-May,October-November 1997
- * May 1998
+ * May,August,November 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -38,40 +40,51 @@
  *                 for future use
  *    05/16/1998 - canceled wrong physical volume device names by
  *                 inserting the correct ones at load time
- *               - changed algorythm in pv_read_all_pv() to avoid
+ *               - changed algorithm in pv_read_all_pv() to avoid
  *                 multiple MD entries according
  *    05/22/1998 - enhanced disk loop once more (see lvm_dir_cache())
+ *    08/19/1998 - seperated disk and core structures in pv_read()
+ *                 by using pv_copy_from_disk()
+ *    09/06/1998 - implemented data layer in pv_read_pe() by using
+ *                 new function pe_copy_from_disk()
+ *    11/15/1998 - avoided pv_read'ing all partition special files, if
+ *                 disk doesn't exist
+ *    01/28/1999 - avoided calling pv_flush each time pv_read was called
+ *                 by pv_read_already_red()
  *
  */
 
 #include <liblvm.h>
 
-char *pv_read_all_pv_dirname = NULL;
 
-int pv_read ( char *dev_name, pv_t **pv, int *open_errno) {
+/* internal function */
+int pv_read_already_red ( char*);
+
+
+int pv_read ( char *pv_name, pv_t **pv, int *open_errno) {
    int pv_handle = -1;
    int ret = 0;
-   static pv_t pv_this;
+   static pv_disk_t pv_this;
    struct stat stat_b;
 
 #ifdef DEBUG
-   debug ( "pv_read -- CALLED with %s\n", dev_name);
+   debug ( "pv_read -- CALLED with %s\n", pv_name);
 #endif
 
-   if ( dev_name == NULL ||
+   if ( pv_name == NULL ||
         pv == NULL ||
-        pv_check_name ( dev_name) < 0) return -LVM_EPARAM;
+        pv_check_name ( pv_name) < 0) return -LVM_EPARAM;
 
-   /* enhance me: flush PV in each and every case ??? */
-   if ( pv_flush ( dev_name) < 0) return -LVM_EPV_READ_PV_FLUSH;
+   if ( pv_read_already_red ( pv_name) == FALSE && pv_flush ( pv_name) < 0)
+      return -LVM_EPV_READ_PV_FLUSH;
 
-   if ( ( pv_handle = open ( dev_name, O_RDONLY)) != -1) {
+   if ( ( pv_handle = open ( pv_name, O_RDONLY)) != -1) {
       if ( fstat ( pv_handle, &stat_b) == 0) {
 #ifdef DEBUG
-         debug ( "pv_read -- going to read %s\n", dev_name);
+         debug ( "pv_read -- going to read %s\n", pv_name);
 #endif
          if ( read ( pv_handle, &pv_this,
-                     sizeof ( pv_t)) != sizeof (pv_t))
+                     sizeof ( pv_this)) != sizeof (pv_this))
             ret = -LVM_EPV_READ_READ;
          else if ( stat_b.st_rdev == 0)
             ret = -LVM_EPV_READ_RDEV;
@@ -85,21 +98,21 @@ int pv_read ( char *dev_name, pv_t **pv, int *open_errno) {
 
    *pv = NULL;
    if ( ret == 0) {
-      *pv = &pv_this;
+      *pv = pv_copy_from_disk ( &pv_this);
       /* correct for imported/moved volumes */
-      memset ( pv_this.pv_name, 0, sizeof ( pv_this.pv_name));
-      strncpy ( pv_this.pv_name,
+      memset ( (*pv)->pv_name, 0, sizeof ( (*pv)->pv_name));
+      strncpy ( (*pv)->pv_name,
                 pv_create_name_from_kdev_t ( stat_b.st_rdev),
-                sizeof ( pv_this.pv_name) - 1);
-      if ( strncmp ( pv_this.id, LVM_ID, sizeof ( pv_this.id)) != 0)
+                sizeof ( (*pv)->pv_name) - 1);
+      if ( strncmp ( (*pv)->id, LVM_ID, sizeof ( (*pv)->id)) != 0)
          ret = -LVM_EPV_READ_ID_INVALID;
-      else if ( pv_this.version != LVM_STRUCT_VERSION)
+      else if ( (*pv)->version != LVM_STRUCT_VERSION)
          ret = -LVM_EPV_READ_LVM_STRUCT_VERSION;
-      else if ( system_id_check_exported ( pv_this.system_id) == TRUE)
+      else if ( system_id_check_exported ( (*pv)->system_id) == TRUE)
          ret = -LVM_EPV_READ_PV_EXPORTED;
-      else if ( MAJOR ( pv_this.pv_dev) == MD_MAJOR)
+      else if ( (*pv)->pv_dev == MD_MAJOR)
          ret = -LVM_EPV_READ_MD_DEVICE;
-      pv_this.pv_dev = stat_b.st_rdev;
+      (*pv)->pv_dev = stat_b.st_rdev;
    }
 
    if ( pv_handle != -1) close ( pv_handle);
@@ -120,6 +133,7 @@ int pv_read_all_pv ( pv_t ***pv, int reread) {
    int p_sav2 = 0;
    int pv_read_errno = 0;
    int ret = 0;
+   int tst = 0;
    char *dev_name = NULL;
    static int first = 0;
    struct partition partition;
@@ -159,10 +173,19 @@ int pv_read_all_pv ( pv_t ***pv, int reread) {
       np = 0;
       for ( n = 0; n < cache_size; n++) {
          dev_name = dir_cache[n].dev_name;
+
 #ifdef DEBUG
          debug ( "pv_read_all_pv -- calling pv_read with \"%s\"\n",
                   dev_name);
 #endif
+         if ( ( tst = open ( dev_name, O_RDONLY)) == -1) {
+            if ( MAJOR ( dir_cache[n].st_rdev) != MD_MAJOR &&
+                 MINOR ( dir_cache[n].st_rdev) % 16 == 0) {
+               n += 15;
+               continue;
+            }
+         } else close ( tst);
+
          pv_read_errno = 0;
          if ( ( ret = pv_read ( dev_name, &pv_tmp, &pv_read_errno)) == 0 ||
               ret == -LVM_EPV_READ_MD_DEVICE ||
@@ -206,7 +229,7 @@ int pv_read_all_pv ( pv_t ***pv, int reread) {
                debug ( "pv_read_all_pv -- device %s NOT used\n", dev_name);
             }
 #endif
-         }
+         } 
 #ifdef DEBUG
          else debug ( "pv_read_all_pv -- pv_read returned: %d\n", ret);
 #endif
@@ -373,24 +396,24 @@ int pv_read_pe ( pv_t *pv, disk_pe_t **pe) {
       ret = -LVM_EPV_READ_PE_OPEN;
    else if ( lseek ( pv_handle, pv->pe_on_disk.base, SEEK_SET) !=
              pv->pe_on_disk.base) ret = -LVM_EPV_READ_PE_LSEEK;
-   else if ( ( pe_this = malloc ( size + sizeof ( disk_pe_t))) == NULL) {
+   else if ( ( pe_this = malloc ( size)) == NULL) {
       fprintf ( stderr, "malloc error in %s [line %d]\n",
                         __FILE__, __LINE__);
       ret = -LVM_EPV_READ_PE_MALLOC;
    } else {
-      memset ( pe_this, 0, size + sizeof ( disk_pe_t));
+      memset ( pe_this, 0, size);
       if ( read ( pv_handle, pe_this, size) != size)
          ret = -LVM_EPV_READ_PE_READ;
+      else {
+         *pe = pe_copy_from_disk ( pe_this, pv->pe_total);
+      }
    }
 #ifdef DEBUG
    debug ( "pv_read_pe -- ret: %d\n", ret);
 #endif
 
-   if ( ret == 0) pe_this[pv->pe_total].lv_num = \
-                  pe_this[pv->pe_total].le_num = 0;
-
    if ( pv_handle != -1) close ( pv_handle);
-   *pe = pe_this;
+   if ( pe_this != NULL) free ( pe_this);
 
 #ifdef DEBUG
    debug ( "pv_read_pe -- LEAVING with ret: %d\n", ret);
@@ -545,3 +568,32 @@ int pv_read_namelist ( char *pv_name, vg_t *vg, char **pv_namelist) {
 #endif
    return ret;
 }
+
+
+int pv_read_already_red ( char *pv_name) {
+   int i = 0;
+   static int pv_count = 0;
+   static char* pv_name_cache = NULL;
+   char* pv_name_cache_sav = NULL;
+
+
+   if ( pv_name == 0 || strlen ( pv_name) > NAME_LEN - 1) return FALSE;
+
+   for ( i = 0; i < pv_count; i++)
+      if ( strcmp ( pv_name, &pv_name_cache[i*NAME_LEN]) == 0) return TRUE;
+
+   pv_name_cache_sav = pv_name_cache;
+   if ( ( pv_name_cache = realloc ( pv_name_cache,
+                                    ( pv_count + 1) * NAME_LEN)) == NULL) {
+      fprintf ( stderr, "realloc error in %s [line %d]\n",
+                        __FILE__, __LINE__);
+      if ( pv_name_cache_sav != NULL) free ( pv_name_cache_sav);
+      return FALSE;
+   }
+
+   memset ( &pv_name_cache[pv_count * NAME_LEN], 0, NAME_LEN);
+   strcpy ( &pv_name_cache[pv_count * NAME_LEN], pv_name);
+   pv_count++;
+
+   return FALSE;
+}
