@@ -1,25 +1,37 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_reserve_release_pe.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * July  1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *    07/07/1998 - added read/write statistic init
+ *    01/21/1999 - fixed contiguous logical volume extension bug
+ *
+ */
+
 #include <liblvm.h>
 
 int pv_reserve_pe ( pv_t *pv, disk_pe_t *lv_pe, uint *pe,
@@ -42,27 +54,22 @@ int pv_reserve_pe ( pv_t *pv, disk_pe_t *lv_pe, uint *pe,
         ( allocation != 0 && allocation != LV_CONTIGUOUS) ||
         ( new != TRUE && new != FALSE)) return -LVM_EPARAM;
 
-   p = np = 0;
-   if ( ( allocation & LV_CONTIGUOUS) && new != TRUE) {
-      p = pv->pe_total - 1;
-      while ( pv->pe[p].lv_num != lv_pe->lv_num && p > -1) p--;
-      if ( p > 0) p++;
-      else return -LVM_EPV_RESERVE_PE_LV; /* this LV not on this PV */
-   }
-
    for ( p = 0; p < pv->pe_total && np < *pe; p++) {
       if ( pv->pe[p].lv_num == 0) {
 #ifdef DEBUG
          debug ( "pv_reserve_pe -- empty PE %d\n", p);
 #endif
-         if ( ( allocation & LV_CONTIGUOUS) && new != TRUE &&
-                pv->pe[p-1].lv_num != lv_pe->lv_num) return -LVM_ESIZE;
+         if ( lv_check_on_pv ( pv, lv_pe->lv_num) == TRUE) {
+            if ( new == FALSE && ( allocation & LV_CONTIGUOUS) &&
+                 pv->pe[p-1].lv_num != lv_pe->lv_num) return -LVM_ESIZE;
+         }
          pv->pe[p].lv_num = lv_pe->lv_num;
          pv->pe[p].le_num = lv_pe->le_num;
          lv_pe->le_num++;
          pv->pe_allocated++;
          pe_p->dev = pv->pv_dev;
-         pe_p->pe = ( p * pv->pe_size) + LVM_DISK_SIZE ( pv) / SECTOR_SIZE;
+         pe_p->pe = LVM_PE_DISK_OFFSET( p, pv);
+         pe_p->reads = pe_p->writes = 0;
          pe_p++;
          np++;
       }
