@@ -1,29 +1,42 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_setup.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * Januaray 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *   01/13/1999 - added memset pv_name and vg_name
+ *              - added zeroing of rest of pv_name
+ *
+ */
+
 #include <liblvm.h>
 
 
 int pv_setup_for_create ( char *dev_name, pv_t *pv, uint size) {
+   int i = 0;
+
 #ifdef DEBUG
    debug ( "pv_setup_for_create -- CALLED\n");
 #endif
@@ -45,8 +58,10 @@ int pv_setup_for_create ( char *dev_name, pv_t *pv, uint size) {
    pv->lv_on_disk.size = 0;
    pv->pe_on_disk.base = 0;
    pv->pe_on_disk.size = 0;
-   strcpy ( pv->pv_name, dev_name);
-   pv->vg_name[0] = 0;
+   strncpy ( pv->pv_name, dev_name, sizeof ( pv->pv_name) - 1);
+   for ( i =  strlen ( pv->pv_name) + 1; i < sizeof ( pv->pv_name); i++)
+      pv->pv_name[i] = 0;
+   memset ( pv->vg_name, 0, sizeof ( pv->vg_name));
    system_id_set ( pv->system_id);
    pv->pv_dev = pv_create_kdev_t ( dev_name);
    pv->pv_number = 0;
