@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_show.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March,October 1997
  * May 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -26,9 +29,11 @@
  *
  *    05/01/1998 - used lv_read_byindex() in  pv_show_pe_text() to avoid
  *                 string generation of LV names
- *    05/09/1998 - corrected display of 1 free physical extend
+ *    05/09/1998 - corrected display of 1 free physical extent
  *                 in pv_show_pe_text()
  *    05/16/1998 - fixed display bug in pv_show_pe_text()
+ *    01/20/1999 - used LVM_PE_DISK_OFFSET macro in pv_show_pe_text()
+ *               - added sector offset on disk to pv_show_pe_text()
  *
  */
 
@@ -179,18 +184,22 @@ int pv_show_pe_text ( pv_t *pv, disk_pe_t *pe, int pe_count) {
 #endif
 
    if ( pv == NULL || pe == NULL || pe_count == 0 ||
-        pv_check_consistency ( pv) < 0) return -LVM_EPARAM;
+        pv_check_consistency ( pv) < 0) {
+      ret = -LVM_EPARAM;
+      goto pv_show_pe_text_end;
+   }
 
    if ( ( pe_this = malloc ( pe_count * sizeof ( disk_pe_t))) == NULL) {
       fprintf ( stderr, "malloc error in %s [line %d]\n",
                         __FILE__, __LINE__);
-      return -LVM_EPV_SHOW_PE_TEXT_MALLOC;
+      ret = -LVM_EPV_SHOW_PE_TEXT_MALLOC;
+      goto pv_show_pe_text_end;
    }
    if ( ( pe_this_count = malloc ( pe_count * sizeof ( int))) == NULL) {
       fprintf ( stderr, "malloc error in %s [line %d]\n",
                         __FILE__, __LINE__);
-      free ( pe_this);
-      return -LVM_EPV_SHOW_PE_TEXT_MALLOC;
+      ret = -LVM_EPV_SHOW_PE_TEXT_MALLOC;
+      goto pv_show_pe_text_end;
    }
    memset ( pe_this, 0, pe_count * sizeof ( disk_pe_t));
    memset ( pe_this_count, 0, pe_count * sizeof ( int));
@@ -218,7 +227,7 @@ int pv_show_pe_text ( pv_t *pv, disk_pe_t *pe, int pe_count) {
 
    if ( lvm_tab_vg_read_with_pv_and_lv ( pv->vg_name, &vg) < 0){
       ret = -LVM_EPV_SHOW_PE_TEXT_VG_READ_WITH_PV_AND_LV;
-      goto pv_show_pe_text;
+      goto pv_show_pe_text_end;
    }
 
    printf ( "   --- Distribution of physical volume ---\n"
@@ -237,10 +246,8 @@ int pv_show_pe_text ( pv_t *pv, disk_pe_t *pe, int pe_count) {
                                      lv_num_last * NAME_LEN)) == NULL) {
             fprintf ( stderr, "realloc error in %s [line %d]\n",
                               __FILE__, __LINE__);
-            free ( pe_this);
-            free ( pe_this_count);
-            free ( lv_names_sav);
-            return -LVM_EPV_SHOW_PE_TEXT_REALLOC;
+            ret = -LVM_EPV_SHOW_PE_TEXT_REALLOC;
+            goto pv_show_pe_text_end;
          }
       }
       strcpy ( &lv_names[(pe_this[pt].lv_num-1)*NAME_LEN],
@@ -248,7 +255,7 @@ int pv_show_pe_text ( pv_t *pv, disk_pe_t *pe, int pe_count) {
    }
 
    printf ( "\n   --- Physical extents ---\n"
-            "   PE    LV                        LE\n");
+            "   PE    LV                        LE      Disk offset\n");
    pe_free = -1;
    for ( p = 0; p < pe_count; p++) {
       if ( pe[p].lv_num != 0) {
@@ -260,21 +267,23 @@ int pv_show_pe_text ( pv_t *pv, disk_pe_t *pe, int pe_count) {
          printf ( "   %05d %-25s ", p, lv_name_this);
          if ( strlen ( lv_name_this) > 25)
             printf ( "\n                                  ");
-         printf ( "%05d\n", pe[p].le_num);
+         printf ( "%05d   %d\n", pe[p].le_num, LVM_PE_DISK_OFFSET( p, pv));
+
       } else if ( pe_free == -1) pe_free = p;
    }
 
    if ( pe_free > 0) pv_show_free ( pe_free, p);
 
-pv_show_pe_text:
-   free ( lv_names);
-   free ( pe_this);
-   free ( pe_this_count);
+pv_show_pe_text_end:
+   if ( lv_names != NULL)      free ( lv_names);
+   if ( lv_names_sav != NULL)  free ( lv_names_sav);
+   if ( pe_this != NULL)       free ( pe_this);
+   if ( pe_this_count != NULL) free ( pe_this_count);
 
 #ifdef DEBUG
-   debug ( "pv_show_text -- LEAVING\n");
+   debug ( "pv_show_text -- LEAVING with ret: %d\n", ret);
 #endif
-   return 0;
+   return ret;
 }
 
 
