@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_status.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March,November 1997
  * May 1998
+ * January
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -27,7 +30,9 @@
  *    11/18/1997 - enhanced disk loop (see disk_type)
  *    05/09/1998 - enxtended pv_status_all_pv_of_vg () to search /dev/dsk too
  *                 for future use
- *    05/03/1998 - enhanced disk loop once more (see lvm_dir_cache())
+ *    05/03/1998 - enhanced disk loop once more (see lvm_dir_cache()) and
+ *                 avoided /dev/dsk in here
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -52,7 +57,7 @@ int pv_status ( char *vg_name, char *pv_name, pv_t **pv) {
 
    *pv = NULL;
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    strcpy ( req.pv_name, pv_name);
    req.pv = &pv_this;
    if ( ( group = open ( group_file, O_RDONLY)) == -1)
