@@ -1,30 +1,44 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/pv_write.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * August-September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *    08/30/1998 - seperated disk and core pv structure in pv_write()
+ *                 by using pv_copy_to_disk()
+ *    09/06/1998 - implemented data layer in pv_write_pe() by using
+ *                 new function pe_copy_to_disk()
+ *
+ */
+
 #include <liblvm.h>
 
 int pv_write ( char *pv_name, pv_t *pv) {
    int pv_handle = -1;
-   int ret  = 0;
+   int ret = 0;
+   pv_disk_t *pv_disk = NULL;
 
 #ifdef DEBUG
    debug ( "pv_write -- CALLED with %s %X\n", pv_name, (uint) pv);
@@ -32,15 +46,21 @@ int pv_write ( char *pv_name, pv_t *pv) {
 
    if ( pv_name == NULL ||
         pv_check_name ( pv_name) < 0 ||
-         pv == NULL) return -LVM_EPARAM;
+        pv == NULL ||
+        pv_check_consistency ( pv) < 0) return -LVM_EPARAM;
+
+   pv_disk = pv_copy_to_disk ( pv);
 
    if ( ( pv_handle = open ( pv_name, O_WRONLY)) == -1)
       ret = -LVM_EPV_STORE_OPEN;
    else if ( lseek ( pv_handle, pv->pv_on_disk.base, SEEK_SET) !=
              pv->pv_on_disk.base) ret = -LVM_EPV_STORE_LSEEK;
-   else if ( write ( pv_handle, pv, sizeof ( pv_t)) != sizeof ( pv_t))
+   else if ( write ( pv_handle, pv_disk,
+                     sizeof ( pv_disk_t)) != sizeof ( pv_disk_t))
       ret = -LVM_EPV_STORE_WRITE;
 
+   free ( pv_disk);
+
    if ( pv_handle != -1) {
       fsync ( pv_handle);
       close ( pv_handle);
@@ -57,6 +77,7 @@ int pv_write_namelist ( char *pv_name, vg_t *vg) {
    int p = 0;
    int pv_handle = -1;
    int ret = 0;
+   int size = 0;
    char *pv_name_list = NULL;
 
 #ifdef DEBUG
@@ -72,18 +93,19 @@ int pv_write_namelist ( char *pv_name, vg_t *vg) {
                      SEEK_SET) != vg->pv[0]->pv_namelist_on_disk.base)
       ret = -LVM_EPV_STORE_NAMELIST_LSEEK;
    else {
-      if ( ( pv_name_list = malloc ( vg->pv_cur * NAME_LEN)) == NULL) {
+      size = vg->pv_cur * NAME_LEN;
+      if ( ( pv_name_list = malloc ( size)) == NULL) {
          fprintf ( stderr, "malloc error in %s [line %d]\n",
                            __FILE__, __LINE__);
          ret = LVM_EPV_STORE_NAMELIST_MALLOC;
       } else {
-         memset ( pv_name_list, 0, vg->pv_cur * NAME_LEN);
-         for ( p = 0; p < vg->pv_cur; p++)
-            strcpy ( &pv_name_list[p*NAME_LEN], vg->pv[p]->pv_name);
-         if ( write ( pv_handle, pv_name_list, vg->pv_cur * NAME_LEN) !=
-              vg->pv_cur * NAME_LEN) {
-            ret = -LVM_EPV_STORE_NAMELIST_WRITE;
+         memset ( pv_name_list, 0, size);
+         for ( p = 0; p < vg->pv_cur; p++) {
+            if ( vg->pv[p] != NULL)
+               strcpy ( &pv_name_list[p*NAME_LEN], vg->pv[p]->pv_name);
          }
+         if ( write ( pv_handle, pv_name_list, size) != size)
+            ret = -LVM_EPV_STORE_NAMELIST_WRITE;
          free ( pv_name_list);
       }
    }
@@ -126,6 +148,7 @@ int pv_write_pe ( char *pv_name, pv_t *pv) {
    int pv_handle = -1;
    int ret  = 0;
    uint size = 0;
+   disk_pe_t *pe_disk = NULL;
 
 #ifdef DEBUG
    debug ( "pv_write_pe -- CALLED  pv->pe_total: %lu\n", pv->pe_total);
@@ -143,8 +166,12 @@ int pv_write_pe ( char *pv_name, pv_t *pv) {
    else if ( lseek ( pv_handle,  pv->pe_on_disk.base, SEEK_SET) !=
               pv->pe_on_disk.base)
       ret = -LVM_EPV_STORE_PE_LSEEK;
-   else if ( write ( pv_handle, pv->pe, size) != size)
-      ret = -LVM_EPV_STORE_PE_WRITE;
+   else {
+      pe_disk = pe_copy_to_disk ( pv->pe, pv->pe_total);
+      if ( write ( pv_handle, pe_disk, size) != size)
+         ret = -LVM_EPV_STORE_PE_WRITE;
+      free ( pe_disk);
+   }
 
    if ( pv_handle != -1) {
       fsync ( pv_handle);
@@ -170,8 +197,7 @@ int pv_write_with_pe ( char *pv_name, pv_t *pv) {
         pv_check_name ( pv_name) < 0 ||
         pv == NULL) return -LVM_EPARAM;
 
-   ret = pv_write ( pv_name, pv);
-   if ( ret == 0) ret = pv_write_pe ( pv_name, pv);
+   if ( ( ret = pv_write ( pv_name, pv)) == 0) ret = pv_write_pe ( pv_name, pv);
 
 #ifdef DEBUG
    debug ( "pv_write_with_pe -- LEAVING\n");
