@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/lib/vg_cfgbackup.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * September 1997
- * May 1998
+ * May,July-August 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -28,16 +30,23 @@
  *                 instead of directory
  *               - enhanced parameter checking
  *               - avoided creating backup if new one is the same
+ *    07/05/1998 - implemented deeper VGDA backup history
+ *    07/07/1998 - zero I/Os and open data to avoid backups
+ *    07/10/1998 - enhanced ERROR output with < pv_cur PVs
+ *    08/30/1998 - seperated disk and core structures by using
+ *                 lv_copy_to_disk(), pv_copy_to_disk() and vg_copy_to_disk()
  *
  */
 
 #include <liblvm.h>
+#define VG_MAX_BACKUPS	9
 
 int vg_cfgbackup ( char* vg_name, char* directory,
                    int opt_v, vg_t *vg_in) {
    int file_differ = FALSE;
    int i = 0;
    int l = 0;
+   int le = 0;
    int p = 0;
    int ret = 0;
    int sz = 0;
@@ -47,7 +56,8 @@ int vg_cfgbackup ( char* vg_name, char* directory,
    char *vg_backup_tmp_buffer = NULL;
    char vg_backup_path[NAME_LEN] = { 0, };
    char vg_backup_path_tmp[NAME_LEN] = { 0, };
-   char vg_backup_path_old[NAME_LEN] = { 0, };
+   char vg_backup_path_old1[NAME_LEN] = { 0, };
+   char vg_backup_path_old2[NAME_LEN] = { 0, };
    struct stat stat_b1;
    struct stat stat_b2;
    lv_t lv_zero;
@@ -61,63 +71,68 @@ int vg_cfgbackup ( char* vg_name, char* directory,
         opt_v < 0 || vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
 
    memset ( &lv_zero, 0, sizeof ( lv_zero));
-   if ( opt_v > 0) printf ( "%s -- checking volume group name %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
                             cmd, vg_name);
    if ( vg_check_name ( vg_name) < 0) {
-      fprintf ( stderr, "%s -- invalid volume group name %s\n",
+      fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n",
                 cmd, vg_name);
       return -LVM_EPARAM;
    }
 
    if ( stat ( directory, &stat_b1) == 0) {
       if ( ! S_ISDIR ( stat_b1.st_mode)) {
-         fprintf ( stderr, "%s -- ERROR: file %s exists; must be directory\n",
-                   cmd, directory);
+         fprintf ( stderr, "%s -- ERROR: file \"%s\" exists; "
+                           "must be directory\n",
+                           cmd, directory);
          ret = -LVM_EVG_CFGBACKUP_FILE_EXISTS;
          goto vg_cfgbackup_end;
       }
    }
 
    if ( vg_in == NULL) {
-      if ( opt_v > 0) printf ( "%s -- checking existence of volume group %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking existence of volume "
+                               "group \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = vg_check_exist ( vg_name)) < 0) {
          if ( ret == -LVM_EPV_READ_PV_EXPORTED)
             fprintf ( stderr, "%s -- INFO: can't access exported "
-                              "volume group %s\n",
+                              "volume group \"%s\"\n",
                               cmd, vg_name);
          else if ( ret == -LVM_EPV_READ_ALL_PV_OF_VG_NP)
-            fprintf ( stderr, "%s -- volume group %s doesn't exist\n",
+            fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
                               cmd, vg_name);
          else if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION)
-            fprintf ( stderr, "%s -- ERROR: volume group %s has physical "
+            fprintf ( stderr, "%s -- ERROR: volume group \"%s\" has physical "
                               "volumes with invalid version\n",
                               cmd, vg_name);
-         else if ( ret == -LVM_EPV_READ_ALL_PV_OF_VG_PV_NUMBER)
-            fprintf ( stderr, "%s -- ERROR: volume group %s has an "
+         else if ( ret == -LVM_EPV_READ_ALL_PV_OF_VG_PV_NUMBER ||
+                   ret == -LVM_EVG_CHECK_EXIST_PV_COUNT)
+            fprintf ( stderr, "%s -- ERROR: volume group \"%s\" has an "
                               "invalid number of physical volumes\n",
                               cmd, vg_name);
          else
-            fprintf ( stderr, "%s -- ERROR %d checking existence of %s\n",
-                      cmd, ret, vg_name);
+            fprintf ( stderr, "%s -- ERROR %d checking existence of "
+                              "colume group \"%s\"\n",
+                              cmd, ret, vg_name);
          ret = -LVM_EVG_CFGBACKUP_VG_CHECK_EXIST;
          goto vg_cfgbackup_end;
       }
    
-      if ( opt_v > 0) printf ( "%s -- reading volume group data for %s "
+      if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
                                "from disk(s)\n", cmd, vg_name);
       if ( ( ret = vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
          if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION) {
-            fprintf ( stderr, "%s -- volume group %s can't be used because:\n",
-                      cmd, vg_name);
+            fprintf ( stderr, "%s -- volume group \"%s\" can't be "
+                              "used because:\n",
+                              cmd, vg_name);
             for ( p = 0; p < vg->pv_cur; p++) {
                if ( vg->pv[p]->version != LVM_STRUCT_VERSION)
-                  fprintf ( stderr, "   physical volume %s has invalid "
+                  fprintf ( stderr, "   physical volume \"%s\" has invalid "
                                     "version %d\n",
                                     vg->pv[p]->pv_name, vg->pv[p]->version);
             }
          } else {
-            fprintf ( stderr, "%s -- ERROR %d: reading data of %s\n",
+            fprintf ( stderr, "%s -- ERROR %d: reading VGDA of \"%s\"\n",
                       cmd, ret, vg_name);
          }
          ret = -LVM_EVG_CFGBACKUP_VG_READ_WITH_PV_AND_LV;
@@ -126,48 +141,63 @@ int vg_cfgbackup ( char* vg_name, char* directory,
    } else vg = vg_in;
 
    if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
-                            "of %s\n", cmd, vg_name);
+                            "of \"%s\"\n", cmd, vg_name);
    if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: %s is inconsistent\n",
+      fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" is inconsistent\n",
                 cmd, ret, vg_name);
       ret = -LVM_EVG_CHECK_CONSISTENCY;
       goto vg_cfgbackup_end;
    }
 
+   /* zero I/Os and open data to avoid difference in compare */
+   for ( l = 0; l < vg->lv_max; l++) {
+      if ( vg->lv[l] != NULL) {
+         vg->lv[l]->lv_open = 0;
+         for ( le = 0; le < vg->lv[l]->lv_current_le; le++)
+         vg->lv[l]->lv_current_pe[le].reads = \
+         vg->lv[l]->lv_current_pe[le].writes = 0;
+      }
+   }
+
    sz = sprintf ( vg_backup_path, "%s/%s", directory, vg_name);
    if ( strcmp ( directory, VG_BACKUP_DIR) == 0)
       sz += sprintf ( &vg_backup_path[sz], ".conf");
    vg_backup_path[sz] = 0;
-   sprintf ( vg_backup_path_old, "%s.old%c", vg_backup_path, 0);
    sprintf ( vg_backup_path_tmp, "%s.tmp%c", vg_backup_path, 0);
 
-   if ( opt_v > 0) printf ( "%s -- checking existence of %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking existence of \"%s\"\n",
                             cmd, directory);
    if ( stat ( directory, &stat_b1) == -1) {
-      if ( opt_v > 0) printf ( "%s -- making directory %s\n",
+      if ( opt_v > 0) printf ( "%s -- making directory \"%s\"\n",
                                cmd, directory);
       if ( mkdir ( directory, 0755) == -1)
-         fprintf ( stderr, "%s -- ERROR %d making directory %s\n",
+         fprintf ( stderr, "%s -- ERROR %d making directory \"%s\"\n",
                            cmd, errno, directory);
    }
 
    if ( ( vg_backup = open ( vg_backup_path_tmp,
                              O_CREAT | O_WRONLY | O_TRUNC, 0640)) == -1) {
-      fprintf ( stderr, "%s -- ERROR %d opening volume group backup file "
-                        "%s for writing\n",
-                        cmd, errno, vg_backup_path_tmp);
+      if ( errno == EROFS)
+         fprintf ( stderr, "%s -- read only filesystem opening volume group "
+                           "backup file \"%s\" for writing\n",
+                           cmd, vg_backup_path_tmp);
+      else
+         fprintf ( stderr, "%s -- ERROR %d opening volume group backup file "
+                           "\"%s\" for writing\n",
+                           cmd, errno, vg_backup_path_tmp);
       ret = -LVM_EVG_CFGBACKUP_OPEN;
       goto vg_cfgbackup_end;
    }
 
    fchmod ( vg_backup, 0640);
 
-   if ( opt_v > 0) printf ( "%s -- storing volume group data of %s in %s\n",
+   if ( opt_v > 0) printf ( "%s -- storing volume group data of \"%s\" "
+                            "in \"%s\"\n",
                             cmd, vg_name, vg_backup_path_tmp);
    VGCFG_WRITE ( vg_backup, vg, sizeof ( vg_t), vg_backup_path_tmp);
 
-   if ( opt_v > 0) printf ( "%s -- storing physical volume data of %s "
-                            "in %s\n",
+   if ( opt_v > 0) printf ( "%s -- storing physical volume data of \"%s\" "
+                            "in \"%s\"\n",
                             cmd, vg_name, vg_backup_path_tmp);
    for ( p = 0; p < vg->pv_cur; p++) {
       VGCFG_WRITE ( vg_backup, vg->pv[p], sizeof ( pv_t), vg_backup_path_tmp);
@@ -176,8 +206,8 @@ int vg_cfgbackup ( char* vg_name, char* directory,
                     vg_backup_path_tmp);
    }
 
-   if ( opt_v > 0) printf ( "%s -- storing logical volume data of %s "
-                            "in %s\n",
+   if ( opt_v > 0) printf ( "%s -- storing logical volume data of "
+                            "volume group \"%s\" in \"%s\"\n",
                             cmd, vg_name, vg_backup_path_tmp);
    for ( l = 0; l < vg->lv_max; l++) {
       if ( vg->lv[l] != NULL) {
@@ -186,7 +216,7 @@ int vg_cfgbackup ( char* vg_name, char* directory,
          VGCFG_WRITE ( vg_backup, vg->lv[l]->lv_current_pe,
                        vg->lv[l]->lv_allocated_le * sizeof ( pe_t),
                        vg_backup_path_tmp);
-      } else VGCFG_WRITE ( vg_backup, &lv_zero, sizeof ( lv_t),
+      } else VGCFG_WRITE ( vg_backup, &lv_zero, sizeof ( lv_zero),
                            vg_backup_path_tmp);
    }
    if ( vg_backup != -1) {
@@ -197,20 +227,20 @@ int vg_cfgbackup ( char* vg_name, char* directory,
 
    /* volume group backup */
    if ( strcmp ( directory, VG_BACKUP_DIR) == 0) {
-      if ( opt_v > 0) printf ( "%s -- checking existence of %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking existence of \"%s\"\n",
                                cmd, vg_backup_path_tmp);
       if ( stat ( vg_backup_path_tmp, &stat_b1) < 0) {
          ret = -LVM_EVG_CFGBACKUP_TMP_FILE;
          goto vg_cfgbackup_end;
       }
 
-      if ( opt_v > 0) printf ( "%s -- checking existence of %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking existence of \"%s\"\n",
                                cmd, vg_backup_path);
       if ( stat ( vg_backup_path, &stat_b2) == -1) {
-         if ( opt_v > 0) printf ( "%s -- renaming %s to %s\n",
+         if ( opt_v > 0) printf ( "%s -- renaming \"%s\" to \"%s\"\n",
                                   cmd, vg_backup_path_tmp, vg_backup_path);
          if ( rename ( vg_backup_path_tmp, vg_backup_path) == -1) {
-            fprintf ( stderr, "%s -- ERROR %d renaming %s\n",
+            fprintf ( stderr, "%s -- ERROR %d renaming \"%s\"\n",
                               cmd, errno, vg_backup_path);
             ret = -LVM_EVG_CFGBACKUP_RENAME;
             goto vg_cfgbackup_end;
@@ -222,39 +252,39 @@ int vg_cfgbackup ( char* vg_name, char* directory,
       /* compare both files if they are of equal size */
       if ( stat_b1.st_size == stat_b2.st_size) {
          if ( ( vg_backup_tmp = open ( vg_backup_path_tmp, O_RDONLY)) == -1) {
-            fprintf ( stderr, "%s -- ERROR opening temporary file %s\n",
+            fprintf ( stderr, "%s -- ERROR opening temporary file \"%s\"\n",
                               cmd, vg_backup_path_tmp);
             ret = -LVM_EVG_CFGBACKUP_OPEN;
             goto vg_cfgbackup_end;
          }
          if ( ( vg_backup = open ( vg_backup_path, O_RDONLY)) == -1) {
-            fprintf ( stderr, "%s -- ERROR opening file %s\n",
+            fprintf ( stderr, "%s -- ERROR opening file \"%s\"\n",
                               cmd, vg_backup_path);
             ret = -LVM_EVG_CFGBACKUP_OPEN;
             goto vg_cfgbackup_end;
          }
          if ( ( vg_backup_tmp_buffer = malloc ( stat_b1.st_size)) == NULL) {
-            fprintf ( stderr, "%s -- malloc error in %s [line %d]\n\n",
+            fprintf ( stderr, "%s -- malloc error in \"%s\" [line %d]\n\n",
                               cmd, __FILE__, __LINE__);
             ret = -LVM_EVG_CFGBACKUP_MALLOC;
             goto vg_cfgbackup_end;
          }
          if ( ( vg_backup_buffer = malloc ( stat_b2.st_size)) == NULL) {
-            fprintf ( stderr, "%s -- malloc error in %s [line %d]\n\n",
+            fprintf ( stderr, "%s -- malloc error in \"%s\" [line %d]\n\n",
                               cmd, __FILE__, __LINE__);
             ret = -LVM_EVG_CFGBACKUP_MALLOC;
             goto vg_cfgbackup_end;
          }
          if ( read ( vg_backup_tmp, vg_backup_tmp_buffer, stat_b1.st_size) != 
               stat_b1.st_size) {
-            fprintf ( stderr, "%s -- ERROR reading %s\n",
+            fprintf ( stderr, "%s -- ERROR reading \"%s\"\n",
                               cmd, vg_backup_path_tmp);
             ret = -LVM_EVG_CFGBACKUP_READ;
             goto vg_cfgbackup_end;
          }
          if ( read ( vg_backup, vg_backup_buffer, stat_b2.st_size) != 
               stat_b2.st_size) {
-            fprintf ( stderr, "%s -- ERROR reading %s\n",
+            fprintf ( stderr, "%s -- ERROR reading \"%s\"\n",
                               cmd, vg_backup_path);
             ret = -LVM_EVG_CFGBACKUP_READ;
             goto vg_cfgbackup_end;
@@ -267,21 +297,15 @@ int vg_cfgbackup ( char* vg_name, char* directory,
                break;
             }
          }
-         if ( vg_backup_tmp != -1) {
-            close ( vg_backup_tmp);
-            vg_backup_tmp = -1;
-         }
-         if ( vg_backup != -1) {
-            close ( vg_backup);
-            vg_backup = -1;
-         }
+
+         close ( vg_backup); vg_backup = -1;
 
          if ( file_differ == FALSE) {
-            printf ( "%s -- no difference to old backup in %s\n",
+            printf ( "%s -- no difference to old backup in \"%s\"\n",
                      cmd, vg_backup_path);
             ret = LVM_VG_CFGBACKUP_NO_DIFF;
             if ( unlink ( vg_backup_path_tmp) == -1) {
-               fprintf ( stderr, "%s -- ERROR %d unlinking %s\n",
+               fprintf ( stderr, "%s -- ERROR %d unlinking \"%s\"\n",
                                  cmd, errno, vg_backup_path_tmp);
                ret = -LVM_EVG_CFGBACKUP_UNLINK;
             }
@@ -289,32 +313,64 @@ int vg_cfgbackup ( char* vg_name, char* directory,
          }
       }
 
-      if ( opt_v > 0) printf ( "%s -- checking for %s\n",
-                               cmd, vg_backup_path_old);
-      if ( ( vg_backup = open ( vg_backup_path_old, O_RDONLY)) != -1) {
-         if ( opt_v > 0) printf ( "%s -- removing %s\n",
-                                  cmd,  vg_backup_path_old);
-         if ( unlink ( vg_backup_path_old) == -1) {
-            fprintf ( stderr, "%s -- ERROR %d unlinking %s\n",
-                              cmd, errno, vg_backup_path_old);
-            ret = -LVM_EVG_CFGBACKUP_UNLINK;
+      /* VGDA backup history */
+      for ( i = VG_MAX_BACKUPS; i > 1; i--) {
+         sprintf ( vg_backup_path_old2, "%s/%s.conf.%d.old%c",
+                                        directory, vg_name, i, 0);
+         if ( opt_v > 0) printf ( "%s -- checking for \"%s\"\n",
+                                  cmd, vg_backup_path_old2);
+         if ( i == VG_MAX_BACKUPS &&
+              stat ( vg_backup_path_old2, &stat_b1) == 0) {
+            if ( unlink ( vg_backup_path_old2) == -1) {
+               fprintf ( stderr, "%s -- ERROR %d unlinking \"%s\"\n",
+                                 cmd, errno, vg_backup_path_old2);
+               ret = -LVM_EVG_CFGBACKUP_UNLINK;
+               goto vg_cfgbackup_end;
+            }
+         }
+
+         sprintf ( vg_backup_path_old1, "%s/%s.conf.%d.old%c",
+                                        directory, vg_name, i - 1, 0);
+         if ( opt_v > 0) printf ( "%s -- checking for \"%s\"\n",
+                                  cmd, vg_backup_path_old1);
+         if ( stat ( vg_backup_path_old1, &stat_b1) == -1) continue;
+
+
+         if ( opt_v > 0) printf ( "%s -- renaming \"%s\" to \"%s\"\n",
+                                  cmd,
+                                  vg_backup_path_old1,
+                                  vg_backup_path_old2);
+         if ( rename ( vg_backup_path_old1, vg_backup_path_old2) == -1) {
+            fprintf ( stderr, "%s -- ERROR %d renaming \"%s\" to \"%s\"\n",
+                              cmd, errno,
+                              vg_backup_path_old1,
+                              vg_backup_path_old2);
+            ret = -LVM_EVG_CFGBACKUP_RENAME;
             goto vg_cfgbackup_end;
          }
       }
-      close  ( vg_backup); vg_backup = -1;
-   
-      if ( opt_v > 0) printf ( "%s -- renaming %s to %s\n",
-                               cmd, vg_backup_path, vg_backup_path_old);
-      if ( rename ( vg_backup_path, vg_backup_path_old) == -1) {
-         fprintf ( stderr, "%s -- ERROR %d renaming %s\n",
-                           cmd, errno, vg_backup_path);
+
+      sprintf ( vg_backup_path_old2, "%s/%s.conf.1.old%c",
+                                     directory, vg_name, 0);
+      sprintf ( vg_backup_path_old1, "%s/%s.conf%c",
+                                     directory, vg_name, 0);
+      if ( opt_v > 0) printf ( "%s -- renaming \"%s\" to \"%s\"\n",
+                               cmd,
+                               vg_backup_path_old1,
+                               vg_backup_path_old2);
+      if ( rename ( vg_backup_path_old1, vg_backup_path_old2) == -1) {
+         fprintf ( stderr, "%s -- ERROR %d renaming \"%s\" to \"%s\"\n",
+                           cmd, errno,
+                           vg_backup_path_old1,
+                           vg_backup_path_old2);
          ret = -LVM_EVG_CFGBACKUP_RENAME;
          goto vg_cfgbackup_end;
       }
-      if ( opt_v > 0) printf ( "%s -- renaming %s to %s\n",
+
+      if ( opt_v > 0) printf ( "%s -- renaming \"%s\" to \"%s\"\n",
                                cmd, vg_backup_path_tmp, vg_backup_path);
       if ( rename ( vg_backup_path_tmp, vg_backup_path) == -1) {
-         fprintf ( stderr, "%s -- ERROR %d renaming %s\n",
+         fprintf ( stderr, "%s -- ERROR %d renaming \"%s\"\n",
                            cmd, errno, vg_backup_path_tmp);
          ret = -LVM_EVG_CFGBACKUP_RENAME;
          goto vg_cfgbackup_end;
@@ -322,10 +378,10 @@ int vg_cfgbackup ( char* vg_name, char* directory,
    /* lvmtab backup */
    } else {
       unlink ( vg_backup_path);
-      if ( opt_v > 0) printf ( "%s -- renaming %s to %s\n",
+      if ( opt_v > 0) printf ( "%s -- renaming \"%s\" to \"%s\"\n",
                                cmd, vg_backup_path_tmp, vg_backup_path);
       if ( rename ( vg_backup_path_tmp, vg_backup_path) == -1) {
-         fprintf ( stderr, "%s -- ERROR %d renaming %s\n",
+         fprintf ( stderr, "%s -- ERROR %d renaming \"%s\"\n",
                            cmd, errno, vg_backup_path);
          ret = -LVM_EVG_CFGBACKUP_RENAME;
       }
@@ -338,7 +394,7 @@ vg_cfgbackup_end:
    if ( vg_backup != -1) close ( vg_backup);
 
 #ifdef DEBUG
-   debug ( "vg_cfgbackup -- LEAVING\n");
+   debug ( "vg_cfgbackup -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
