@@ -1,21 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/lib/vg_cfgrestore.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * November 1997
- * May 1998
+ * May,August 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -26,6 +28,9 @@
  *
  *    05/16/1998 - added errror return code for ordinary file existing
  *                 instead of directory
+ *    08/30/1998 - seperated disk and core structures by using
+ *                 lv_copy_from_disk(), pv_copy_from_disk
+ *                 and vg_copy_from_disk()
  *
  */
 
@@ -55,23 +60,24 @@ int vg_cfgrestore ( char *vg_name, char *vg_backup_path,
 
    if ( stat ( directory, &stat_buf) == 0) {
       if ( ! S_ISDIR ( stat_buf.st_mode)) {
-         fprintf ( stderr, "%s -- ERROR: file %s exists; must be directory\n",
-                   cmd, directory);
+         fprintf ( stderr, "%s -- ERROR: file \"%s\" exists; "
+                           "must be directory\n",
+                           cmd, directory);
          ret = -LVM_EVG_CFGRESTORE_FILE_EXISTS;
          goto vg_cfgrestore_end;
       }
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking existence of %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking existence of \"%s\"\n",
                             cmd, vg_backup_path);
    if ( ( vg_backup = open ( vg_backup_path, O_RDONLY)) == -1) {
-      fprintf ( stderr, "%s -- %s doesn't exist\n", cmd, vg_backup_path);
+      fprintf ( stderr, "%s -- \"%s\" doesn't exist\n", cmd, vg_backup_path);
       ret = -LVM_EVG_CFGRESTORE_OPEN;
       goto vg_cfgrestore_end;
    }
 
-   if ( opt_v > 0) printf ( "%s -- reading volume group data for %s "
-                            "out of %s\n",
+   if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
+                            "from \"%s\"\n",
                             cmd, vg_name, vg_backup_path);
 
    VGCFG_READ ( vg_backup, vg, sizeof ( vg_t), vg_backup_path);
@@ -81,12 +87,12 @@ int vg_cfgrestore ( char *vg_name, char *vg_backup_path,
       goto vg_cfgrestore_end;
    }
 
-   if ( opt_v > 0) printf ( "%s -- reading physical volume data for %s "
-                            "from %s\n",
+   if ( opt_v > 0) printf ( "%s -- reading physical volume data for \"%s\" "
+                            "from \"%s\"\n",
                             cmd, vg_name, vg_backup_path);
    for ( p = 0; p < vg->pv_cur; p++) {
       if ( ( vg->pv[p] = malloc ( sizeof ( pv_t))) == NULL) {
-         fprintf ( stderr, "%s -- malloc error in file %s [line %d]\n",
+         fprintf ( stderr, "%s -- malloc error in file \"%s\" [line %d]\n",
                            cmd, __FILE__, __LINE__);
          ret = -LVM_EVG_CFGRESTORE_MALLOC;
          goto vg_cfgrestore_end;
@@ -98,7 +104,7 @@ int vg_cfgrestore ( char *vg_name, char *vg_backup_path,
       }
       if ( ( vg->pv[p]->pe =
                 malloc ( vg->pv[p]->pe_total * sizeof ( disk_pe_t))) == NULL) {
-         fprintf ( stderr, "%s -- malloc error in file %s [line %d]\n",
+         fprintf ( stderr, "%s -- malloc error in file \"%s\" [line %d]\n",
                            cmd, __FILE__, __LINE__);
          ret = -LVM_EVG_CFGRESTORE_MALLOC;
          goto vg_cfgrestore_end;
@@ -108,12 +114,12 @@ int vg_cfgrestore ( char *vg_name, char *vg_backup_path,
                               vg_backup_path);
    }
 
-   if ( opt_v > 0) printf ( "%s -- reading logical volume data for %s "
-                            "from %s\n",
+   if ( opt_v > 0) printf ( "%s -- reading logical volume data for \"%s\" "
+                            "from \"%s\"\n",
                             cmd, vg_name, vg_backup_path);
    for ( l = 0; l < vg->lv_max; l++) {
       if ( ( vg->lv[l] = malloc ( sizeof ( lv_t))) == NULL) {
-         fprintf ( stderr, "%s -- malloc error in file %s [line %d]\n",
+         fprintf ( stderr, "%s -- malloc error in file \"%s\" [line %d]\n",
                            cmd, __FILE__, __LINE__);
          ret = -LVM_EVG_CFGRESTORE_MALLOC;
          goto vg_cfgrestore_end;
@@ -132,7 +138,7 @@ int vg_cfgrestore ( char *vg_name, char *vg_backup_path,
          if ( ( vg->lv[l]->lv_current_pe =
                    malloc ( vg->lv[l]->lv_allocated_le * \
                             sizeof ( pe_t))) == NULL) {
-            fprintf ( stderr, "%s -- malloc error in file %s [line: %d]\n",
+            fprintf ( stderr, "%s -- malloc error in file \"%s\" [line: %d]\n",
                               cmd, __FILE__, __LINE__);
             ret = -LVM_EVG_CFGRESTORE_MALLOC;
             goto vg_cfgrestore_end;
@@ -143,6 +149,9 @@ int vg_cfgrestore ( char *vg_name, char *vg_backup_path,
       }
    }
 
+   if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
+                            "of \"%s\"\n",
+                            cmd, vg_name);
    if ( vg_check_consistency_with_pv_and_lv ( vg) < 0)
       ret = -LVM_EVG_CFGRESTORE_VG_CHECK_CONSISTENCY_WITH_PV_AND_LV;
 
@@ -150,7 +159,7 @@ vg_cfgrestore_end:
    if ( vg_backup != -1) close ( vg_backup);
 
 #ifdef DEBUG
-   debug ( "vg_cfgrestore -- LEAVING\n");
+   debug ( "vg_cfgrestore -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
