@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/vg_check_consistency.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * May 1998
+ * May,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -25,6 +28,7 @@
  * Changelog
  *
  *    05/09/1998 - added vg_status checks
+ *    01/15/1999 -           "
  *
  */
 
@@ -39,7 +43,7 @@ int vg_check_consistency ( vg_t *vg) {
 
    if ( vg == NULL) return -LVM_EPARAM;
 
-   if ( vg_check_name ( vg->vg_name) != 0)
+   if ( vg_check_name ( vg->vg_name) < 0)
       return -LVM_EVG_CHECK_CONSISTENCY_VG_NAME;
    if ( vg->vg_access != VG_READ &&
         vg->vg_access != VG_WRITE &&
@@ -48,7 +52,9 @@ int vg_check_consistency ( vg_t *vg) {
    if ( vg->vg_status != 0 &&
         vg->vg_status != VG_ACTIVE &&
         vg->vg_status != VG_EXTENDABLE &&
-        vg->vg_status != ( VG_ACTIVE | VG_EXTENDABLE))
+        vg->vg_status != VG_EXPORTED &&
+        vg->vg_status != ( VG_ACTIVE | VG_EXTENDABLE) &&
+        vg->vg_status != ( VG_EXTENDABLE | VG_EXPORTED))
       return -LVM_EVG_CHECK_CONSISTENCY_VG_STATUS;
    if ( vg->lv_cur > vg->lv_max)
       return -LVM_EVG_CHECK_CONSISTENCY_LV_CUR;
