@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/lib/vg_create_dir_and_group_and_nodes.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * September 1997
  * January 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -24,7 +27,8 @@
 /*
  * Changelog
  *
- * 01/17/1998 - fixed lv index bug
+ *    01/17/1998 - fixed lv index bug
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -50,24 +54,31 @@ int vg_create_dir_and_group_and_nodes ( vg_t *vg, int opt_v) {
       total_ret--;
       if ( ret == -LVM_EVG_CREATE_DIR_AND_GROUP_MKDIR) {
          fprintf ( stderr, "%s -- problem creating volume group "
-                           "directory /dev/%s\n",
-                   cmd, vg->vg_name);
+                           "directory "
+                           LVM_DIR_PREFIX
+                           "%s\n",
+                           cmd, vg->vg_name);
       } else if ( ret == -LVM_EVG_CREATE_DIR_AND_GROUP_CHMOD_DIR) {
          fprintf ( stderr, "%s -- problem changing permission "
-                           "for volume group directory /dev/%s\n",
-                   cmd, vg->vg_name);
+                           "for volume group directory "
+                           LVM_DIR_PREFIX
+                           "%s\n",
+                           cmd, vg->vg_name);
       } else if ( ret == -LVM_EVG_CREATE_DIR_AND_GROUP_CHMOD_GROUP) {
          fprintf ( stderr, "%s -- problem changing permission "
-                           "for volume group file /dev/%s/group\n",
-                   cmd, vg->vg_name);
+                           "for volume group file "
+                           LVM_DIR_PREFIX
+                           "%s/group\n",
+                           cmd, vg->vg_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d creating volume group "
-                           "directory /dev/%s\n",
-                   cmd, ret, vg->vg_name);
+         fprintf ( stderr, "%s -- ERROR %d creating volume group directory "
+                           LVM_DIR_PREFIX
+                           "%s\n",
+                           cmd, ret, vg->vg_name);
       }
    }
 
-   /* mknod logical volume special files */
+   /* create logical volume special files */
    if ( opt_v > 0) printf ( "%s -- creating block device special "
                             "files for %s\n",
                             cmd, vg->vg_name);
