@@ -1,36 +1,47 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/vg_extend_reduce.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * April 1997
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
+ *
+ */
+
 #include <liblvm.h>
 
+/* module internal prototype */
 int vg_extend_reduce ( char *, pv_t *, vg_t *, int);
 
 
-int vg_extend ( char *vg_name, pv_t *pv, vg_t *vg) {
+inline int vg_extend ( char *vg_name, pv_t *pv, vg_t *vg) {
    return vg_extend_reduce ( vg_name, pv, vg, VG_EXTEND);
 }
 
 
-int vg_reduce ( char *vg_name, pv_t *pv, vg_t *vg) {
+inline int vg_reduce ( char *vg_name, pv_t *pv, vg_t *vg) {
    return vg_extend_reduce ( vg_name, pv, vg, VG_REDUCE);
 }
 
@@ -51,7 +62,7 @@ int vg_extend_reduce ( char *vg_name, pv_t *pv, vg_t *vg, int cr) {
         pv_check_consistency ( pv) < 0 ||
         vg_check_consistency_with_pv_and_lv ( vg) < 0) return -LVM_EPARAM;
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    if ( ( group = open ( group_file, O_RDWR)) == -1)
       ret = -LVM_EVG_EXTEND_REDUCE_OPEN;
    else {
