@@ -1,20 +1,22 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/lib/vg_free.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -28,10 +30,9 @@ int vg_free ( vg_t *vg, int all) {
 
    if ( vg == NULL || ( all != TRUE && all != FALSE)) return -LVM_EPARAM;
 
-   for ( p = 0; p < vg->pv_max; p++) {
+   for ( p = 0; p < vg->pv_cur; p++) {
       if ( vg->pv[p] != NULL) {
-         if ( vg->pv[p]->pe != NULL)
-            free ( vg->pv[p]->pe);
+         if ( vg->pv[p]->pe != NULL) free ( vg->pv[p]->pe);
          free ( vg->pv[p]);
          vg->pv[p] = NULL;
       }
