@@ -1,21 +1,24 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/vg_read.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March,October,November 1997
- * June 1998
+ * June,August 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -28,6 +31,9 @@
  *    11/13/1997 - implemented vg_read_from_pv ()
  *    06/12/1998 - seperated memory for PVs, PEs and LVs
  *                 in vg_read_with_pv_and_lv() from pv_read_all_pv cache
+ *    08/29/1998 - seperated disk and core structures in vg_read()
+ *                 by using vg_copy_from_disk()
+ *    01/15/1999 - hand back VG pointer in vg_read() in case of exported VG
  *
  */
 
@@ -38,7 +44,7 @@ int vg_read ( char *vg_name, vg_t **vg) {
    int p = 0;
    int pv_handle = -1;
    int ret = 0;
-   static vg_t vg_this;
+   static vg_disk_t vg_this;
    pv_t **pv = NULL;
 
 #ifdef DEBUG
@@ -49,8 +55,6 @@ int vg_read ( char *vg_name, vg_t **vg) {
         vg_check_name ( vg_name) < 0 ||
         vg == NULL) return -LVM_EPARAM;
 
-   *vg = NULL;
-
    if ( ( ret = pv_read_all_pv_of_vg ( vg_name, &pv, FALSE)) < 0) {
 #ifdef DEBUG
       debug ( "vg_read -- pv_read_all_pv_of_vg returned: %d\n", ret);
@@ -71,14 +75,13 @@ int vg_read ( char *vg_name, vg_t **vg) {
          ret = -LVM_EVG_READ_OPEN;
       else if ( lseek ( pv_handle, pv[0]->vg_on_disk.base, SEEK_SET) != 
                 pv[0]->vg_on_disk.base) ret = -LVM_EVG_READ_LSEEK;
-      else if ( read ( pv_handle, &vg_this, sizeof ( vg_t)) != \
-                sizeof ( vg_t)) ret = -LVM_EVG_READ_READ;
-      for ( p = 0; p < vg_this.pv_max; p++) vg_this.pv[p] = NULL;
-   
+      else if ( read ( pv_handle, &vg_this, sizeof ( vg_this)) != \
+                sizeof ( vg_this)) ret = -LVM_EVG_READ_READ;
       if ( pv_handle != -1) close ( pv_handle);
-      if ( vg_this.vg_status & VG_EXPORTED) ret = -LVM_EVG_READ_VG_EXPORTED;
    
-      *vg = &vg_this;
+      *vg = vg_copy_from_disk ( &vg_this);
+      if ( (*vg)->vg_status & VG_EXPORTED) ret = -LVM_EVG_READ_VG_EXPORTED;
+   
       if ( ret == 0) {
          if ( pv != NULL) for ( p = 0; pv[p] != NULL; p++) {
             if ( pv[p]->version != LVM_STRUCT_VERSION) {
@@ -89,18 +92,19 @@ int vg_read ( char *vg_name, vg_t **vg) {
       }
    } else ret = -LVM_EVG_READ_PV;
 
+   if ( ret != 0 && ret != -LVM_EVG_READ_VG_EXPORTED) *vg = NULL;
+
 #ifdef DEBUG
-   debug ( "vg_read -- LEAVING %d\n", ret);
+   debug ( "vg_read -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
 
 
 int vg_read_from_pv ( char *pv_name, vg_t **vg) {
-   int p = 0;
    int pv_handle = -1;
    int ret = 0;
-   static vg_t vg_this;
+   static vg_disk_t vg_this;
    pv_t *pv = NULL;
 
 #ifdef DEBUG
@@ -111,8 +115,6 @@ int vg_read_from_pv ( char *pv_name, vg_t **vg) {
         pv_check_name ( pv_name) < 0 ||
         vg == NULL) return -LVM_EPARAM;
 
-   *vg = NULL;
-
    if ( pv_read ( pv_name, &pv, NULL) < 0)
       ret = -LVM_EVG_READ_FROM_PV_PV_READ;
    else if ( pv_check_consistency ( pv) < 0)
@@ -121,21 +123,19 @@ int vg_read_from_pv ( char *pv_name, vg_t **vg) {
       ret = -LVM_EVG_READ_FROM_PV_OPEN;
    else if ( lseek ( pv_handle, pv->vg_on_disk.base, SEEK_SET) != 
              pv->vg_on_disk.base) ret = -LVM_EVG_READ_FROM_PV_LSEEK;
-   else if ( read ( pv_handle, &vg_this, sizeof ( vg_t)) != \
-             sizeof ( vg_t)) ret = -LVM_EVG_READ_FROM_PV_READ;
+   else if ( read ( pv_handle, &vg_this, sizeof ( vg_this)) != \
+             sizeof ( vg_this)) ret = -LVM_EVG_READ_FROM_PV_READ;
    else {
-      vg_this.pv[0] = pv;
-      for ( p = 1; p < vg_this.pv_max; p++) vg_this.pv[p] = NULL;
-   
-      if ( vg_this.vg_status & VG_EXPORTED) ret = -LVM_EVG_READ_VG_EXPORTED;
-   
-      *vg = &vg_this;
-
+      *vg = vg_copy_from_disk ( &vg_this);
+      (*vg)->pv[0] = pv;
+      if ( (*vg)->vg_status & VG_EXPORTED) ret = -LVM_EVG_READ_VG_EXPORTED;
       if ( ret == 0 && pv->version != LVM_STRUCT_VERSION)
          ret = -LVM_EVG_READ_FROM_PV_LVM_STRUCT_VERSION;
    }
 
    if ( pv_handle != -1) close ( pv_handle);
+   if ( ret != 0) *vg = NULL;
+
 #ifdef DEBUG
    debug ( "vg_read_from_pv -- LEAVING %d\n", ret);
 #endif
@@ -172,7 +172,7 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
    debug ( "vg_read_with_pv_and_lv -- AFTER vg_read; "
             " vg_this->lv_cur: %lu\n", vg_this->lv_cur);
    debug ( "vg_read_with_pv_and_lv -- vg_show AFTER first read\n");
-   if ( opt_d > 0) vg_show ( *vg);
+   if ( opt_d > 0) vg_show ( vg_this);
 #endif
 
    if ( vg_this->pv_cur == 0) return -LVM_EVG_READ_WITH_PV_AND_LV_PV_CUR;
@@ -203,7 +203,7 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
 #endif
       /* set up PE pointers in PVs */
       for ( p = 0; p < vg_this->pv_cur; p++) {
-         if ( ( vg_this->pv[p] = malloc ( sizeof ( vg_t))) == NULL) {
+         if ( ( vg_this->pv[p] = malloc ( sizeof ( pv_t))) == NULL) {
             fprintf ( stderr, "malloc error in %s [line %d]\n",
                               __FILE__, __LINE__);
             vg_free ( vg_this, FALSE);
@@ -254,20 +254,18 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
                }
                /* construct the lv_current_pe pointer array */
                p = npe = 0;
-               while ( p < vg_this->pv_cur &&
-                       npe < vg_this->lv[l]->lv_allocated_le) {
+               for ( p = 0; p < vg_this->pv_cur &&
+                            npe < vg_this->lv[l]->lv_allocated_le; p++) {
                   for ( ope = 0; ope < vg_this->pv[p]->pe_total; ope++) {
                      if ( vg_this->pv[p]->pe[ope].lv_num == lv_num) {
                         pe_index = vg_this->pv[p]->pe[ope].le_num;
                         vg_this->lv[l]->lv_current_pe[pe_index].dev =
                            vg_this->pv[p]->pv_dev;
                         vg_this->lv[l]->lv_current_pe[pe_index].pe =
-                           ( ope * vg_this->pv[p]->pe_size) +
-                           ( LVM_DISK_SIZE ( vg_this->pv[p]) / SECTOR_SIZE);
+			   LVM_PE_DISK_OFFSET( ope,  vg_this->pv[p]);
                         npe++;
                      }
                   }
-                  p++;
                }
                if ( npe != vg_this->lv[l]->lv_allocated_le)
                   return -LVM_EVG_READ_WITH_PV_AND_LV_LV_ALLOCATED_LE;
