@@ -1,25 +1,35 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/vg_set_clear_extendable.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * May 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
  */
 
+/*
+ * Changelog
+ *
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
+ *
+ */
+
 #include <liblvm.h>
 
 /* internal function */
@@ -46,7 +56,7 @@ int vg_set_clear_extendable ( char *vg_name, int what) {
 
    if ( vg_name == NULL || vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    if ( ( group = open ( group_file, O_RDWR)) == -1)
       ret = -LVM_EVG_SET_CLEAR_EXTENDABLE_OPEN;
    else {
