@@ -1,20 +1,23 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lib/vg_status.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March,November 1997
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  * 
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * You should have received a copy of the GNU Library General Public License
+ * along with GNU CC; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -25,6 +28,7 @@
  *
  *    1997/11/18 - changed formal parameters in lv_status_all_lv_of_vg to
  *                 avoid another vg_read in there
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -46,7 +50,7 @@ int vg_status ( char *vg_name, vg_t **vg) {
 
    *vg = NULL;
 
-   sprintf ( group_file, "/dev/%s/group%c", vg_name, 0);
+   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
    if ( ( group = open ( group_file, O_RDONLY)) == -1)
       ret = -LVM_EVG_STATUS_OPEN;
    else ret = ioctl ( group, VG_STATUS, &vg_this);
