@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lvchange.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * May,October 1997
- * April-June 1998
+ * April-June,August 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,7 @@
  *    05/12/1998 - added some error messages
  *    05/16/1998 - added lvmtab checking
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/04/1998 - fixed lv_write_all_pv() called every time
  *
  */
 
@@ -77,35 +80,35 @@ int main ( int argc, char **argv)
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'a':
             if ( opt_a > 0) {
-               fprintf ( stderr, "%s -- a option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- a option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_a++;
             if ( strcmp ( optarg, "y") == 0) { lv_stat |= LV_ACTIVE; break;}
             if ( strcmp ( optarg, "n") == 0) break;
             fprintf ( stderr, "%s -- a option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
          case 'C':
             if ( opt_C > 0) {
-               fprintf ( stderr, "%s -- C option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- C option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_C++;
             if ( strcmp ( optarg, "y") == 0) { lv_allocation |= LV_CONTIGUOUS;
@@ -113,14 +116,14 @@ int main ( int argc, char **argv)
             if ( strcmp ( optarg, "n") == 0) break;
             fprintf ( stderr, "%s -- C option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -148,8 +151,8 @@ int main ( int argc, char **argv)
 
          case 'p':
             if ( opt_p > 0) {
-               fprintf ( stderr, "%s -- p option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- p option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_p++;
             if ( strcmp ( optarg, "rw") == 0) {
@@ -159,13 +162,13 @@ int main ( int argc, char **argv)
             if ( strcmp ( optarg, "r") == 0) { lv_access = LV_READ; break;}
             fprintf ( stderr, "%s -- p option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -173,19 +176,21 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( opt_a == 0 && opt_C == 0 && opt_p == 0) {
       fprintf ( stderr, "%s -- please give one or more options a, C, p\n\n",
                 cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please give a logical volume path\n\n", cmd);
-      return 1;
+      return LVM_ELVCHANGE_LV_PATH;
    }
 
    LVM_LOCK ( 0);
@@ -197,88 +202,101 @@ int main ( int argc, char **argv)
 
       if ( opt_v > 0) printf ( "%s -- checking logical volume name\n", cmd);
       if ( lv_check_name ( lv_name) < 0) {
-         fprintf ( stderr, "%s -- invalid logical volume name %s\n\n",
+         fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
                    cmd, lv_name);
          continue;
       }
    
       vg_name = vg_name_of_lv ( lv_name);
    
-      if ( opt_v > 0) printf ( "%s -- checking for active volume group %s\n",
+      /* does VG exist? */
+      if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
+      if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
+         fprintf ( stderr, "%s -- can't change logical volume: volume group "
+                           "\"%s\" doesn't exist\n\n", cmd, vg_name);
+         return LVM_ELVCHANGE_VG_CHECK_EXIST;
+      }
+
+      if ( opt_v > 0) printf ( "%s -- checking for active volume "
+                               "group \"%s\"\n",
                                cmd, vg_name);
       if ( vg_check_active ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- volume group %s is not active\n\n",
+         fprintf ( stderr, "%s -- volume group \"%s\" is not active\n\n",
                    cmd, vg_name);
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- reading volume group data of %s "
+      if ( opt_v > 0) printf ( "%s -- reading volume group data of \"%s\" "
                                "from disk(s)\n", cmd, vg_name);
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
          if ( ret == -LVM_EPV_READ_PV_EXPORTED) {
-            fprintf ( stderr, "%s -- volume group %s is exported\n\n",
+            fprintf ( stderr, "%s -- volume group \"%s\" is exported\n\n",
                       cmd, vg_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d: reading data of %s\n\n",
+            fprintf ( stderr, "%s -- ERROR %d: reading data of \"%s\"\n\n",
                       cmd, ret, vg_name);
          }
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking logical volume %s existence\n",
+      if ( opt_v > 0) printf ( "%s -- checking logical volume \"%s\" "
+                               "existence\n",
                                cmd, lv_name);
       if ( ( ret = lvm_tab_lv_check_exist ( lv_name)) == FALSE) {
-         fprintf ( stderr, "%s -- logical volume %s doesn't exist\n\n",
+         fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
                    cmd, lv_name);
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking consistency of %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking consistency of \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d checking consistency of %s\n\n",
+         fprintf ( stderr, "%s -- ERROR %d checking consistency of \"%s\"\n\n",
                    cmd, ret, vg_name);
          continue;
       }
       
-      if ( opt_v > 0) printf ( "%s -- getting index of %s in %s\n",
+      if ( opt_v > 0) printf ( "%s -- getting index of logical volume \"%s\" "
+                               "in volume group \"%s\"\n",
                                cmd, lv_name, vg_name);
       if ( ( l = lv_get_index_by_name ( vg, lv_name)) < 0) {
-         fprintf ( stderr, "%s -- couldn't get logical volume index in %s\n\n",
+         fprintf ( stderr, "%s -- couldn't get logical volume index in "
+                           "volume group \"%s\"\n\n",
                    cmd, vg_name);
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- getting status of %s from VGDA in "
+      if ( opt_v > 0) printf ( "%s -- getting status of \"%s\" from VGDA in "
                                "kernel\n", cmd, lv_name);
       if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d getting status of logical "
-                           "volume %s\n\n", cmd, ret, lv_name);
+                           "volume \"%s\"\n\n", cmd, ret, lv_name);
          continue;
       }
    
       lvm_dont_interrupt ( 0);
    
-      if ( opt_v > 0) printf ( "%s -- changing logical volume %s in VGDA "
+      if ( opt_v > 0) printf ( "%s -- changing logical volume \"%s\" in VGDA "
                                "of kernel\n", cmd, lv_name);
       /* O_RDONLY to avoid error when LV is inactive!!! (see lvm_blk_open) */
-      if ( ( blkdev = open ( lv_name, O_RDONLY)) == -1) {
-         fprintf ( stderr, "%s -- couldn't open logical volume %s\n\n",
-                   cmd, lv_name);
-         return 1;
+      if ( ( blkdev = open ( lv_name, O_RDONLY)) < 0) {
+         fprintf ( stderr, "%s -- ERROR %d: couldn't open logical "
+                           "volume \"%s\"\n\n",
+                           cmd, errno, lv_name);
+         return LVM_ELVCHANGE_LV_OPEN;
       }
    
       /* access permission change */
       if ( opt_p > 0) {
          doit = 0;
          if ( lv_access & LV_WRITE) {
-            if ( lv->lv_access & LV_WRITE) printf ( "%s -- %s is yet "
-                                                    "writable\n",
+            if ( lv->lv_access & LV_WRITE)
+               printf ( "%s -- logical volume \"%s\" is already writable\n",
                                                     cmd, lv_name);
             else doit++;
          } else {
             if ( ! ( lv->lv_access & LV_WRITE))
-               printf ( "%s -- %s is yet readonly\n",
+               printf ( "%s -- logical volume \"%s\" is already readonly\n",
                         cmd, lv_name);
             else doit++;
          }
@@ -287,23 +305,25 @@ int main ( int argc, char **argv)
             doit_sum++;
             lv->lv_access = lv_access;
             if ( opt_v > 0) printf ( "%s -- changing access permissions "
-                                     "of %s\n", cmd, lv_name);
+                                     "of logical volume \"%s\"\n",
+                                     cmd, lv_name);
             if ( ( ret = ioctl ( blkdev, LV_SET_ACCESS,
                                  ( ulong) lv->lv_access)) < 0) {
                if ( ret == -EPERM) {
                   fprintf ( stderr, "%s -- operation not permitted "
-                                    "for (open) logical volume %s\n\n",
+                                    "for (open) logical volume \"%s\"\n\n",
                                     cmd, lv_name);
                } else {
                   fprintf ( stderr, "%s -- ERROR %d: couldn't set access "
-                                    "permissions for logical volume %s\n\n",
+                                    "permissions for logical volume \"%s\"\n\n",
                                     cmd, ret, lv_name);
                }
-               return 1;
+               return LVM_ELVCHANGE_LV_SET_ACCESS;
             }
          } else {
-            printf ( "%s -- access permissions of %s don't have to "
-                     "be changed\n", cmd, lv_name);
+            printf ( "%s -- permissions of logical volume \"%s\" don't have "
+                     "to be changed\n",
+                     cmd, lv_name);
          }
       }
    
@@ -312,11 +332,13 @@ int main ( int argc, char **argv)
          doit = 0;
          if ( lv_stat & LV_ACTIVE) {
             if ( lv->lv_status & LV_ACTIVE)
-               printf ( "%s -- %s is yet active\n", cmd, lv_name);
+               printf ( "%s -- logical volume \"%s\" is already active\n",
+                        cmd, lv_name);
             else doit++;
          } else {
             if ( ! ( lv->lv_status & LV_ACTIVE))
-               printf ( "%s -- %s isn't active\n", cmd, lv_name);
+               printf ( "%s -- logical volume \"%s\" isn't active\n",
+                        cmd, lv_name);
             else doit++;
          }
    
@@ -324,23 +346,25 @@ int main ( int argc, char **argv)
             doit_sum++;
             if ( lv_stat & LV_ACTIVE) lv->lv_status |= LV_ACTIVE;
             else                      lv->lv_status &= ~LV_ACTIVE;
-            if ( opt_v > 0) printf ( "%s -- changing availiability of %s\n",
+            if ( opt_v > 0) printf ( "%s -- changing availiability of "
+                                     "logical volume \"%s\"\n",
                                      cmd, lv_name);
             if ( ( ret = ioctl ( blkdev, LV_SET_STATUS,
                                  ( ulong) lv->lv_status)) < 0) {
                if ( ret == -EPERM) {
                   fprintf ( stderr, "%s -- operation not permitted "
-                                    "for (open) logical volume %s\n\n",
+                                    "for (open) logical volume \"%s\"\n\n",
                                     cmd, lv_name);
                } else {
                   fprintf ( stderr, "%s -- ERROR %d: couldn't set state for "
-                                    "logical volume %s\n\n",
+                                    "logical volume \"%s\"\n\n",
                                     cmd, ret, lv_name);
                }
-               return 1;
+               return LVM_ELVCHANGE_LV_SET_STATUS;
             }
          } else {
-            printf ( "%s -- availiability of %s doesn't have to be changed\n",
+            printf ( "%s -- availiability of logical volume \"%s\" doesn't "
+                     "have to be changed\n",
                       cmd, lv_name);
          }
       }
@@ -350,12 +374,14 @@ int main ( int argc, char **argv)
          doit = 0;
          if ( lv_allocation & LV_CONTIGUOUS) {
             if ( lv->lv_allocation & LV_CONTIGUOUS)
-               printf ( "%s -- allocation policy of %s is yet contiguous\n",
+               printf ( "%s -- allocation policy of logical volume \"%s\" "
+                        "is already contiguous\n",
                          cmd, lv_name);
             else doit++;
          } else {
             if ( ! ( lv->lv_allocation & LV_CONTIGUOUS))
-               printf ( "%s -- allocation policy of %s is not contiguous\n",
+               printf ( "%s -- allocation policy of logical volume \"%s\" "
+                        "is not contiguous\n",
                          cmd, lv_name);
             else doit++;
          }
@@ -367,29 +393,31 @@ int main ( int argc, char **argv)
                lv->lv_allocation &= ~LV_CONTIGUOUS;
             if ( ( lv_allocation & LV_CONTIGUOUS ) &&
                  ( ret = lv_check_contiguous ( vg, l + 1)) == FALSE) {
-               fprintf ( stderr, "%s -- no contiguous volume %s\n",
+               fprintf ( stderr, "%s -- no contiguous logical volume \"%s\"\n",
                          cmd, lv_name);
             } else {
                if ( opt_v > 0) printf ( "%s -- changing allocation policy "
-                                        "of %s\n", cmd, lv_name);
+                                        "of logical volume \"%s\"\n",
+                                        cmd, lv_name);
                if ( ( ret = ioctl ( blkdev, LV_SET_ALLOCATION,
                                     ( ulong) lv->lv_allocation)) < 0) {
                   if ( ret == -EPERM) {
                      fprintf ( stderr, "%s -- operation not permitted "
-                                       "for (open) logical volume %s\n\n",
+                                       "for (open) logical volume \"%s\"\n\n",
                                        cmd, lv_name);
                   } else {
                      fprintf ( stderr, "%s -- ERROR %d: couldn't set "
-                                       "allocation for logical volume %s\n\n",
+                                       "allocation for logical volume "
+                                       "\"%s\"\n\n",
                                        cmd, ret, lv_name);
                   }
-                  return 1;
+                  return LVM_ELVCHANGE_LV_SET_ALLOCATION;
                }
                doit_sum++;
             }
          } else {
-            printf ( "%s -- allocation policy of %s doesn't have "
-                     "to be changed\n", cmd, lv_name);
+            printf ( "%s -- allocation policy of logical volume \"%s\" "
+                     "doesn't have to be changed\n", cmd, lv_name);
          }
       }
    
@@ -397,25 +425,26 @@ int main ( int argc, char **argv)
       vg->lv[l]->lv_allocation = lv->lv_allocation;
       vg->lv[l]->lv_status = lv->lv_status;
    
-      if ( opt_v > 0) printf ( "%s -- storing logical volume on disk(s)\n",
-                               cmd);
-      if ( ( ret = lv_write_all_pv ( vg, l)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: couldn't store logical volume "
-                           "information of %s\n",
-                   cmd, ret, lv_name);
-         return 1;
-      }
-   
       if ( doit_sum > 0) {
-         printf ( "%s -- logical volume %s changed\n",
+         if ( opt_v > 0) printf ( "%s -- storing logical volume on disk(s)\n",
+                                  cmd);
+         if ( ( ret = lv_write_all_pv ( vg, l)) < 0) {
+            fprintf ( stderr, "%s -- ERROR %d: couldn't store logical volume "
+                              "information of \"%s\"\n",
+                      cmd, ret, lv_name);
+            return LVM_ELVCHANGE_LV_WRITE_ALL_PV;
+         }
+   
+         printf ( "%s -- logical volume \"%s\" changed\n",
                    cmd, lv_name);
          if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
          if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
               opt_A > 0) {
-            printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+            printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+                     cmd, vg_name);
             vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
          }
-      } else  printf ( "%s -- logical volume %s didn't have "
+      } else  printf ( "%s -- logical volume \"%s\" didn't have "
                        "to be changed\n", cmd, lv_name);
 
       doit_sum = 0;
