@@ -1,8 +1,11 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lvcreate.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March,October,November 1997
- * January,May,June 1998
+ * January,May-July,September,December 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +37,14 @@
  *    06/14/1998 - supported physical volumes on command line for
  *                 striped logical volumes
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    07/05/1998 - fixed display bug with option verbose and
+ *                 physical volumes on command line
+ *    07/09/1998 - fixed VG name bug
+ *                 (VG name starting with "/dev/" was an error);
+ *                 Thanks to Pascal van Dam <pascal@ramoth.xs4all.nl>
+ *    09/02/1998 - corrected some messages
+ *    12/20/1998 - added check for volume group existence
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -50,6 +61,7 @@
  *
  *    01/18/1998 - added option z to give possibility to disallow zeroing 
  *                 of newly created logical volumes
+ *    01/28/1999 - fixed logical volume path too long in command line
  *
  */
 
@@ -110,22 +122,22 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'C':
             if ( opt_C > 0) {
-               fprintf ( stderr, "%s -- C option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- C option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_C++;
             if ( strcmp ( optarg, "y") == 0) { allocation |= LV_CONTIGUOUS;
@@ -133,14 +145,14 @@ int main ( int argc, char **argv) {
             if ( strcmp ( optarg, "n") == 0) break;
             fprintf ( stderr, "%s -- C option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -172,8 +184,8 @@ int main ( int argc, char **argv) {
 
          case 'i':
             if ( opt_i > 0) {
-               fprintf ( stderr, "%s -- i option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- i option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( lvm_check_number ( optarg) == TRUE) {
                stripes = atoi ( optarg);
@@ -182,14 +194,14 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option i argument \"%s\"\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'I':
             if ( opt_I > 0) {
-               fprintf ( stderr, "%s -- I option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- I option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
 
             if ( lvm_check_number ( optarg) == TRUE) {
@@ -198,18 +210,18 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option I argument \"%s\"\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'l':
             if ( opt_L > 0) {
-               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- L option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( opt_l > 0) {
-               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- l option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
 
             if ( lvm_check_number ( optarg) == TRUE) {
@@ -218,18 +230,18 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'L':
             if ( opt_l > 0) {
-               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- l option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( opt_L > 0) {
-               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- L option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
 
             if ( lvm_check_number ( optarg) == TRUE) {
@@ -238,19 +250,19 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option L argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'n':
             if ( opt_n > 1) {
-               fprintf ( stderr, "%s -- n option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- n option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strchr ( optarg, '/') != NULL) {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             strncpy ( lv_name, optarg, sizeof ( lv_name) - 1);
             lv_name[sizeof ( lv_name) - 1] = 0;
@@ -258,8 +270,8 @@ int main ( int argc, char **argv) {
 
          case 'p':
             if ( opt_p > 0) {
-               fprintf ( stderr, "%s -- p option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- p option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_p++;
             if ( strcmp ( optarg, "rw") == 0) {
@@ -269,64 +281,93 @@ int main ( int argc, char **argv) {
             if ( strcmp ( optarg, "r") == 0) { permission = LV_READ; break;}
             fprintf ( stderr, "%s -- p option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
 
          case 'Z':
             if ( opt_Z > 1) {
-               fprintf ( stderr, "%s -- Z option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- Z option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0) opt_Z = 1;
             else if ( strcmp ( optarg, "n") == 0) opt_Z = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
-   if ( optind < argc && *argv[optind] == '-') {
-      fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
-      return 1;
-   }
+   CMD_MINUS_CHK;
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name\n\n", cmd);
-      return 1;
+      return LVM_ELVCREATE_VG_NAME;
    }
    vg_name = argv[optind];
    optind++;
 
+   if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
+                            cmd, vg_name);
+   if ( vg_check_name ( vg_name) < 0) {
+      fprintf ( stderr, "%s -- no valid volume group name \"%s\"\n\n",
+                cmd, vg_name);
+      return LVM_ELVCREATE_VG_NAME;
+   }
+
+   /* does VG exist? */
+   if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
+   if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
+      fprintf ( stderr, "%s -- can't create logical volume: volume group "
+                        "\"%s\" doesn't exist\n\n", cmd, vg_name);
+      return LVM_ELVCREATE_VG_CHECK_EXIST;
+   }
+
+   if ( opt_v > 0) printf ( "%s -- checking volume group activity\n", cmd);
+   if ( vg_check_active ( vg_name) != TRUE) {
+      fprintf ( stderr, "%s -- can't create logical volume:"
+                        " \"%s\" isn't active\n\n", cmd, vg_name);
+      return LVM_ELVCREATE_VG_CHECK_ACTIVE;
+   }
+
    if ( strlen ( lv_name) > 0) {
-      sprintf ( lv_path, "/dev/%s/%s%c", vg_name, lv_name, 0);
+      if ( strlen ( LVM_DIR_PREFIX) + strlen ( vg_name) +
+           strlen ( lv_name) + 1 > NAME_LEN - 1) {
+         fprintf ( stderr, "%s -- the logical volume path is longer than "
+                           "the maximum of %d!\n\n",
+                           cmd, sizeof ( lv_path));
+         return LVM_ELVCREATE_LV_CHECK_NAME;
+      }
+      sprintf ( lv_path, LVM_DIR_PREFIX "%s/%s%c", vg_name, lv_name, 0);
+
+      if ( opt_v > 0) printf ( "%s -- checking logical volume path \"%s\"\n",
+                               cmd, lv_path);
       if ( lv_check_name ( lv_path) < 0) {
-         fprintf ( stderr, "%s -- invalid logical volume name %s\n\n",
+         fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
                            cmd, lv_name);
-         return 1;
+         return LVM_ELVCREATE_LV_CHECK_NAME;
       } else if ( lstat ( lv_path, &stat_buf) != -1) {
          lvm_show_filetype ( stat_buf.st_mode, lv_path);
-         return 1;
-      }
-      else if ( lvm_tab_lv_check_exist ( lv_path) == TRUE) {
-         fprintf ( stderr, "%s -- logical volume %s allready exists\n\n",
+         return LVM_ELVCREATE_LV_LSTAT;
+      } else if ( lvm_tab_lv_check_exist ( lv_path) == TRUE) {
+         fprintf ( stderr, "%s -- logical volume \"%s\" already exists\n\n",
                            cmd, lv_path);
-         return 1;
+         return LVM_ELVCREATE_LV_CHECK_EXIST;
       }
    }
 
@@ -335,9 +376,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- checking physical volume names\n", cmd);
       for ( i = optind; i < argc; i++) {
          if ( pv_check_name ( argv[i]) < 0) {
-            fprintf ( stderr, "%s -- %s is an invalid physical "
+            fprintf ( stderr, "%s -- \"%s\" is an invalid physical "
                               "volume name\n\n", cmd, argv[i]);
-            return 1;
+            return LVM_ELVCREATE_PV_CHECK_NAME;
          }
       }
       pv_allowed = &argv[optind];
@@ -349,7 +390,7 @@ int main ( int argc, char **argv) {
       if ( pv_allowed != NULL && argc - optind != stripes) {
          fprintf ( stderr, "%s -- invalid number of physical "
                            "volumes on command line\n\n", cmd);
-         return 1;
+         return LVM_ELVCREATE_PV_NUMBER;
       }
       if ( stripes == 1) {
          printf ( "%s -- redundant option i with value 1 is default\n", cmd);
@@ -363,56 +404,34 @@ int main ( int argc, char **argv) {
       fprintf ( stderr, "%s -- invalid number of stripes: %d\n"
                         "%s -- must be between %d and %d\n\n",
                         cmd, stripes, cmd, 1, LVM_MAX_STRIPES);
-      return 1;
+      return LVM_ELVCREATE_STRIPES;
    }
 
    if ( opt_v > 0) printf ( "%s -- checking stripe size\n", cmd);
    if ( stripesize > 0 && lv_check_stripesize ( stripesize) < 0) {
       fprintf ( stderr, "%s -- invalid stripesize %d\n\n",
                 cmd, stripesize);
-      return 1;
+      return LVM_ELVCREATE_STRIPE_SIZE;
    }
    stripesize *= 2;
    if ( stripes == 1) stripesize = 0;
 
-   if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
-   if ( vg_check_name ( vg_name) < 0) {
-      fprintf ( stderr, "%s -- no valid volume group name %s\n\n",
-                cmd, vg_name);
-      return 1;
-   }
-
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
 
-   /* does VG exist? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
-   if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
-      fprintf ( stderr, "%s -- can't create logical volume:"
-                        " %s doesn't exists\n\n", cmd, vg_name);
-      return 1;
-   }
-
-   if ( opt_v > 0) printf ( "%s -- checking volume group activity\n", cmd);
-   if ( vg_check_active ( vg_name) != TRUE) {
-      fprintf ( stderr, "%s -- can't create logical volume:"
-                        " %s isn't active\n\n", cmd, vg_name);
-      return 1;
-   }
-
    if ( opt_v > 0) printf ( "%s -- getting volume group status from "
                             "VGDA in kernel\n", cmd);
-   if ( vg_status ( vg_name, &vg) != 0) {
+   if ( vg_status ( vg_name, &vg) < 0) {
       fprintf ( stderr, "%s -- can't create logical volume"
-                        " in inactive volume group %s\n\n", cmd, vg_name);
-      return 1;
+                        " in inactive volume group \"%s\"\n\n", cmd, vg_name);
+      return LVM_ELVCREATE_VG_STATUS;
    }
 
    if ( opt_v > 0) printf ( "%s -- checking stripe size against "
-                            "volume group physical extend size\n", cmd);
+                            "volume group physical extent size\n", cmd);
    if ( stripesize > vg->pe_size) {
       fprintf ( stderr, "%s -- setting stripe size %d KB to "
-                        "physical extend size %u KB\n",
+                        "physical extent size %u KB\n",
                         cmd, stripesize / 2, vg->pe_size / 2);
       stripesize = vg->pe_size;
    }
@@ -422,7 +441,7 @@ int main ( int argc, char **argv) {
       else            size *= vg->pe_size;
       if ( size % vg->pe_size > 0) {
          size += vg->pe_size - size % vg->pe_size;
-         printf ( "%s -- rounding up size to physical extend boundary %s\n",
+         printf ( "%s -- rounding up size to physical extent boundary \"%s\"\n",
                   cmd, ( dummy = lvm_show_size ( size / 2, SHORT)));
          free ( dummy); dummy = NULL;
       }
@@ -436,68 +455,69 @@ int main ( int argc, char **argv) {
       }
    } else {
       fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
-      return 1;
-   }
-
-   if ( opt_v > 0) printf ( "%s -- checking volume group free space\n", cmd);
-   pe_free = vg->pe_total - vg->pe_allocated;
-   if ( pe_free == 0) {
-      fprintf ( stderr, "%s -- no free physical extends in %s\n\n",
-                cmd, vg_name);
-      return 1;
-   }
-   else if ( size / vg->pe_size > pe_free) {
-      fprintf ( stderr, "%s -- only %lu free physical extend", cmd, pe_free);
-      if ( pe_free > 1) fprintf ( stderr, "s");
-      fprintf ( stderr, " in %s\n\n", vg_name);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( opt_v > 0) printf ( "%s -- checking logical volume maximum size\n",
                             cmd);
    if ( size > LVM_LV_SIZE_MAX( vg)) {
-      fprintf ( stderr, "%s -- size %d is to large for the VGDA in kernel\n\n",
-                cmd, size / 2);
-      return 1;
+      fprintf ( stderr, "%s -- size %d is larger than maximum VGDA kernel size "
+                        "of %llu\n\n",
+                        cmd, size / 2, LVM_LV_SIZE_MAX( vg) / 2);
+      return LVM_ELVCREATE_LV_SIZE;
    }
 
+   if ( opt_v > 0) printf ( "%s -- checking volume group free space\n", cmd);
+   pe_free = vg->pe_total - vg->pe_allocated;
+   if ( pe_free == 0) {
+      fprintf ( stderr, "%s -- no free physical extents in "
+                        "volume group \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_ELVCREATE_PE_FREE;
+   }
+   else if ( size / vg->pe_size > pe_free) {
+      fprintf ( stderr, "%s -- only %lu free physical extent%s in "
+                        "volume group \"%s\"\n\n",
+                        cmd, pe_free, pe_free > 1 ? "s" : "", vg_name);
+      return LVM_ELVCREATE_PE_FREE;
+   }
 
    if ( opt_v > 0) printf ( "%s -- checking stripe count against "
                             "physical volume count\n", cmd);
    if ( stripes > vg->pv_cur) {
-      fprintf ( stderr, "%s -- too many stripes for %s with %u "
-                        "physical volume",
-                cmd, vg_name, vg->pv_cur);
-      if ( vg->pv_cur > 1) fprintf ( stderr, "s");
-      fprintf ( stderr, "\n\n");
-      return 1;
+      fprintf ( stderr, "%s -- too many stripes for \"%s\" with %u "
+                        "physical volume%s\n\n",
+                        cmd, vg_name, vg->pv_cur, vg->pv_cur > 1 ? "s" : "");
+      return LVM_ELVCREATE_STRIPE_COUNT;
    }
 
-   if ( opt_v > 0) printf ( "%s -- reading volume group data of %s\n",
+   if ( opt_v > 0) printf ( "%s -- reading volume group data of \"%s\"\n",
                             cmd, vg_name);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
       fprintf ( stderr, "%s -- ERROR %d: couldn't get volume "
-                        "group data of %s\n\n",
+                        "group data of \"%s\"\n\n",
                         cmd, ret, vg_name);
-      return 1;
+      return LVM_ELVCREATE_VG_READ;
    }
 
    if ( pv_allowed != NULL) {
       for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
-         if ( opt_v > 0) printf ( "%s -- checking physical volume %s against "
-                                  "volume group\n", pv_allowed[pa], cmd);
+         if ( opt_v > 0) printf ( "%s -- checking physical volume \"%s\" "
+                                  "against volume group\n",
+                                  cmd, pv_allowed[pa]);
          if ( pv_check_in_vg ( vg, pv_allowed[pa]) == FALSE) {
-            fprintf ( stderr, "%s -- %s doesn't belong to volume group %s\n\n",
+            fprintf ( stderr, "%s -- \"%s\" doesn't belong to volume "
+                              "group \"%s\"\n\n",
                               cmd, pv_allowed[pa], vg_name);
-            return 1;
+            return LVM_ELVCREATE_PV_CHECK_IN_VG;
          }
          if ( pv_read ( pv_allowed[pa], &pv, NULL) < 0) {
-            fprintf ( stderr, "%s -- couldn't read physical volume %s\n\n",
+            fprintf ( stderr, "%s -- couldn't read physical volume \"%s\"\n\n",
                               cmd, pv_allowed[pa]);
-            return 1;
+            return LVM_ELVCREATE_PV_READ;
          }
          if ( pv->pe_total - pv->pe_allocated == 0)
-            printf ( "%s -- no space on physical volume %s\n",
+            printf ( "%s -- no space on physical volume \"%s\"\n",
                      cmd, pv_allowed[pa]);
       }
    }
@@ -506,9 +526,9 @@ int main ( int argc, char **argv) {
                             cmd);
    if ( vg->lv_cur == vg->lv_max) {
       fprintf ( stderr, "%s -- maximum number of logical volumes "
-                        "reached in %s\n\n",
+                        "reached in \"%s\"\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_ELVCREATE_LV_COUNT;
    }
 
    /* setup logical volume */
@@ -519,18 +539,18 @@ int main ( int argc, char **argv) {
                                       pv_allowed)) != 0) {
       if ( ret == -LVM_ESIZE) {
          if ( pv_allowed != NULL)
-            fprintf ( stderr, "%s -- not enough allocatable/free physical "
-                              "extends on the given physical volumes\n", cmd);
+            fprintf ( stderr, "%s -- not enough allocatable/free space "
+                              "on the given physical volume(s)\n", cmd);
          else
             fprintf ( stderr, "%s -- not enough allocatable/free physical "
-                              "volume space in %s\n", cmd, vg_name);
+                              "volume space in \"%s\"\n", cmd, vg_name);
          fprintf ( stderr, "%s -- please check, if physical volumes are "
                            "allocatable\n\n", cmd);
-         return 1;
+         return LVM_ELVCREATE_VG_SIZE;
       } else {
-         fprintf ( stderr, "%s -- ERROR %d: creating %s in %s\n\n", 
+         fprintf ( stderr, "%s -- ERROR %d: creating \"%s\" in \"%s\"\n\n", 
                    cmd, ret, lv_name, vg_name);
-         return 1;
+         return LVM_ELVCREATE_LV_SETUP;
       }
    }
 
@@ -541,22 +561,23 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- creating logical volume VGDA "
                             "in kernel\n", cmd);
    if ( ( ret = lv_create ( vg, vg->lv[l], lv_name)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating VGDA for %s in kernel\n\n", 
-                cmd, ret, lv_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d creating VGDA for \"%s\" "
+                        "in kernel\n\n", 
+                        cmd, ret, lv_name);
+      return LVM_ELVCREATE_LV_CREATE;
    }
 
    /* store it on disks */
    if ( opt_v > 0) printf ( "%s -- storing logical volume VGDA on disk(s)\n",
                             cmd);
-   if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of %s"
+   if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
+      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of \"%s\""
                         " on disks\n\n", 
                 cmd, ret, vg_name);
-      if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name)) != 0)
-         fprintf ( stderr, "%s -- ERROR %d removing VGDA for %s "
+      if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name)) < 0)
+         fprintf ( stderr, "%s -- ERROR %d removing VGDA for \"%s\" "
                            "from kernel\n\n", cmd, ret, lv_name);
-      return 1;
+      return LVM_ELVCREATE_VG_WRITE;
    }
 
    if ( opt_v > 0) printf ( "%s -- creating device special file\n", cmd);
@@ -564,51 +585,53 @@ int main ( int argc, char **argv) {
                                  vg->vg_number,
                                  vg->lv[l]->lv_number + 1)) < 0) {
       if ( ret == -LVM_ELV_CREATE_MODE_MKNOD) {
-         fprintf ( stderr, "%s -- logical volume node %s allready existent\n\n",
-                   cmd, lv_name);
+         fprintf ( stderr, "%s -- logical volume node \"%s\" already "
+                           "existent\n\n",
+                           cmd, lv_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d creating logical volume node %s\n\n",
-                   cmd, ret, lv_name);
+         fprintf ( stderr, "%s -- ERROR %d creating logical volume "
+                           "node \"%s\"\n\n",
+                           cmd, ret, lv_name);
       }
-      return 1;
+      return LVM_ELVCREATE_LV_CREATE_NODE;
    }
 
    if ( opt_Z > 0) {
-      if ( opt_v > 0) printf ( "%s -- opening logical volume %s for init\n",
+      if ( opt_v > 0) printf ( "%s -- opening logical volume \"%s\" for init\n",
                                cmd, lv_name);
       if ( ( lv_handle = open ( lv_name, O_WRONLY)) == -1) {
-         fprintf ( stderr, "%s -- ERROR %d opening logical volume %s\n\n",
+         fprintf ( stderr, "%s -- ERROR %d opening logical volume \"%s\"\n\n",
                    cmd, errno, lv_name);
-         return 1;
+         return LVM_ELVCREATE_LV_OPEN;
       }
    
-      memset ( buf, 0, 2 * BLOCK_SIZE);
-      if ( opt_v > 0) printf ( "%s -- initializing logical volume %s\n",
+      memset ( buf, 0, sizeof ( buf));
+      if ( opt_v > 0) printf ( "%s -- initializing logical volume \"%s\"\n",
                                cmd, lv_name);
-      if ( write ( lv_handle, buf, 2 * BLOCK_SIZE) != 2 * BLOCK_SIZE) {
+      if ( write ( lv_handle, buf, sizeof ( buf)) != sizeof ( buf)) {
          close ( lv_handle);
          fprintf ( stderr, "%s -- ERROR %d initializing logical "
-                           "volume %s\n\n", cmd, errno, lv_name);
-         return 1;
+                           "volume \"%s\"\n\n", cmd, errno, lv_name);
+         return LVM_ELVCREATE_LV_WRITE;
       }
       fsync ( lv_handle);
       close ( lv_handle);
    } else {
-      printf ( "%s -- WARNING: you are not initializing data on %s\n",
-               cmd, lv_name);
+      printf ( "%s -- WARNING: you are not initializing logical "
+               "volume \"%s\"\n", cmd, lv_name);
    }
 
    if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
    if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
         opt_A > 0) {
-      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+      printf ( "%s -- doing automatic backup of \"%s\"\n", cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
    }
 
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
-   printf ( "%s -- logical volume %s successfully created\n\n",
+   printf ( "%s -- logical volume \"%s\" successfully created\n\n",
              cmd, lv_name);
 
    return 0;
