@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lvdisplay.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * May-June 1998
+ * May-June,August,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +30,9 @@
  *    05/16/1998 - added lvmtab checking
  *    06/07/1998 - added warning message for option -Z
  *    06/27/1998 - changed lvm_tab_* calling convention
- *                 changed to new lv_show_current_pe_tex() calling convention
+ *                 changed to new lv_show_current_pe_text() calling convention
+ *    09/07/1998 - used lv_status_byname() instead of obsolete
+ *                 lv_status_with_pe()
  *
  */
 
@@ -65,8 +69,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -74,8 +78,8 @@ int main ( int argc, char **argv)
 
          case 'D':
             if ( opt_D > 0) {
-               fprintf ( stderr, "%s -- D option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- D option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_D++;
             break;
@@ -99,8 +103,8 @@ int main ( int argc, char **argv)
 
          case 'v':
             if ( opt_v > 1) {
-               fprintf ( stderr, "%s -- v option yet given two times\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given two times\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -108,13 +112,15 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( argc == optind) {
       fprintf ( stderr, "%s -- please enter a logical volume path\n\n", cmd);
-      return 1;
+      return LVM_ELVDISPLAY_LV_MISSING;
    }
 
    LVM_LOCK ( 1);
@@ -122,41 +128,42 @@ int main ( int argc, char **argv)
 
    for ( ;optind < argc; optind++) {
       lv_name = argv[optind];
-      if ( opt_v > 1) printf ( "%s -- checking logical volume name %s\n",
+      if ( opt_v > 1) printf ( "%s -- checking logical volume name \"%s\"\n",
                                cmd, lv_name);
       if ( ( ret = lv_check_name ( lv_name)) < 0) {
-         fprintf ( stderr, "%s -- invalid logical volume name %s\n\n",
+         fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
                    cmd, lv_name);
          continue;
       }
    
       vg_name = vg_name_of_lv ( lv_name);
 
-      if ( opt_v > 1) printf ( "%s -- checking logical volume %s existence\n",
-                               cmd, lv_name);
+      if ( opt_v > 1) printf ( "%s -- checking logical volume \"%s\" "
+                               "existence\n", cmd, lv_name);
       if ( opt_D > 0) ret = lv_check_exist ( lv_name);
       else            ret = lvm_tab_lv_check_exist ( lv_name);
       if ( ret != TRUE) {
-         fprintf ( stderr, "%s -- %s doesn't exist\n\n", cmd, lv_name);
-         continue;
-      }
-   
-      if ( opt_v > 1) printf ( "%s -- checking logical volume activity of %s\n",
-                               cmd, lv_name);
-      if ( lv_check_active ( vg_name, lv_name) < 0 && opt_D == 0) {
-         fprintf ( stderr, "%s -- logical volume %s is not active; try -D\n\n",
+         fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
                            cmd, lv_name);
          continue;
       }
    
-      if ( opt_v > 1) printf ( "%s -- getting logical volume data for %s from ",
-                               cmd, lv_name);
+      if ( opt_v > 1) printf ( "%s -- checking logical volume activity "
+                               "of \"%s\"\n", cmd, lv_name);
+      if ( lv_check_active ( vg_name, lv_name) < 0 && opt_D == 0) {
+         fprintf ( stderr, "%s -- logical volume \"%s\" is not active; "
+                           "try -D\n\n", cmd, lv_name);
+         continue;
+      }
+   
+      if ( opt_v > 1) printf ( "%s -- getting logical volume data for \"%s\" "
+                               "from ", cmd, lv_name);
       /* from disk(s) */
       if ( opt_D > 0) {
          if ( opt_v > 1) printf ( "disk (lvmtab)\n");
          if ( ( ret = lv_read_with_pe ( vg_name, lv_name, &lv)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d: no VALID logical volume %s\n\n",
-                      cmd, ret, lv_name);
+            fprintf ( stderr, "%s -- ERROR %d: no VALID logical volume "
+                              "\"%s\"\n\n", cmd, ret, lv_name);
             continue;
          }
          if ( opt_v > 1) printf ( "%s -- checking logical volume consistency\n",
@@ -164,15 +171,16 @@ int main ( int argc, char **argv)
       /* from kernel */
       } else {
          if ( opt_v > 1) printf ( "kernel\n");
-         if ( ( ret = lv_status_with_pe ( vg_name, lv_name, &lv)) < 0) {
+         if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
             if ( ret == -LVM_ELV_STATUS_INTERNAL_OPEN ||
                  ret == -ENXIO) {
-               fprintf ( stderr, "%s -- logical volume %s doesn't exist\n\n",
-                         cmd, lv_name);
+               fprintf ( stderr, "%s -- logical volume \"%s\" "
+                                 "doesn't exist\n\n",
+                                 cmd, lv_name);
                continue;
             } else {
                fprintf ( stderr, "%s -- ERROR %d getting logical"
-                                 " volume info of %s\n\n",
+                                 " volume info of \"%s\"\n\n",
                          cmd, ret, lv_name);
                continue;
             }
@@ -180,13 +188,13 @@ int main ( int argc, char **argv)
       }
    
       if ( lv == NULL) {
-         fprintf ( stderr, "%s -- logical volume %s not existing\n\n",
+         fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
                            cmd, lv_name);
          continue;
       }
 
       if ( ( ret = lv_check_consistency ( lv)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: logical volume %s "
+         fprintf ( stderr, "%s -- ERROR %d: logical volume \"%s\" "
                            "is inconsistent\n\n", cmd, ret, lv_name);
          continue;
       }
