@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lvmchange.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March-May 1997
- * June 1998
+ * June,August 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +27,7 @@
  * Changelog
  *
  *    06/12/1998 - enhanced checking numbers in option arguments
+ *    09/04/1998 - corrected some messages
  *
  */
 
@@ -61,8 +64,8 @@ int main ( int argc, char **argv) {
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -70,8 +73,8 @@ int main ( int argc, char **argv) {
 
          case 'f':
             if ( opt_f > 0) {
-               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- f option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_f++;
             break;
@@ -96,38 +99,38 @@ int main ( int argc, char **argv) {
 
          case 'r':
             if ( opt_r > 0) {
-               fprintf ( stderr, "%s -- r option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- r option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
 
             if ( lvm_check_number ( optarg) == TRUE) {
                read_ahead = atoi ( optarg);
-               opt_r++;
-               if ( read_ahead < LVM_MIN_READ_AHEAD -10||
+               if ( read_ahead < LVM_MIN_READ_AHEAD  ||
                     read_ahead > LVM_MAX_READ_AHEAD) {
                   fprintf ( stderr, "%s -- read ahead sector number %d is "
                                     "invalid\n\n", cmd, read_ahead);
-                  return 1;
+                  return LVM_EINVALID_CMD_LINE;
                }
             } else {
                fprintf ( stderr, "%s -- ERROR option r argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
+            opt_r++;
             break;
 
          case 'R':
             if ( opt_R > 0) {
-               fprintf ( stderr, "%s -- R option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- R option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_R++;
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -135,18 +138,20 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( opt_r == 0 && opt_R == 0) {
       fprintf ( stderr, "%s -- please give the r and/or R option\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( optind < argc) {
       fprintf ( stderr, "%s -- error on command line\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    LVM_LOCK ( 0);
@@ -154,8 +159,9 @@ int main ( int argc, char **argv) {
    lvm_dont_interrupt ( 0);
 
    if ( ( lvm = open ( LVM_DEV, O_RDONLY)) == -1) {
-      fprintf ( stderr, "%s -- ERROR opening %s\n\n", cmd, LVM_DEV);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR opening device special \"%s\"\n\n",
+                        cmd, LVM_DEV);
+      return LVM_ELVMCHANGE_OPEN;
    }
 
    if ( opt_r > 0) {
@@ -164,7 +170,7 @@ int main ( int argc, char **argv) {
                                cmd, read_ahead);
       if ( ( ret = ioctl ( lvm, BLKRASET, read_ahead)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d doing BLKRASET ioctl\n", cmd, ret);
-         ret = -2;
+         ret = LVM_ELVMCHANGE_READ_AHEAD;
       } else {
          printf ( "%s -- LVM global read ahead sectors set\n", cmd);
       }
@@ -188,7 +194,7 @@ int main ( int argc, char **argv) {
          if ( opt_v > 0) printf ( "%s -- resetting LVM ...\n", cmd);
          if ( ( ret = ioctl ( lvm, LVM_RESET, &cmd)) < 0) {
             fprintf ( stderr, "%s -- ERROR %d doing reset ioctl\n", cmd, ret);
-            ret = -3;
+            ret = LVM_ELVMCHANGE_RESET;
          } else {
             printf ( "%s -- LVM has been reset\n", cmd);
          }
