@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lvreduce.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * April 1997
- * April-June 1998
+ * April-June,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,8 @@
  *    05/16/1998 - added lvmtab checking
  *    06/12/1998 - enhanced checking numbers in option arguments
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    06/29/1998 - corrected size error message
+ *    09/04/1998 - corrected some messages
  *
  */
 
@@ -54,6 +58,7 @@ int main ( int argc, char **argv) {
    int sign = 0;
    int size_rest = 0;
    uint new_size = 0;
+   char *dummy = NULL;
 #ifdef DEBUG
    char *options = "A:dfh?l:L:yv";
 #else
@@ -73,23 +78,23 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -97,8 +102,8 @@ int main ( int argc, char **argv) {
 
          case 'f':
             if ( opt_f > 0) {
-               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- f option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_f++;
             break;
@@ -125,12 +130,12 @@ int main ( int argc, char **argv) {
 
          case 'l':
             if ( opt_L > 0) {
-               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- L option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( opt_l > 0) {
-               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- l option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             sign = 0;
             if ( *optarg == '-') { sign = -1; optarg++;}
@@ -140,18 +145,18 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'L':
             if ( opt_l > 0) {
-               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- l option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( opt_L > 0) {
-               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- L option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             sign = 0;
             if ( *optarg == '-') { sign = -1; optarg++;}
@@ -161,14 +166,14 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option L argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -176,72 +181,73 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
-
-   if ( optind < argc && *argv[optind] == '-') {
-      fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
-      return 1;
-   }
+   CMD_MINUS_CHK
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a logical volume name\n\n", cmd);
-      return 1;
+      return LVM_ELVREDUCE_LV_MISSING;
    }
    lv_name = argv[optind];
 
    if ( lv_check_name ( lv_name) < 0) {
-      fprintf ( stderr, "%s -- no valid logical volume name %s\n\n",
+      fprintf ( stderr, "%s -- no valid logical volume name \"%s\"\n\n",
                 cmd, lv_name);
-      return 1;
+      return LVM_ELVREDUCE_LV_NAME;
    }
 
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
 
    vg_name = vg_name_of_lv ( lv_name);
-   if ( opt_v > 0) printf ( "%s -- checking for active volume group %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking for active volume group \"%s\"\n",
                             cmd, vg_name);
    if ( vg_check_active ( vg_name) == FALSE) {
-      fprintf ( stderr, "%s -- can't reduce logical volume:"
-                        " %s isn't active\n\n", cmd, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- can't reduce logical volume "
+                        " in inactive volume group \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_ELVREDUCE_VG_CHECK_ACTIVE;
    }
 
    if ( opt_v > 0) printf ( "%s -- getting volume group status from "
                             "the VGDA in kernel\n", cmd);
    if ( vg_status ( vg_name, &vg) != 0) {
       fprintf ( stderr, "%s -- can't reduce logical volume:"
-                        " couldn't get status of %s\n\n", cmd, vg_name);
-      return 1;
+                        " couldn't get status of volume group \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_ELVREDUCE_VG_STATUS;
    }
 
    /* does VG exist? */
-   if ( opt_v > 0) printf ( "%s -- checking existence of logical volume %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking existence of "
+                            "logical volume \"%s\"\n",
                             cmd, lv_name);
    if ( lvm_tab_lv_check_exist ( lv_name) == FALSE) {
-      fprintf ( stderr, "%s -- logical volume %s doesn't exist\n\n",
+      fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
                 cmd, lv_name);
-      return 1;
+      return LVM_ELVREDUCE_LV_CHECK_EXIST;
    }
 
-   if ( opt_v > 0) printf ( "%s -- reading volume group data of %s "
+   if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
                             "from disk(s)\n",
                             cmd, vg_name);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
       fprintf ( stderr, "%s -- can't reduce logical volume:"
-                        " couldn't read data of %s\n\n", cmd, vg_name);
-      return 1;
+                        " couldn't read volume group data of \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_ELVREDUCE_VG_READ;
    }
 
-   if ( opt_v > 0) printf ( "%s -- getting index of %s in %s\n",
+   if ( opt_v > 0) printf ( "%s -- getting index of logical volume \"%s\" "
+                            "in volume group \"%s\"\n",
                             cmd, lv_name, vg_name);
    if ( ( l = lv_get_index_by_name ( vg, lv_name)) < 0) {
-      fprintf ( stderr, "%s -- can't reduce logical volume:"
-                        " couldn't get index of %s\n\n", cmd, lv_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR: can't reduce logical volume \"%s\" "
+                        "without getting it's index\n\n", cmd, lv_name);
+      return LVM_ELVREDUCE_LV_GET_INDEX;
    }
 
    if ( opt_L > 0 || opt_l > 0) {
@@ -249,13 +255,13 @@ int main ( int argc, char **argv) {
       else            new_size *= vg->pe_size;
       if ( new_size % vg->pe_size > 0) {
          new_size += vg->pe_size;
-         printf ( "%s -- rounding size to physical extend boundary\n", cmd);
+         printf ( "%s -- rounding size to physical extent boundary\n", cmd);
       }
       new_size = new_size / vg->pe_size * vg->pe_size;
       if ( new_size >= vg->lv[l]->lv_size) {
-         fprintf ( stderr, "%s -- new size is larger than old one\n\n",
+         fprintf ( stderr, "%s -- new size is not smaller than old one\n\n",
                    cmd);
-         return 1;
+         return LVM_ELVREDUCE_LV_SIZE;
       }
       if ( sign != 0) new_size = vg->lv[l]->lv_size - new_size;
       size_rest = new_size %
@@ -268,39 +274,44 @@ int main ( int argc, char **argv) {
       }
    } else {
       fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
-   if ( opt_v > 0) printf ( "%s -- getting status of %s from VGDA in kernel\n",
+   if ( opt_v > 0) printf ( "%s -- getting status of logical volume \"%s\" "
+                            "from kernel\n",
                             cmd, lv_name);
    if ( lv_status_byname ( vg_name, lv_name, &lv) == 0 &&
         lv->lv_status == LV_ACTIVE) {
       printf ( "%s -- WARNING: reducing active", cmd);
       if ( lv->lv_open > 0) printf ( " and open");
-      printf ( " logical volume\n"
-               "%s -- THIS MAY DESTROY YOUR DATA\n", cmd);
+      printf ( " logical volume to %s\n"
+               "%s -- THIS MAY DESTROY YOUR DATA\n",
+               ( dummy = lvm_show_size ( new_size / 2, SHORT)), cmd);
+      free ( dummy); dummy = NULL;
    }
 
-   if ( opt_v > 0) printf ( "%s -- setting up logical volume %s for reduce\n",
+   if ( opt_v > 0) printf ( "%s -- setting up logical volume \"%s\" "
+                            "for reduce\n",
                             cmd, lv_name);
    if ( ( ret = lv_setup_for_reduce ( vg_name, vg, lv_name, new_size)) != 0) {
-      fprintf ( stderr, "%s -- ERROR setting %s up for reduce\n\n",
+      fprintf ( stderr, "%s -- ERROR setting logical volume \"%s\" up "
+                        "for reduce\n\n",
                         cmd, lv_name);
-      return 1;
+      return LVM_ELVREDUCE_LV_SETUP;
    }
 
    if ( opt_f == 0) {
       c = 0;
       while ( c != 'n' && c != 'y') {
          if ( c == '\n' || c == 0)
-            printf ( "%s -- do you really want to reduce %s? [y/n]: ",
+            printf ( "%s -- do you really want to reduce \"%s\"? [y/n]: ",
                      cmd, lv_name);
          c = tolower ( getchar ());
       }
       c1 = c;
       while ( c != '\n') c = tolower ( getchar ());
       if ( c1 == 'n') {
-         printf ( "%s -- %s not reduced\n\n", cmd, lv_name);
+         printf ( "%s -- logical volume \"%s\" not reduced\n\n", cmd, lv_name);
          return 0;
       }
    }
@@ -312,32 +323,34 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- creating reduced logical volume "
                             "VGDA in kernel\n", cmd);
    if ( ( ret = lv_reduce ( vg, vg->lv[l], lv_name)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d reducing %s in kernel\n\n", 
-                cmd, ret, lv_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d reducing logical volume \"%s\" "
+                        "in kernel\n\n", 
+                        cmd, ret, lv_name);
+      return LVM_ELVREDUCE_LV_REDUCE;
    }
 
    if ( opt_v > 0) printf ( "%s -- storing reduced logical volume "
                             "data on disk(s)\n", cmd);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of %s"
-                        " on disks\n\n", 
-                cmd, ret, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d: storing data of volume group \"%s\" "
+                        "on disks\n\n", 
+                        cmd, ret, vg_name);
+      return LVM_ELVREDUCE_VG_WRITE;
    }
 
 
    if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
    if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
         opt_A > 0) {
-      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+      printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+               cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
    }
 
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
-   printf ( "%s -- logical volume %s successfully reduced\n\n",
+   printf ( "%s -- logical volume \"%s\" successfully reduced\n\n",
              cmd, lv_name);
 
    return 0;
