@@ -1,8 +1,11 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lvremove.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * April-June 1998
+ * April-June,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +30,8 @@
  *    04/30/1998 - changed to lv_status_byname
  *    05/16/1998 - added lvmtab checking
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/04/1998 - corrected some messages
+ *    01/12/1999 - don't continue with next LV in case of an erro with one LV
  *
  */
 
@@ -66,23 +71,23 @@ int main ( int argc, char **argv)
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -108,16 +113,16 @@ int main ( int argc, char **argv)
 
          case 'f':
             if ( opt_f > 0) {
-               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- f option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_f++;
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -125,13 +130,15 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a logical volume path\n\n", cmd);
-      return 1;
+      return LVM_ELVREMOVE_LV_MISSING;
    }
 
    LVM_LOCK ( 0);
@@ -140,92 +147,96 @@ int main ( int argc, char **argv)
    for ( ; optind < argc; optind++) {
       lv_name = argv[optind];
    
-      if ( opt_v > 0) printf ( "%s -- checking logical volume path %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking logical volume name \"%s\"\n",
                                cmd, lv_name);
       if ( ( ret = lv_check_name ( lv_name)) < 0) {
          fprintf ( stderr, "%s -- ERROR: no valid logical volume "
-                           "name %s\n", cmd, lv_name);
-         continue;
+                           "name \"%s\"\n", cmd, lv_name);
+         return LVM_ELVREMOVE_LV_CHECK_NAME;
       }
    
       /* does VG exist? */
       vg_name = vg_name_of_lv ( lv_name);
 
-      if ( opt_v > 0) printf ( "%s -- checking existence of volume group %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking existence of volume "
+                               "group \"%s\"\n",
                                cmd, vg_name);
       if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
-         fprintf ( stderr, "%s -- volume group %s doesn't exist\n",
+         fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
                    cmd, vg_name);
-         continue;
+         return LVM_ELVREMOVE_VG_CHECK_EXIST;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking volume group %s activity\n",
+      if ( opt_v > 0) printf ( "%s -- checking for active volume group "
+                               "\"%s\"\n",
                                cmd, vg_name);
       if ( vg_check_active ( vg_name) == FALSE) {
          fprintf ( stderr, "%s -- can't remove logical volume in "
-                           "inactive volume group %s\n", cmd, vg_name);
-         continue;
+                           "inactive volume group \"%s\"\n", cmd, vg_name);
+         return LVM_ELVREMOVE_VG_CHECK_ACTIVE;
       }
    
-      if ( opt_v > 0) printf ( "%s -- getting status of volume group %s "
-                               "from VGDA in kernel\n", cmd, vg_name);
+      if ( opt_v > 0) printf ( "%s -- getting status of volume group \"%s\" "
+                               "from kernel\n", cmd, vg_name);
       if ( vg_status ( vg_name, &vg) < 0) {
          fprintf ( stderr, "%s -- can't remove logical volume in inactive"
-                           " volume group %s\n", cmd, vg_name);
-         continue;
+                           " volume group \"%s\"\n", cmd, vg_name);
+         return LVM_ELVREMOVE_VG_STATUS;
       }
    
       if ( opt_v > 0) printf ( "%s -- getting status of logical volume "
                                "%s from VGDA in kernel\n", cmd, lv_name);
       if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
          if ( ret == -ENXIO) {
-            fprintf ( stderr, "%s -- logical volume %s doesn't exist\n",
+            fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n",
                               cmd, lv_name);
          } else {
             fprintf ( stderr, "%s -- ERROR %d getting status"
-                              " of logical volume %s\n",
+                              " of logical volume \"%s\"\n",
                               cmd, ret, lv_name);
          }
-         continue;
+         return LVM_ELVREMOVE_LV_STATUS;
       }
    
       if ( lv->lv_open > 0) {
-         fprintf ( stderr, "%s -- can't remove open logical volume %s\n",
+         fprintf ( stderr, "%s -- can't remove open logical volume \"%s\"\n",
                            cmd, lv_name);
-         continue;
+         return LVM_ELVREMOVE_LV_OPEN;
       }
    
-      if ( opt_v > 0) printf ( "%s -- reading volume group data of %s "
+      if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
                                "from disk(s)\n", cmd, lv_name);
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d reading volume group data of %s\n",
+         fprintf ( stderr, "%s -- ERROR %d reading data of volume "
+                           "group \"%s\"\n",
                            cmd, ret, vg_name);
-         continue;
+         return LVM_ELVREMOVE_VG_READ;
       }
 
       if ( opt_f == 0) {
          c = 0;
          while ( c != 'y' && c != 'n') {
             if ( c == '\n' || c == 0)
-               printf ( "%s -- do you really want to remove %s? [y/n]: ",
+               printf ( "%s -- do you really want to remove \"%s\"? [y/n]: ",
                         cmd, lv_name);
             c = tolower ( getchar ());
          }
          c1 = c;
          while ( c != '\n') c = tolower ( getchar ());
          if ( c1 == 'n') {
-            printf ( "%s -- %s not removed\n", cmd, lv_name);
+            printf ( "%s -- logical volume \"%s\" not removed\n", cmd, lv_name);
             continue;
          }
       }
 
       /* release it in structures */
-      if ( opt_v > 0) printf ( "%s -- releasing logical volume %s\n",
+      if ( opt_v > 0) printf ( "%s -- releasing logical volume \"%s\"\n",
                                cmd, lv_name);
       if ( ( ret = lv_release ( vg, lv_name)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing %s\n", 
-                   cmd, ret, lv_name);
-         continue;
+         fprintf ( stderr, "%s -- ERROR %d releasing logical "
+                           "volume \"%s\"\n", 
+                           cmd, ret, lv_name);
+         return LVM_ELVREMOVE_LV_RELEASE;
       }
    
    
@@ -235,40 +246,40 @@ int main ( int argc, char **argv)
       if ( opt_v > 0) printf ( "%s -- removing logical volume from VGDA "
                                "in kernel\n", cmd);
       if ( ( ret = lv_remove ( vg, lv, lv_name)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing %s\n", 
-                   cmd, ret, lv_name);
-         lvm_interrupt ();
-         continue;
+         fprintf ( stderr, "%s -- ERROR %d removing logical "
+                           "volume \"%s\"\n", 
+                           cmd, ret, lv_name);
+         return -LVM_ELVREMOVE_LV_REMOVE;
       }
    
-      if ( opt_v > 0) printf ( "%s -- unlinking special file %s\n",
+      if ( opt_v > 0) printf ( "%s -- unlinking special file \"%s\"\n",
          cmd, lv_name);
       if ( unlink ( lv_name) == -1)
-         fprintf ( stderr, "%s -- ERROR %d unlinking %s\n",
+         fprintf ( stderr, "%s -- ERROR %d unlinking special file \"%s\"\n",
                            cmd, errno, lv_name);
    
       /* store it on disks */
       if ( opt_v > 0) printf ( "%s -- storing volume group data on disk(s)\n",
                                cmd);
       if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d storing volume group data of %s"
-                           " on disks\n\n", 
-                   cmd, ret, vg_name);
-         lvm_interrupt ();
-         return 1;
+         fprintf ( stderr, "%s -- ERROR %d storing data of volume group \"%s\" "
+                           "on disks\n\n", 
+                           cmd, ret, vg_name);
+         return LVM_ELVREMOVE_VG_WRITE;
       }
 
 
       if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
       if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
            opt_A > 0) {
-         printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+         printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+                  cmd, vg_name);
          vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
       }
 
       lvm_interrupt ();
 
-      printf ( "%s -- logical volume %s successfully removed\n",
+      printf ( "%s -- logical volume \"%s\" successfully removed\n",
                cmd, lv_name);
       if ( argc - optind > 1 && opt_v > 0) printf ( "\n");
    }
