@@ -1,7 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/lvrename.c
  *
- * April-June 1998
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ *
+ * April-June,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +27,10 @@
  * Changelog
  *
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/04/1998 - corrected some messages
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
+ *    01/28/1999 - fixed command line bug
+ *               - fixed logical volume path too long in command line
  *
  */
 
@@ -67,23 +74,23 @@ int main ( int argc, char** argv) {
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -110,8 +117,8 @@ int main ( int argc, char** argv) {
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -119,25 +126,53 @@ int main ( int argc, char** argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    /* check alternate command line syntax */
-   par_count = argc -optind;
+   par_count = argc - optind;
    if ( par_count != 2 && par_count != 3) {
       fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( par_count == 2) {
-      lv_name_old = argv[optind];
-      lv_name_new = argv[optind+1];
+      if ( strlen ( lv_name_old = argv[optind]) > NAME_LEN - 1) {
+         fprintf ( stderr, "%s -- the old logical volume path is longer than "
+                           "the maximum of %d!\n\n",
+                           cmd, NAME_LEN - 1);
+         return LVM_ELVRENAME_LV_NAME;
+      }
+      if ( strlen ( lv_name_new = argv[optind+1]) > NAME_LEN - 1) {
+         fprintf ( stderr, "%s -- the old logical volume path is longer than "
+                           "the maximum of %d!\n\n",
+                           cmd, NAME_LEN - 1);
+         return LVM_ELVRENAME_LV_NAME;
+      }
    } else {
-      if ( strncmp ( argv[optind], "/dev/", 5) == 0) prefix = "";
-      else                                           prefix = "/dev/";
+      if ( strncmp ( argv[optind], LVM_DIR_PREFIX,
+                     strlen ( LVM_DIR_PREFIX)) == 0) prefix = "";
+      else                                           prefix = LVM_DIR_PREFIX;
+
+      if ( strlen ( prefix) + strlen ( argv[optind]) +
+           strlen ( argv[optind+1] + 1) > NAME_LEN - 1) {
+         fprintf ( stderr, "%s -- the old logical volume path is longer than "
+                           "the maximum of %d!\n\n",
+                           cmd, sizeof ( lv_name_old_buf));
+         return LVM_ELVRENAME_LV_NAME;
+      }
       sprintf ( lv_name_old_buf, "%s%s/%s%c",
                 prefix, argv[optind], argv[optind+1], 0);
+      if ( strlen ( prefix) + strlen ( argv[optind]) +
+           strlen ( argv[optind+2] + 1) > NAME_LEN - 1) {
+         fprintf ( stderr, "%s -- the new logical volume path is longer than "
+                           "the maximum of %d!\n\n",
+                           cmd, sizeof ( lv_name_new_buf));
+         return LVM_ELVRENAME_LV_NAME;
+      }
       sprintf ( lv_name_new_buf, "%s%s/%s%c",
                 prefix, argv[optind], argv[optind+2], 0);
       lv_name_old = lv_name_old_buf;
@@ -146,92 +181,94 @@ int main ( int argc, char** argv) {
 
    if ( opt_v > 0) printf ( "%s -- checking old logical volume name\n", cmd);
    if ( lv_check_name ( lv_name_old) < 0) {
-      fprintf ( stderr, "%s -- invalid logical volume name %s\n\n",
+      fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
                         cmd, lv_name_old);
-      return 1;
-   }
-
-   if ( opt_v > 0) printf ( "%s -- checking new logical volume name\n", cmd);
-   if ( lv_check_name ( lv_name_new) < 0) {
-      fprintf ( stderr, "%s -- invalid logical volume name %s\n\n",
-                        cmd, lv_name_new);
-      return 1;
-   }
-
-   strcpy ( vg_name, vg_name_of_lv ( lv_name_old));
-   if ( lvm_tab_vg_check_exist ( vg_name, &vg) != TRUE) {
-      fprintf ( stderr, "%s -- volume group %s doesn't exist\n\n",
-                        cmd, vg_name);
-      return 1;
-   }
-
-   if ( vg_check_active ( vg_name) != TRUE) {
-      fprintf ( stderr, "%s -- volume group %s is inactive\n\n", cmd, vg_name);
-      return 1;
-   }
-
-   if ( lstat ( lv_name_new, &stat_buf) != -1) {
-      lvm_show_filetype ( stat_buf.st_mode, lv_name_new);
-      return 1;
+      return LVM_ELVRENAME_LV_NAME;
    }
 
    if ( opt_v > 0) printf ( "%s -- checking for existence of old "
                             "logical volume\n",
                             cmd);
    if ( lvm_tab_lv_check_exist ( lv_name_old) != TRUE) {
-      fprintf ( stderr, "%s -- logical volume %s doesn't exist\n\n",
+      fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
                         cmd, lv_name_old);
-      return 1;
-   }
-
-   if ( strcmp ( vg_name, vg_name_of_lv ( lv_name_new)) != 0) {
-      fprintf ( stderr, "%s -- volume group names are different\n\n", cmd);
-      return 1;
-   }
-
-   if ( opt_v > 0) printf ( "%s -- checking for nonexistence of new "
-                            "logical volume\n", cmd);
-   if ( lvm_tab_lv_check_exist ( lv_name_new) == TRUE) {
-      fprintf ( stderr, "%s -- logical volume %s already exists\n\n",
-                        cmd, lv_name_new);
-      return 1;
+      return LVM_ELVRENAME_LV_CHECK_EXIST_OLD;
    }
 
    if ( opt_v > 0) printf ( "%s -- checking active status for existing "
                             "logical volume\n", cmd);
    if ( lv_check_active ( vg_name, lv_name_old) == TRUE) {
-      fprintf ( stderr, "%s -- can't rename active logical volume %s\n\n",
+      fprintf ( stderr, "%s -- can't rename active logical volume \"%s\"\n\n",
                         cmd, lv_name_old);
-      return 1;
+      return LVM_ELVRENAME_LV_CHECK_ACTIVE;
+   }
+
+   if ( opt_v > 0) printf ( "%s -- checking new logical volume name\n", cmd);
+   if ( lv_check_name ( lv_name_new) < 0) {
+      fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
+                        cmd, lv_name_new);
+      return LVM_ELVRENAME_LV_NAME;
+   }
+
+   if ( opt_v > 0) printf ( "%s -- checking for nonexistence of new "
+                            "logical volume\n", cmd);
+   if ( lvm_tab_lv_check_exist ( lv_name_new) == TRUE) {
+      fprintf ( stderr, "%s -- logical volume \"%s\" already exists\n\n",
+                        cmd, lv_name_new);
+      return LVM_ELVRENAME_LV_CHECK_EXIST_NEW;
+   }
+
+   if ( lstat ( lv_name_new, &stat_buf) != -1) {
+      lvm_show_filetype ( stat_buf.st_mode, lv_name_new);
+      return LVM_ELVRENAME_LSTAT;
+   }
+
+   strcpy ( vg_name, vg_name_of_lv ( lv_name_old));
+   if ( lvm_tab_vg_check_exist ( vg_name, &vg) != TRUE) {
+      fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n\n",
+                        cmd, vg_name);
+      return LVM_ELVRENAME_VG_CHECK_EXIST;
+   }
+
+   if ( vg_check_active ( vg_name) != TRUE) {
+      fprintf ( stderr, "%s -- volume group \"%s\" is inactive\n\n",
+                        cmd, vg_name);
+      return LVM_ELVRENAME_VG_CHECK_ACTIVE;
+   }
+
+   if ( strcmp ( vg_name, vg_name_of_lv ( lv_name_new)) != 0) {
+      fprintf ( stderr, "%s -- volume group names are different\n\n", cmd);
+      return LVM_ELVRENAME_VG_NAME_DIFFER;
    }
 
 
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
 
-   if ( opt_v > 0) printf ( "%s -- reading volume group data of %s\n",
+   if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\"\n",
                             cmd, vg_name);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: couldn't get volume "
-                        "group data of %s\n\n",
+      fprintf ( stderr, "%s -- ERROR %d: couldn't get data of volume "
+                        "group \"%s\"\n\n",
                         cmd, ret, vg_name);
-      return 1;
+      return LVM_ELVRENAME_VG_READ;
    }
 
    if ( ( l = lv_get_index_by_name ( vg, lv_name_old)) < 0) {
       fprintf ( stderr, "%s -- ERROR: couldn't get index of logical "
-                        "volume %s\n\n", cmd, lv_name_old);
-      return 1;
+                        "volume \"%s\"\n\n", cmd, lv_name_old);
+      return LVM_ELVRENAME_LV_GET_INDEX;
    }
 
 
    /* remove it in kernel */
    if ( opt_v > 0) printf ( "%s -- removing old logical volume VGDA "
                             "in kernel\n", cmd);
-   if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name_old)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d removing VGDA for %s "
-                        "from kernel\n\n", cmd, ret, lv_name_old);
-      return 1;
+   if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name_old)) < 0) {
+      fprintf ( stderr, "%s -- ERROR %d removing VGDA for logical volume "
+                        "\"%s\" from kernel\n\n",
+                        cmd, ret, lv_name_old);
+      return LVM_ELVRENAME_LV_REMOVE_OLD;
    }
 
    /* change the logical volume name */
@@ -241,24 +278,26 @@ int main ( int argc, char** argv) {
    if ( opt_v > 0) printf ( "%s -- creating new logical volume VGDA "
                             "in kernel\n", cmd);
    if ( ( ret = lv_create ( vg, vg->lv[l], lv_name_new)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating VGDA for %s in kernel\n\n",
-                cmd, ret, lv_name_new);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d creating VGDA for logical volume "
+                        "\"%s\" in kernel\n\n",
+                        cmd, ret, lv_name_new);
+      return LVM_ELVRENAME_LV_CREATE;
    }
 
    /* store it on disks */
    if ( opt_v > 0) printf ( "%s -- storing logical volume VGDA on disk(s)\n",
                             cmd);
-   if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of %s"
+   if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
+      fprintf ( stderr, "%s -- ERROR %d: storing data of volume group \"%s\""
                         " on disks\n\n", 
                 cmd, ret, vg_name);
       fprintf ( stderr, "%s -- removing new logical volume VGDA in kernel\n",
                 cmd);
-      if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name_new)) != 0)
-         fprintf ( stderr, "%s -- ERROR %d removing VGDA for %s "
-                           "from kernel\n\n", cmd, ret, lv_name_new);
-      return 1;
+      if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name_new)) < 0)
+         fprintf ( stderr, "%s -- ERROR %d removing VGDA for logical volume "
+                           "\"%s\" from kernel\n\n",
+                           cmd, ret, lv_name_new);
+      return LVM_ELVRENAME_VG_WRITE;
    }
 
    unlink ( lv_name_old);
@@ -268,20 +307,23 @@ int main ( int argc, char** argv) {
                                  vg->vg_number,
                                  vg->lv[l]->lv_number + 1)) < 0) {
       if ( ret == -LVM_ELV_CREATE_MODE_MKNOD) {
-         fprintf ( stderr, "%s -- logical volume node %s allready existent\n\n",
-                   cmd, lv_name_new);
+         fprintf ( stderr, "%s -- logical volume node \"%s\" already "
+                           "exists\n\n",
+                           cmd, lv_name_new);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d creating logical volume node %s\n\n",
-                   cmd, ret, lv_name_new);
+         fprintf ( stderr, "%s -- ERROR %d creating logical volume "
+                           "node \"%s\"\n\n",
+                           cmd, ret, lv_name_new);
       }
-      return 1;
+      return LVM_ELVRENAME_LV_CREATE_NODE;
    }
 
 
    if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
    if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
         opt_A > 0) {
-      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+      printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+               cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
    }
 
@@ -289,7 +331,7 @@ int main ( int argc, char** argv) {
    LVM_UNLOCK ( 0);
 
 
-   printf ( "%s -- logical volume %s successfully renamed to %s\n\n",
+   printf ( "%s -- logical volume \"%s\" successfully renamed to \"%s\"\n\n",
             cmd, lv_name_old, lv_name_new);
    return 0;
 }
