@@ -1,7 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/lvscan.c
  *
- * May,June 1998
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ *
+ * May,June,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +27,8 @@
  * Changelog
  *
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/04/1998 - corrected some messages
+ *    01/13/1999 - aborted in case of error
  *
  */
 
@@ -64,8 +69,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -73,8 +78,8 @@ int main ( int argc, char **argv)
 
          case 'D':
             if ( opt_D > 0) {
-               fprintf ( stderr, "%s -- D option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- D option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_D++;
             break;
@@ -97,8 +102,8 @@ int main ( int argc, char **argv)
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -106,14 +111,16 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( optind != argc) {
       fprintf ( stderr, "%s -- no additional command line arguments "
                         "allowed\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    argv = lvm_tab_vg_check_exist_all_vg ();
@@ -122,7 +129,7 @@ int main ( int argc, char **argv)
 
    if ( optind == argc) {
       printf ( "%s -- no volume groups found\n\n", cmd);
-      return 1;
+      return LVM_ELVSCAN_NO_VGS;
    }
 
    if ( opt_D > 0) printf ( "%s -- reading all physical volumes "
@@ -134,38 +141,40 @@ int main ( int argc, char **argv)
    for ( ; optind < argc; optind++) {
       vg_name = argv[optind];
 
-      if ( opt_v > 0) printf ( "%s -- checking volume group name %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
                                cmd, vg_name);
       if ( vg_check_name ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- %s is an invalid volume group name\n",
+         fprintf ( stderr, "%s -- \"%s\" is an invalid volume group name\n",
                    cmd, vg_name);
-         continue;
+         return LVM_ELVSCAN_VG_CHECK_NAME;
       }
 
-      if ( opt_v > 0) printf ( "%s -- checking volume group %s existence\n",
+      if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" existence\n",
                                cmd, vg_name);
       if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) == TRUE ||
            ret == -LVM_EVG_READ_VG_EXPORTED ||
            ret == -LVM_EVG_READ_LVM_STRUCT_VERSION) {
          if ( ret == -LVM_EVG_READ_VG_EXPORTED) {
-            printf ( "%s -- WARNING: volume group %s is exported\n",
+            printf ( "%s -- WARNING: volume group \"%s\" is exported\n",
                      cmd, vg_name);
          }
          if ( opt_D == 0) {
             if ( opt_v > 0) printf ( "%s -- checking volume group "
-                                     "%s activity\n", cmd, vg_name);
+                                     "\"%s\" activity\n", cmd, vg_name);
             if ( vg_check_active ( vg_name) == TRUE) {
-               if ( opt_v > 0) printf ( "%s -- getting VGDA of %s from "
-                                        "kernel\n", cmd, vg_name);
+               if ( opt_v > 0) printf ( "%s -- getting VGDA of volume "
+                                        "group \"%s\" from kernel\n",
+                                        cmd, vg_name);
                if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg,
                                                        TRUE)) < 0) {
                   fprintf ( stderr,
-                            "%s -- ERROR %d getting status of %s\n",
+                            "%s -- ERROR %d getting status of volume "
+                            " group \"%s\"\n",
                              cmd, ret, vg_name);
-                  continue;
+                  return LVM_ELVSCAN_VG_STATUS;
                }
             } else {
-               printf ( "%s -- %s is NOT active; try -D\n",
+               printf ( "%s -- volume group \"%s\" is NOT active; try -D\n",
                         cmd, vg_name);
                continue;
             }
@@ -177,13 +186,13 @@ int main ( int argc, char **argv)
                exit_code = 1;
                if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION) {
                   fprintf ( stderr,
-                            "%s -- can't show volume group %s with "
+                            "%s -- can't show volume group \"%s\" with "
                             "invalid physical volume version\n",
                              cmd, vg_name);
                } else {
                   fprintf ( stderr,
-                            "%s -- ERROR %d getting data of %s form "
-                            "kernel\n",
+                            "%s -- ERROR %d getting data of volume "
+                            "group \"%s\" form kernel\n",
                             cmd, ret, vg_name);
                }
                continue;
@@ -197,7 +206,7 @@ int main ( int argc, char **argv)
                   printf ( "inactive");
                else
                   printf ( "ACTIVE  ");
-               printf ( " LV %s [%s]",
+               printf ( " \"%s\" [%s]",
                         vg->lv[l]->lv_name,
                         ( dummy = lvm_show_size ( vg->lv[l]->lv_size / 2,
                                                   SHORT)));
@@ -213,10 +222,10 @@ int main ( int argc, char **argv)
          }
       } else if ( ret == -LVM_EVG_CHECK_EXIST_PV_COUNT) {
          fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
-                           "volume group %s online\n", cmd, vg_name);
+                           "volume group \"%s\" online\n", cmd, vg_name);
          exit_code = 1;
       } else {
-         fprintf ( stderr, "%s -- volume group %s not found\n",
+         fprintf ( stderr, "%s -- volume group \"%s\" not found\n",
                            cmd, vg_name);
          exit_code = 1;
       }
