@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/pvchange.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * May,November 1997
- * March,May 1998
+ * March,May,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +35,7 @@
  *                 (LVM_LOCK called too late :-( )
  *    05/17/1998 - obsoleted physical volume name change (it's no in pv_read())
  *    05/26/1998 - fixed display bug of (not) done physical volumes
+ *    09/04/1998 - corrected some messages
  *
  */
 
@@ -75,31 +78,31 @@ int main ( int argc, char **argv)
       switch ( c) {
          case 'a':
             if ( opt_a > 0) {
-               fprintf ( stderr, "%s -- a option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- a option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_a++;
             break;
 
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -125,36 +128,38 @@ int main ( int argc, char **argv)
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
 
          case 'x':
             if ( opt_x > 0) {
-               fprintf ( stderr, "%s -- x option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- x option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_x++;
             if ( strcmp ( optarg, "y") == 0) { act = PV_ALLOCATABLE; break;}
             if ( strcmp ( optarg, "n") == 0) { act = 0; break;}
             fprintf ( stderr, "%s -- x option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
 
    if ( opt_x == 0) {
-      fprintf ( stderr, "%s -- please give x option\n\n", cmd);
-      return 1;
+      fprintf ( stderr, "%s -- please give the x option\n\n", cmd);
+      return LVM_EINVALID_CMD_LINE;
    }
 
    LVM_LOCK ( 0);
@@ -163,19 +168,19 @@ int main ( int argc, char **argv)
    if ( opt_a == 0) {
       if ( optind == argc) {
          fprintf ( stderr, "%s -- please give a physical volume path\n\n", cmd);
-         return 1;
+         return LVM_EPVCHANGE_PV_MISSING;
       }
    } else {
       if ( optind != argc) {
          fprintf ( stderr, "%s -- option a and PhysicalVolumePath are "
                            "exclusive\n\n", cmd);
-         return 1;
+         return LVM_EINVALID_CMD_LINE;
       }
       if ( opt_v > 0) printf ( "%s -- searching for physical volume names\n",
                                cmd);
       if ( ( argv = pv_find_all_pv_names ()) == NULL) {
          fprintf ( stderr, "%s -- can't find any physical volumes\n\n", cmd);
-         return 1;
+         return LVM_EPVCHANGE_PV_FIND_ALL_PV_NAMES;
       } else {
          optind = argc = 0;
          while ( argv[argc] != NULL) argc++;
@@ -187,7 +192,7 @@ int main ( int argc, char **argv)
       change_msg = FALSE;
    
       if ( pv_check_name ( pv_name) < 0) {
-         fprintf ( stderr, "%s -- physical volume name %s is invalid\n",
+         fprintf ( stderr, "%s -- physical volume name \"%s\" is invalid\n",
                            cmd, pv_name);
          continue;
       }
@@ -197,59 +202,62 @@ int main ( int argc, char **argv)
       if ( ( ret = pv_read ( pv_name, &pv, NULL)) < 0 &&
              ret != -LVM_EPV_READ_MD_DEVICE) {
          if ( ret == -LVM_EPV_READ_LVM_STRUCT_VERSION)
-            fprintf ( stderr, "%s -- physical volume %s has invalid version\n",
-                      cmd, pv_name);
+            fprintf ( stderr, "%s -- physical volume \"%s\" has an invalid "
+                              "version\n",
+                              cmd, pv_name);
          else if ( ret == -LVM_EPV_READ_OPEN)
-            fprintf ( stderr, "%s -- cannot open physical volume %s\n",
+            fprintf ( stderr, "%s -- cannot open physical volume \"%s\"\n",
                       cmd, pv_name);
          else if ( ret == -LVM_EPV_READ_ID_INVALID)
-            fprintf ( stderr, "%s -- physical volume %s has invalid identity\n",
+            fprintf ( stderr, "%s -- physical volume \"%s\" has invalid identity\n",
                       cmd, pv_name);
          else 
-            fprintf ( stderr, "%s -- ERROR %d reading physical volume %s\n",
+            fprintf ( stderr, "%s -- ERROR %d reading physical volume \"%s\"\n",
                       cmd, ret, pv_name);
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking physical volume %s "
+      if ( opt_v > 0) printf ( "%s -- checking physical volume \"%s\" "
                                "consistency\n", cmd, pv_name);
       if ( ( ret = pv_check_consistency ( pv)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d checking consistency of "
-                           "physical volume %s\n\n", cmd, ret, pv_name);
+                           "physical volume \"%s\"\n\n", cmd, ret, pv_name);
          continue;
       }
    
       if ( opt_v > 0) printf ( "%s -- finding volume group of physical "
-                               "volume %s\n", cmd, pv_name);
+                               "volume \"%s\"\n", cmd, pv_name);
       if ( pv_find_vg ( pv_name, &vg_name) == FALSE) {
-         printf ( "%s -- unable to find volume group of %s (VG not active?)\n",
+         printf ( "%s -- unable to find volume group of \"%s\" "
+                  "(VG not active?)\n",
                    cmd, pv_name);
          doit = FALSE;
+         continue;
       } else back_it_up = doit = TRUE;
    
       /* change allocatability for a PV */
       if ( opt_x > 0) {
          if ( act == PV_ALLOCATABLE &&
               ( pv->pv_allocatable & PV_ALLOCATABLE)) {
-            fprintf ( stderr, "%s -- physical volume %s is yet allocatable\n",
+            fprintf ( stderr, "%s -- physical volume \"%s\" is allocatable\n",
                       cmd, pv_name);
             not_done++;
             continue;
          } else change_msg = TRUE;
    
          if ( act == 0 && ! ( pv->pv_allocatable & PV_ALLOCATABLE)) {
-            fprintf ( stderr, "%s -- physical volume %s is yet "
+            fprintf ( stderr, "%s -- physical volume \"%s\" is "
                               "unallocatable\n", cmd, pv_name);
             not_done++;
             continue;
          } else change_msg = TRUE;
    
          if ( act == PV_ALLOCATABLE) {
-            if ( opt_v > 0) printf ( "%s -- setting physical volume %s "
+            if ( opt_v > 0) printf ( "%s -- setting physical volume \"%s\" "
                                      "allocatable\n", cmd, pv_name);
             pv->pv_allocatable |= PV_ALLOCATABLE;
          } else {
-            if ( opt_v > 0) printf ( "%s -- setting physical volume %s "
+            if ( opt_v > 0) printf ( "%s -- setting physical volume \"%s\" "
                                      "NOT allocatable\n", cmd, pv_name);
             pv->pv_allocatable &= ~PV_ALLOCATABLE;
          }
@@ -259,7 +267,7 @@ int main ( int argc, char **argv)
    
       if ( doit == TRUE) {
          lvm_dont_interrupt ( 0);
-         if ( opt_v > 0) printf ( "%s -- checking physical volume %s "
+         if ( opt_v > 0) printf ( "%s -- checking physical volume \"%s\" "
                                   "activity\n", cmd, pv_name);
          if ( pv_check_active ( vg_name, pv_name) == TRUE) {
             if ( opt_v > 0) printf ( "%s -- changing physical volume "
@@ -267,22 +275,24 @@ int main ( int argc, char **argv)
                                      cmd, pv_name);
             if ( ( ret = pv_change ( vg_name, pv)) != 0) {
                fprintf ( stderr, "%s -- ERROR %d: unable to change physical "
-                                 "volume %s in kernel\n",
-                         cmd, ret, pv_name);
-               return 1;
+                                 "volume \"%s\" in kernel\n",
+                                 cmd, ret, pv_name);
+               return LVM_EPVCHANGE_PV_CHANGE;
             }
-         }
+         } else if ( opt_v > 0) printf ( "%s -- physical volume \"%s\" "
+                                         "inactive\n",
+                                         cmd, pv_name);
    
-         if ( opt_v > 0) printf ( "%s -- storing physical volume %s\n",
-                               cmd, pv_name);
+         if ( opt_v > 0) printf ( "%s -- storing physical volume \"%s\"\n",
+                                  cmd, pv_name);
          if ( ( ret = pv_write ( pv_name, pv)) != 0) {
             fprintf ( stderr, "%s -- ERROR %d: unable to store physical "
-                              "volume %s on disk\n",
+                              "volume \"%s\"\n",
                               cmd, ret, pv_name);
-            return 1;
+            return LVM_EPVCHANGE_PV_WRITE;
          }
 
-         printf ( "%s -- physical volume %s ", cmd, pv_name);
+         printf ( "%s -- physical volume \"%s\" ", cmd, pv_name);
          if ( change_msg == FALSE) printf ( "not ");
          printf ( "changed\n");
          lvm_interrupt ();
@@ -293,7 +303,8 @@ int main ( int argc, char **argv)
       if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
       if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, NULL) == 0 &&
            opt_A > 0) {
-         printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+         printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+                  cmd, vg_name);
          vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, NULL);
          }
    }
@@ -301,11 +312,10 @@ int main ( int argc, char **argv)
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
-   printf ( "%s -- %d physical volume", cmd, done);
-   if ( done != 1) printf ( "s");
-   printf ( " changed / %d physical volume", not_done);
-   if ( not_done != 1) printf ( "s");
-   printf ( " yet o.k.\n\n");
+   printf ( "%s -- %d physical volume%s"
+            " changed / %d physical volume%s already o.k.\n\n",
+            cmd, done, done != 1 ? "s" : "",
+            not_done, not_done != 1 ? "s" : "");
 
    return 0;
 }
