@@ -1,8 +1,11 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/pvcreate.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March, October-November 1997
- * May,June 1998
+ * May,June,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,13 +33,13 @@
  *    05/04/1998 - added multiple device support
  *    05/16/1998 - added lvmtab checking
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/04/1998 - corrected some messages
+ *    01/13/1999 - avoided error message for uninitialized physical volume
  *
  */
 
 #include <lvm_user.h>
 
-int do_pv_create ( char *, pv_t *, uint);
-
 char *cmd = NULL;
 
 #ifdef DEBUG
@@ -74,8 +77,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -83,8 +86,8 @@ int main ( int argc, char **argv)
 
          case 'f':
             if ( opt_f > 1) {
-               fprintf ( stderr, "%s -- f option yet given two times\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- f option already given two times\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_f++;
             break;
@@ -109,16 +112,16 @@ int main ( int argc, char **argv)
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
 
          case 'y':
             if ( opt_y > 1) {
-               fprintf ( stderr, "%s -- y option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- y option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_y++;
             break;
@@ -126,19 +129,21 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a physical volume path\n\n", cmd);
-      return 1;
+      return LVM_EPVCREATE_PV_MISSING;
    }
 
    if ( opt_y > 0 && opt_f == 0) {
       fprintf ( stderr, "%s -- option y can only be given with option f\n",
                          cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    LVM_LOCK ( 0);
@@ -148,10 +153,10 @@ int main ( int argc, char **argv)
       force_needed = 0;
       pv_name = argv[optind];
    
-      if ( opt_v > 0) printf ( "%s -- checking physical volume name %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking physical volume name \"%s\"\n",
                                cmd, pv_name);
       if ( pv_check_name ( pv_name) < 0) {
-         fprintf ( stderr, "%s -- invalid physical volume path %s\n",
+         fprintf ( stderr, "%s -- invalid physical volume name \"%s\"\n",
                             cmd, pv_name);
          continue;
       }
@@ -159,17 +164,19 @@ int main ( int argc, char **argv)
       if ( opt_v > 0) printf ( "%s -- getting physical volume size\n", cmd);
       if ( ( size = pv_get_size ( pv_name, &part)) < 0) {
          if ( size == -LVM_EPV_GET_SIZE_PART) {
-            fprintf ( stderr, "%s -- partition %s doesn't exist\n",
+            fprintf ( stderr, "%s -- partition \"%s\" doesn't exist\n",
                               cmd, pv_name);
             continue;
          } else if ( size == -LVM_EPV_GET_SIZE_OPEN) {
-            fprintf ( stderr, "%s -- can't open %s to get its size\n",
+            fprintf ( stderr, "%s -- can't open physical volume \"%s\" to "
+                              "get its size\n",
                               cmd, pv_name);
          } else if ( size == -LVM_EPV_GET_SIZE_NO_EXTENDED) {
-            fprintf ( stderr, "%s -- extended partition %s doesn't exist\n",
+            fprintf ( stderr, "%s -- extended partition \"%s\" doesn't exist\n",
                               cmd, pv_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d getting size of %s\n",
+            fprintf ( stderr, "%s -- ERROR %d getting size of physical "
+                              "volume \"%s\"\n",
                               cmd, size, pv_name);
          }
          continue;
@@ -177,13 +184,23 @@ int main ( int argc, char **argv)
 
       if ( opt_v > 0) printf ( "%s -- checking partition type\n", cmd);
       if ( part.sys_ind != 0) {
-         if ( part.sys_ind == DOS_EXTENDED_PARTITION ||
-              part.sys_ind == LINUX_EXTENDED_PARTITION) {
-            fprintf ( stderr, "%s -- can't use extended partition %s\n\n",
-                      cmd, pv_name);
+         if ( part.sys_ind == DOS_EXTENDED_PARTITION) {
+            fprintf ( stderr, "%s -- can't use DOS extended partition \"%s\"\n",
+                              cmd, pv_name);
+            continue;
+#ifdef LINUX_SWAP_PARTITION
+         } else if ( part.sys_ind == LINUX_SWAP_PARTITION) {
+            fprintf ( stderr, "%s -- can't use Linux SWAP partition \"%s\"\n",
+                              cmd, pv_name);
+            continue;
+#endif
+         } else if ( part.sys_ind == LINUX_EXTENDED_PARTITION) {
+            fprintf ( stderr, "%s -- can't use Linux extended partition "
+                              "\"%s\"\n",
+                              cmd, pv_name);
             continue;
          } else if ( part.sys_ind != LVM_PARTITION) {
-            fprintf ( stderr, "%s -- invalid partition type 0x%x for %s"
+            fprintf ( stderr, "%s -- invalid partition type 0x%x for \"%s\""
                               " (must be 0x%x)\n", cmd, part.sys_ind,
                               pv_name, LVM_PARTITION);
             continue;
@@ -194,26 +211,20 @@ int main ( int argc, char **argv)
            ret != -LVM_EPV_READ_MD_DEVICE) {
          if ( ret == -LVM_EPV_READ_PV_EXPORTED) {
             pv->vg_name[strlen(pv->vg_name)-strlen(EXPORTED)] = 0;
-            fprintf ( stderr, "%s -- physical volume %s belongs to "
-                              "exported volume group %s\n",
+            fprintf ( stderr, "%s -- physical volume \"%s\" belongs to "
+                              "exported volume group \"%s\"\n",
                                cmd, pv_name, pv->vg_name);
             force_needed++;
-         } else if ( ret == -LVM_EPV_READ_ID_INVALID ) {
-            fprintf ( stderr, "%s -- %s has an invalid physical "
-                              "volume identifier\n", cmd, pv_name);
-            force_needed = 0;
-         } else {
-            fprintf ( stderr, "%s -- ERROR %d: invalid physical volume %s\n",
-                               cmd, ret, pv_name);
-            force_needed = 0;
-         }
+         } else force_needed = 0;
       } else {
          if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
          if ( vg_check_name ( pv->vg_name) == 0 &&
               pv->vg_name[0] != 0) {
             if ( opt_f < 2) {
-               printf ( "%s -- can't initialize %s of volume group %s "
-                        "without -ff\n", cmd, pv_name, pv->vg_name);
+               fprintf ( stderr, "%s -- ERROR: can't initialize physical "
+                                 "volume \"%s\" of volume group \"%s\" "
+                                 "without -ff\n",
+                                 cmd, pv_name, pv->vg_name);
                continue;
             } else {
                if ( opt_y == 0) {
@@ -221,7 +232,7 @@ int main ( int argc, char **argv)
                   while ( c != 'y' && c != 'n') {
                      if ( c == '\n' || c == 0)
                         printf ( "%s -- really INITIALIZE physical "
-                                 "volume %s of %s [y/n]? ",
+                                 "volume \"%s\" of volume group \"%s\" [y/n]? ",
                                  cmd, pv_name, pv->vg_name);
                      c = tolower ( getchar ());
                   }
@@ -230,7 +241,7 @@ int main ( int argc, char **argv)
                   if ( c1 == 'y') {
                      force_needed = 0;
                   } else {
-                     printf ( "%s -- physical volume %s not initialized\n",
+                     printf ( "%s -- physical volume \"%s\" not initialized\n",
                               cmd, pv_name);
                      continue;
                   }
@@ -240,20 +251,22 @@ int main ( int argc, char **argv)
       }
    
       if ( opt_f == 0 && force_needed > 0) {
-         printf ( "%s -- need -f to initialize physical volume %s\n",
+         printf ( "%s -- need -f to initialize physical volume \"%s\"\n",
                    cmd, pv_name);
          continue;
       }
 
       if ( pv_check_active ( pv->vg_name, pv_name) == TRUE) {
-         fprintf ( stderr, "%s -- can't force create on active physical "
-                           "volume %s\n", cmd, pv_name);
+         fprintf ( stderr, "%s -- ERROR: can't force create on active physical "
+                           "volume \"%s\"\n",
+                           cmd, pv_name);
          continue;
       }
 
       if ( opt_f > 0) {
-         printf ( "%s -- forcing physical volume creation on %s", cmd, pv_name);
-         if ( pv_check_new == FALSE) printf ( " of volume group %s",
+         printf ( "%s -- WARNING: forcing physical volume creation on \"%s\"",
+                  cmd, pv_name);
+         if ( pv_check_new == FALSE) printf ( " of volume group \"%s\"",
                                               pv->vg_name);
          printf ( "\n");
       }
@@ -265,18 +278,39 @@ int main ( int argc, char **argv)
       if ( lvm_tab_vg_check_exist ( pv->vg_name, NULL) == TRUE) {
          printf ( "%s -- removing lvmtab entry\n", cmd);
          if ( ( ret = lvm_tab_vg_remove ( pv->vg_name)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d removing %s from %s\n\n",
+            fprintf ( stderr, "%s -- ERROR %d removing volume group \"%s\" "
+                              "from \"%s\"\n\n",
                               cmd, ret, pv->vg_name, LVMTAB);
-            return 1;
+            return LVM_EPVCREATE_VG_CHECK_EXIST;
          }
       }
 
       if ( opt_v > 0) printf ( "%s -- creating new physical volume\n", cmd);
-      if ( ( ret = do_pv_create ( pv_name, &pv_new, size)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d creating physical volume %s\n",
-                   cmd, ret, pv_name);
-         continue;
+
+      if ( opt_v > 0) printf ( "%s -- setting up physical volume for "
+                               "%s with %u sectors\n",
+                               cmd, pv_name, size);
+      if ( ( ret = pv_setup_for_create ( pv_name, &pv_new, size)) < 0) {
+         fprintf ( stderr, "%s -- ERROR %d setting up  physical "
+                           "volume \"%s\"\n",
+                           cmd, ret, pv_name);
+         return LVM_EPVCREATE_PV_SETUP;
       }
+   
+      lvm_dont_interrupt ( 0);
+   
+      if ( opt_v > 0) printf ( "%s -- writing physical volume data to "
+                               "disk \"%s\"\n",
+                               cmd, pv_name);
+      if ( ( ret = pv_write ( pv_name, &pv_new)) == 0) {
+         printf ( "%s -- physical volume \"%s\" successfully created\n",
+                   cmd, pv_name);
+      } else {
+         fprintf ( stderr, "%s -- ERROR %d creating physical volume \"%s\"\n",
+                   cmd, ret, pv_name);
+         return LVM_EPVCREATE_PV_WRITE;
+      }
+      lvm_interrupt ();
    }
 
    lvm_interrupt ();
@@ -286,34 +320,3 @@ int main ( int argc, char **argv)
    return 0;
 }
 
-
-int do_pv_create ( char *pv_name, pv_t *pv, uint size) {
-   int ret = 0;
-
-   if ( opt_v > 0) printf ( "%s -- creating physical volume on "
-                            "%s with %u sectors\n",
-                            cmd, pv_name, size);
-
-   if ( opt_v > 0) printf ( "%s -- setting physical volume up\n", cmd);
-   if ( ( ret = pv_setup_for_create ( pv_name, pv, size)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d setting up  physical volume on %s\n",
-                cmd, ret, pv_name);
-      return 1;
-   }
-
-   lvm_dont_interrupt ( 0);
-
-   if ( opt_v > 0) printf ( "%s -- storing physical volume data on disk\n",
-                            cmd);
-   if ( ( ret = pv_write ( pv_name, pv)) == 0) {
-      printf ( "%s -- physical volume on %s successfully created\n",
-                cmd, pv_name);
-   } else {
-      fprintf ( stderr, "%s -- ERROR %d creating physical volume on %s\n",
-                cmd, ret, pv_name);
-      return -2;
-   }
-   lvm_interrupt ();
-
-   return 0;
-}
