@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/pvdisplay.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March 1997
- * May 1998
+ * May,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +28,7 @@
  *
  *    05/05/1998 - implemented option s
  *    05/12/1998 - checked for new physical volume
+ *    09/04/1998 - corrected some messages
  *
  */
 
@@ -62,8 +65,8 @@ int main ( int argc, char **argv) {
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -88,16 +91,16 @@ int main ( int argc, char **argv) {
 
          case 's':
             if ( opt_s > 0) {
-               fprintf ( stderr, "%s -- s option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- s option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_s++;
             break;
 
          case 'v':
             if ( opt_v > 1) {
-               fprintf ( stderr, "%s -- v option yet given two times\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given two times\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -105,14 +108,16 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a physical volume path\n\n", cmd);
-      return 1;
+      return LVM_EPVDISPLAY_PV_MISSING;
    }
 
    LVM_LOCK ( 1);
@@ -121,10 +126,10 @@ int main ( int argc, char **argv) {
    for ( ; optind < argc; optind++) {
       pv_name = argv[optind];
 
-      if ( opt_v > 1) printf ( "%s -- checking physical volume path %s\n\n",
+      if ( opt_v > 1) printf ( "%s -- checking physical volume name \"%s\"\n",
                                cmd, pv_name);
       if ( pv_check_name ( pv_name) != 0) {
-         fprintf ( stderr, "%s -- invalid physical volume path %s\n\n",
+         fprintf ( stderr, "%s -- invalid physical volume name \"%s\"\n\n",
                    cmd, pv_name);
          continue;
       }
@@ -133,53 +138,58 @@ int main ( int argc, char **argv) {
          if ( opt_v > 0) printf ( "%s -- getting physical volume size\n", cmd);
          if ( ( size = pv_get_size ( pv_name, NULL)) < 0) {
             if ( size == -LVM_EPV_GET_SIZE_PART) {
-               fprintf ( stderr, "%s -- partition %s doesn't exist\n",
+               fprintf ( stderr, "%s -- partition \"%s\" doesn't exist\n",
                                  cmd, pv_name);
                continue;
             } else if ( size == -LVM_EPV_GET_SIZE_OPEN) {
-               fprintf ( stderr, "%s -- can't open %s to get its size\n",
+               fprintf ( stderr, "%s -- can't open physical volume \"%s\" "
+                                 "to get its size\n",
                                  cmd, pv_name);
             } else if ( size == -LVM_EPV_GET_SIZE_NO_EXTENDED) {
-               fprintf ( stderr, "%s -- extended partition %s doesn't exist\n",
+               fprintf ( stderr, "%s -- extended partition \"%s\" "
+                                 "doesn't exist\n",
                                  cmd, pv_name);
             } else {
-               fprintf ( stderr, "%s -- ERROR %d getting size of %s\n",
+               fprintf ( stderr, "%s -- ERROR %d getting size of physical "
+                                 "volume \"%s\"\n",
                                  cmd, size, pv_name);
             }
             continue;
          }
-         printf ( "%s -- device %s has a capacity of %s\n",
+         printf ( "%s -- device \"%s\" has a capacity of %s\n",
                   cmd, pv_name, ( dummy = lvm_show_size ( size / 2, SHORT)));
          free  ( dummy); dummy = NULL;
          continue;
       }
 
       if ( opt_v > 1) printf ( "%s -- creating physical volume device number "
-                               "from path %s\n\n",
+                               "from name \"%s\"\n",
                                cmd, pv_name);
       if ( ( ret = pv_create_kdev_t ( pv_name)) == 0) {
-         fprintf ( stderr, "%s -- stat error on %s\n\n", cmd, pv_name);
+         fprintf ( stderr, "%s -- ERROR: stat of \"%s\"\n\n", cmd, pv_name);
          continue;
       }
    
-      if ( opt_v > 1) printf ( "%s -- reading physical volume "
-                               "data for %s from disk\n", cmd, pv_name);
+      if ( opt_v > 1) printf ( "%s -- reading data of physical volume "
+                               "\"%s\" from disk\n", cmd, pv_name);
       if ( ( ret = pv_read ( pv_name, &pv, NULL)) < 0 &&
            ret != -LVM_EPV_READ_MD_DEVICE) {
          if ( ret == -LVM_EPV_READ_PV_EXPORTED ) {
             pv->vg_name[strlen(pv->vg_name)-strlen(EXPORTED)] = 0;
-            printf ( "%s -- physical volume %s of volume group %s "
+            printf ( "%s -- physical volume \"%s\" of volume group \"%s\" "
                      "is exported\n\n", cmd, pv_name, pv->vg_name);
          } else if ( ret == -LVM_EPV_READ_ID_INVALID) {
-            fprintf ( stderr, "%s -- no physical volume identifier on %s\n\n",
-                      cmd, pv_name);
+            fprintf ( stderr, "%s -- no physical volume identifier "
+                              "on \"%s\"\n\n",
+                              cmd, pv_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d: no VALID physical volume %s\n\n",
-                      cmd, ret, pv_name);
+            fprintf ( stderr, "%s -- ERROR %d: no VALID physical "
+                              "volume \"%s\"\n\n",
+                              cmd, ret, pv_name);
          }
          continue;
       } else if ( pv_check_new ( pv) == TRUE) {
-         printf ( "%s -- %s is a new physical volume of %s\n\n",
+         printf ( "%s -- \"%s\" is a new physical volume of %s\n\n",
                   cmd, pv_name,
                   ( dummy = lvm_show_size ( pv_get_size ( pv_name,NULL) / 2,
                                                           SHORT)));
@@ -187,28 +197,30 @@ int main ( int argc, char **argv) {
          continue;
       }
    
-
-
       if ( opt_v > 1) printf ( "%s -- checking physical volume activity\n",
                                 cmd);
       if ( pv_check_active ( pv->vg_name, pv->pv_name) == TRUE) {
          if ( ( ret = pv_status  ( pv->vg_name, pv->pv_name, &pv)) != 0) {
             if ( ret == -ENXIO) {
-               fprintf ( stderr, "%s -- no such physical volume %s\n\n",
-                         cmd, pv_name);
+               fprintf ( stderr, "%s -- no such physical volume \"%s\"\n\n",
+                                 cmd, pv_name);
                continue;
             } else {
-               fprintf ( stderr, "%s -- ERROR %d: getting status of %s\n\n",
-                         cmd, ret, pv_name);
+               fprintf ( stderr, "%s -- ERROR %d: getting status of "
+                                 "physical volume \"%s\"\n\n",
+                                 cmd, ret, pv_name);
                continue;
             }
          }
       }
    
+      if ( opt_v > 1) printf ( "%s -- checking physical volume consistency\n",
+                               cmd);
       if ( ( ret = pv_check_consistency ( pv)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d checking consistency of physical"
-                           " volume %s\n\n", cmd, ret, pv_name);
-         return 1;
+                           " volume \"%s\"\n\n",
+                           cmd, ret, pv_name);
+         return LVM_EPVDISPLAY_PV_CHECK_CONSISTENCY;
       }
 
       pv_show ( pv);
@@ -217,13 +229,14 @@ int main ( int argc, char **argv) {
          if ( pv->pe_allocated > 0) {
             if ( ( ret = pv_read_pe ( pv, &pv->pe)) != 0) {
                fprintf ( stderr, "%s -- ERROR %d reading physical"
-                                 " extend information\n\n", cmd, ret);
-               return 1;
+                                 " extent information\n\n", cmd, ret);
+               return LVM_EPVDISPLAY_PV_READ_PE;
             }
             printf ( "\n");
             pv_show_pe_text ( pv, pv->pe, pv->pe_total);
          } else {
-            printf ( "\n%s -- no logical volume on %s\n", cmd, pv_name);
+            printf ( "\n%s -- no logical volume on physical volume \"%s\"\n",
+                     cmd, pv_name);
          }
       }
    
