@@ -1,5 +1,7 @@
 /*
- * Author: Heinz Mauelshagen, Germany (mge@ez-darmstadt.telekom.de)
+ * tools/pvscan.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * May, November 1997
  * May,June 1998
@@ -48,10 +50,13 @@ int opt_d = 0;
 int main ( int argc, char **argv)
 {
    int c = 0;
+   int len = 0;
    int opt_v = 0;
    int p = 0;
+   int pv_max_name_len = 0;
    int ret = 0;
    int vg_name_len = 0;
+   int vg_max_name_len = 0;
    char *dummy1 = NULL;
    char *dummy2 = NULL;
 #ifdef DEBUG
@@ -59,6 +64,8 @@ int main ( int argc, char **argv)
 #else
    char *options = "h?v";
 #endif
+   char pv_tmp_name[NAME_LEN] = { 0, };
+   char vg_tmp_name[NAME_LEN] = { 0, };
    char vg_name_this[NAME_LEN] = { 0, };
    pv_t **pv = NULL;
    pv_t *pv_tmp;
@@ -73,8 +80,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -97,8 +104,8 @@ int main ( int argc, char **argv)
 
          case 'v':
             if ( opt_v > 1) {
-               fprintf ( stderr, "%s -- v option yet given two times\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given two times\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -106,10 +113,12 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    LVM_LOCK ( 1);
    LVM_CHECK_IOP;
 
@@ -118,17 +127,27 @@ int main ( int argc, char **argv)
    if ( ( ret = pv_read_all_pv ( &pv, TRUE)) < 0) {
       fprintf ( stderr, "%s -- ERROR %d: reading physical volumes\n\n",
                 cmd, ret);
-      return 1;
+      return LVM_EPVSCAN_PV_READ_ALL_PV;
    }
 
 
    if ( opt_v > 0) printf ( "%s -- walking through all physical "
                             "volumes found\n", cmd);
+   /* check maximum vg and pv name length */
+   if ( pv != NULL) for ( p = 0; pv[p] != NULL; p++) {
+      len = strlen ( pv[p]->pv_name);
+      if ( pv_max_name_len < len) pv_max_name_len = len;
+      len = strlen ( pv[p]->vg_name);
+      if ( vg_max_name_len < len) vg_max_name_len = len;
+   }
+   pv_max_name_len += 2;
+   vg_max_name_len += 2;
+
    if ( pv != NULL) for ( p = 0; pv[p] != NULL; p++) {
       if ( ( ret = pv_read ( pv[p]->pv_name, &pv_tmp, NULL)) < 0 &&
            ret == -LVM_EPV_READ_MD_DEVICE) {
          if ( MAJOR ( pv_create_kdev_t ( pv[p]->pv_name)) != MD_MAJOR) {
-            printf ( "%s -- WARNING: physical volume %s belongs "
+            printf ( "%s -- WARNING: physical volume \"%s\" belongs "
                      "to a multiple device\n",
                      cmd, pv[p]->pv_name);
          }
@@ -136,10 +155,10 @@ int main ( int argc, char **argv)
       }
 
       if ( opt_v > 1) {
-         printf ( "%s -- getting data for %s from kernel\n",
+         printf ( "%s -- getting data for physical volume \"%s\" from kernel\n",
                   cmd, pv[p]->pv_name);
          if ( pv_status ( pv[p]->vg_name, pv[p]->pv_name, &pv_tmp) < 0)
-            printf ( "%s -- physical volume %s is not active\n",
+            printf ( "%s -- physical volume \"%s\" is not active\n",
                      cmd, pv[p]->pv_name);
          pv_show ( pv[p]);
          printf ( "System Id             %s\n\n", pv[p]->system_id);
@@ -152,14 +171,14 @@ int main ( int argc, char **argv)
             printf ( "ACTIVE   ");
          vg_name_len = strlen ( pv[p]->vg_name) - sizeof ( EXPORTED) + 1;
          if ( pv_check_new ( pv[p]) == TRUE) {
-            printf ( "PV %s  is in no VG  [%s]\n",
+            printf ( "PV \"%s\"  is in no VG  [%s]\n",
                      pv[p]->pv_name,
                      ( dummy1 = lvm_show_size ( pv[p]->pv_size / 2, SHORT)));
             free ( dummy1); dummy1 = NULL;
          } else if ( strcmp ( &pv[p]->vg_name[vg_name_len],
                               EXPORTED) == 0) {
             strncpy ( vg_name_this, pv[p]->vg_name, vg_name_len);
-            printf ( "PV %s  is in EXPORTED VG %s [%s / %s free]\n",
+            printf ( "PV \"%s\"  is in EXPORTED VG \"%s\" [%s / %s free]\n",
                      pv[p]->pv_name, vg_name_this,
                      ( dummy1 = lvm_show_size ( pv[p]->pe_total *
                                             pv[p]->pe_size / 2, SHORT)),
@@ -167,14 +186,19 @@ int main ( int argc, char **argv)
             free ( dummy1); dummy1 = NULL;
             free ( dummy2); dummy2 = NULL;
          } else if ( lvm_tab_vg_check_exist ( pv[p]->vg_name, NULL) != TRUE) {
-            printf ( "PV %s  is associated to an unknown VG "
+            printf ( "PV \"%s\"  is associated to an unknown VG "
                      "(run vgscan)\n", pv[p]->pv_name);
          } else {
-            printf ( "PV %-10s of VG %s [%s / %s free]\n",
-                     pv[p]->pv_name, pv[p]->vg_name,
+            sprintf ( pv_tmp_name, "\"%s\"", pv[p]->pv_name);
+            sprintf ( vg_tmp_name, "\"%s\"", pv[p]->vg_name);
+            printf ( "PV %-*s of VG %-*s [%s / %s free]\n",
+                     pv_max_name_len, pv_tmp_name, vg_max_name_len,
+                     vg_tmp_name,
                      ( dummy1 = lvm_show_size ( pv[p]->pe_total *
                                             pv[p]->pe_size / 2, SHORT)),
-                     ( dummy2 = lvm_show_size ( ( pv[p]->pe_total - pv[p]->pe_allocated) *pv[p]->pe_size / 2, SHORT)));
+                     ( dummy2 = lvm_show_size ( ( pv[p]->pe_total -
+                                                  pv[p]->pe_allocated) *
+                                                pv[p]->pe_size / 2, SHORT)));
             free ( dummy1); dummy1 = NULL;
             free ( dummy2); dummy2 = NULL;
          }
@@ -187,6 +211,6 @@ int main ( int argc, char **argv)
    else         printf ( "%s -- %d", cmd, p);
    printf ( " valid physical volumes found\n\n");
 
-   if ( p == 0) return 1;
+   if ( p == 0) return LVM_EPVSCAN_NO_PV_FOUND;
    else         return 0;
 }
