@@ -1,5 +1,7 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgcfgbackup.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * June 1997
  * May 1998
@@ -59,8 +61,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -84,8 +86,8 @@ int main ( int argc, char **argv)
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -93,10 +95,12 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( optind < argc) {
       if ( opt_v > 0) printf ( "%s -- using volume group(s) on command line\n",
                                cmd);
@@ -109,9 +113,7 @@ int main ( int argc, char **argv)
       if ( vg_name_ptr != NULL)
          for ( v = 0; vg_name_ptr[v] != NULL; v++) argc++;
       if ( argc > 0 && opt_v > 0) {
-         printf ( "%s -- volume group", cmd);
-         if ( argc > 1) printf ( "s");
-         printf ( " found: ");
+         printf ( "%s -- volume group%s found: ", cmd, argc > 1 ? "s" : "");
          for ( v = 0; vg_name_ptr[v] != NULL; v++)
             printf ( "%s ", vg_name_ptr[v]);
          printf ( "\n");
@@ -134,16 +136,17 @@ int main ( int argc, char **argv)
    for ( ; optind < argc; optind++)
       if ( ( ret = vg_cfgbackup ( argv[optind],
                                   VG_BACKUP_DIR, opt_v, NULL)) < 0) {
-         fprintf ( stderr, "%s -- backup ERROR %d volume group %s\n\n",
+         fprintf ( stderr, "%s -- ERROR %d backing up volume group \"%s\"\n\n",
                    cmd, ret, argv[optind]);
-         return 1;
+         return LVM_EVGCFGBACKUP_VG_CFGBACKUP;
       }
       else {
          if ( ret == LVM_VG_CFGBACKUP_NO_DIFF) {
-            printf ( "%s -- backup of volume group %s is not necessary\n",
+            printf ( "%s -- VGDA backup of volume group \"%s\" is "
+                     "not necessary\n",
                      cmd, argv[optind]);
          } else {
-            printf ( "%s -- successfull backup of volume group %s\n",
+            printf ( "%s -- successfull VGDA backup of volume group \"%s\"\n",
                      cmd, argv[optind]);
          }
       }
