@@ -1,5 +1,7 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgcfgrestore.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * June,October 1997
  * May 1998
@@ -87,8 +89,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -96,8 +98,8 @@ int main ( int argc, char **argv)
 
          case 'f':
             if ( opt_f> 0) {
-               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- f option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             vg_conf_path = optarg;
             opt_f++;
@@ -126,52 +128,53 @@ int main ( int argc, char **argv)
 
          case 'l':
             if ( opt_l > 1) {
-               fprintf ( stderr, "%s -- l option yet given two times\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- l option already given two times\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_l++;
             break;
 
          case 'n':
             if ( opt_n> 0) {
-               fprintf ( stderr, "%s -- n option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- n option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             vg_name = optarg;
             if ( vg_check_name ( vg_name) < 0) {
-               fprintf ( stderr, "%s -- invalid volume group name %s\n\n",
+               fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n\n",
                                  cmd, vg_name);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_n++;
             break;
 
          case 'o':
             if ( opt_o> 0) {
-               fprintf ( stderr, "%s -- o option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- o option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             pv_name_old = optarg;
             if ( pv_check_name ( pv_name_old) < 0) {
-               fprintf ( stderr, "%s -- invalid physical volume path %s\n\n",
+               fprintf ( stderr, "%s -- invalid physical volume path "
+                                 "\"%s\"\n\n",
                                  cmd, pv_name_old);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_o++;
             break;
 
          case 't':
             if ( opt_t > 0) {
-               fprintf ( stderr, "%s -- t option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- t option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_t++;
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -179,38 +182,40 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( opt_f > 0 && opt_n == 0) {
       fprintf ( stderr, "%s -- please give option n with option f\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( opt_l == 0) {
       if ( optind != argc - 1) {
          printf ( "%s -- please enter a physical volume name\n\n", cmd);
-         return 1;
+         return LVM_EVGCFGRESTORE_PV_MISSING;
       }
       pv_name = argv[optind];
       if ( pv_check_name ( pv_name) < 0) {
-         fprintf ( stderr, "%s -- invalid physical volume %s\n\n",
+         fprintf ( stderr, "%s -- invalid physical volume \"%s\"\n\n",
                            cmd, pv_name);
-         return 1;
+         return LVM_EVGCFGRESTORE_PV_CHECK_NAME;
       }
       if ( opt_f == 0 && opt_n == 0) {
          fprintf ( stderr, "%s -- please enter a volume group name or a "
                            "volume group backup\n\n", cmd);
-         return 1;
+         return LVM_EINVALID_CMD_LINE;
       }
    }
 
    if ( opt_t == 0 && opt_l == 0 && vg_check_active ( vg_name) == TRUE) {
       fprintf ( stderr, "%s -- can't restore part of active "
-                        "volume group %s\n\n",
+                        "volume group \"%s\"\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_EVGCFGRESTORE_VG_CHECK_ACTIVE;
    }
 
    if ( opt_f == 0) sprintf ( vg_backup_path,
@@ -222,7 +227,7 @@ int main ( int argc, char **argv)
    LVM_CHECK_IOP;
    lvm_dont_interrupt ( 0);
 
-   if ( opt_v > 0) printf ( "%s -- restoring %s from %s\n",
+   if ( opt_v > 0) printf ( "%s -- restoring volume group \"%s\" from \"%s\"\n",
                             cmd, vg_name, vg_backup_path);
    if ( ( ret = vg_cfgrestore ( vg_name, vg_backup_path,
                                 opt_v, &vg)) < 0) {
@@ -231,48 +236,51 @@ int main ( int argc, char **argv)
                            "%s is not available\n\n",
                            cmd, vg_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d restoring %s\n\n",
+         fprintf ( stderr, "%s -- ERROR %d restoring volume group \"%s\"\n\n",
                            cmd, ret, vg_name);
       }
-      return 1;
+      return LVM_EVGCFGRESTORE_VG_CFGRESTORE;
    }
 
    if ( opt_n == 0) {
       vg_name = vg.vg_name;
-      printf ( "%s -- this is a backup of volume group %s\n", cmd, vg_name);
+      printf ( "%s -- this is a backup of volume group \"%s\"\n", cmd, vg_name);
    } else if ( strcmp ( vg_name, vg.vg_name) != 0) {
-      fprintf ( stderr, "%s -- this is no backup of volume group %s\n\n",
+      fprintf ( stderr, "%s -- this is no backup of volume group \"%s\"\n\n",
                 cmd, vg_name);
-      return 1;
+      return LVM_EVGCFGRESTORE_INVALID;
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
-                            " of %s\n", cmd, vg_name);
+   if ( opt_v > 0) printf ( "%s -- checking volume group consistency"
+                            " of \"%s\"\n", cmd, vg_name);
    if ( ( ret = vg_check_consistency_with_pv_and_lv ( &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: %s is inconsistent\n\n",
-                cmd, ret, vg_name);
-      return 1;
-   } else if ( opt_t > 0) printf ( "%s -- backup of %s is consistent\n",
-                                      cmd, vg_name);
+      fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" "
+                        "is inconsistent\n\n",
+                        cmd, ret, vg_name);
+      return LVM_EVGCFGRESTORE_VG_CHECK_CONSISTENCY;
+   } else if ( opt_t > 0) printf ( "%s -- backup of volume group \"%s\" "
+                                   " is consistent\n",
+                                   cmd, vg_name);
 
    pv_index = pv_index_old = -1;
    if ( opt_o > 0) {
       if ( pv_check_in_vg ( &vg, pv_name_old) == FALSE) {
-         fprintf ( stderr, "%s -- %s doesn't belong to volume group "
-                              "%s backup\n\n",
-                        cmd, pv_name_old, vg_name);
-         return 1;
+         fprintf ( stderr, "%s -- physical volume \"%s\" doesn't belong "
+                           "to volume group \"%s\" backup\n\n",
+                           cmd, pv_name_old, vg_name);
+         return LVM_EVGCFGRESTORE_PV_CHECK_IN_VG;
       } else pv_index_old = pv_get_index_by_name ( &vg, pv_name_old);
       if ( pv_check_in_vg ( &vg, pv_name) == TRUE) {
-         fprintf ( stderr, "%s -- %s may not be an actual physical volume "
-                           "of %s\n\n", cmd, pv_name, vg_name);
-         return 1;
+         fprintf ( stderr, "%s -- \"%s\" may not be an actual physical volume "
+                           "of volume group \"%s\"\n\n", cmd, pv_name, vg_name);
+         return LVM_EVGCFGRESTORE_PV_CHECK_IN_VG;
       } else pv_index = pv_get_index_by_name ( &vg, pv_name);
    } else {
       if ( pv_check_in_vg ( &vg, pv_name) == FALSE) {
-         fprintf ( stderr, "%s -- %s doesn't belong to volume group %s\n\n",
+         fprintf ( stderr, "%s -- physical volume \"%s\" doesn't belong "
+                           "to volume group \"%s\"\n\n",
                            cmd, pv_name, vg_name);
-         return 1;
+         return LVM_EVGCFGRESTORE_PV_CHECK_IN_VG;
       } else pv_index = pv_get_index_by_name ( &vg, pv_name);
    }
 
@@ -284,23 +292,26 @@ int main ( int argc, char **argv)
       /* no test run */
       if ( opt_t == 0) {
          /* check given physical volume */
-         if ( opt_v > 0) printf ( "%s -- reading physical volume %s\n",
+         if ( opt_v > 0) printf ( "%s -- reading physical volume \"%s\"\n",
                                   cmd, pv_name);
          if ( ( ret = pv_read ( pv_name, &pv, NULL)) < 0 &&
               ret != -LVM_EPV_READ_MD_DEVICE) {
             fprintf ( stderr, "%s -- ERROR %d reading physical "
-                              "volume %s\n\n",
+                              "volume \"%s\"\n\n",
                               cmd, ret, pv_name);
-            return 1;
+            return LVM_EVGCFGRESTORE_PV_READ;
          }
 
+         if ( opt_v > 0) printf ( "%s -- checking for new physical "
+                                  "volume \"%s\"\n",
+                                  cmd, pv_name);
          if ( pv_check_new ( pv) == FALSE) {
             if ( vg_check_name ( pv->vg_name) == 0 &&
                  strcmp ( pv->vg_name, vg_name) != 0) {
-               fprintf ( stderr, "%s -- physical volume %s belongs "
-                                 "to %s\n\n",
+               fprintf ( stderr, "%s -- physical volume \"%s\" belongs "
+                                 "to volume group \"%s\"\n\n",
                                  cmd, pv->pv_name, pv->vg_name);
-               return 1;
+               return LVM_EVGCFGRESTORE_PV_CHECK_IN_VG;
             }
          }
 
@@ -308,9 +319,9 @@ int main ( int argc, char **argv)
          else                    index = pv_index;
 
          if ( pv_get_size ( pv_name, NULL) !=  vg.pv[index]->pv_size) {
-            fprintf ( stderr, "%s -- size of physical volume %s differs "
+            fprintf ( stderr, "%s -- size of physical volume \"%s\" differs "
                               "from backup\n\n", cmd, pv_name);
-            return 1;
+            return LVM_EVGCFGRESTORE_PV_GET_SIZE;
          }
 
          memcpy ( &vg_write, &vg, sizeof ( vg));
@@ -318,47 +329,53 @@ int main ( int argc, char **argv)
          vg_write.pv[1] = NULL;
 
          /* store VGDA from backup on physical volume */
+         if ( opt_v) printf ( "%s -- writing VGDA of \"%s\" to physical "
+                              "volume \"%s\"\n",
+                              cmd, vg_name, vg_write.pv[0]->pv_name);
          if ( ( ret = vg_write_with_pv_and_lv ( &vg_write)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d storing volume group "
-                              "data of %s on disks\n\n", 
+            fprintf ( stderr, "%s -- ERROR %d storing VGDA "
+                              "of \"%s\" on disks\n\n", 
                               cmd, ret, vg_name);
-            return 1;
+            return LVM_EVGCFGRESTORE_VG_WRITE;
          }
    
          if ( opt_v) printf ( "%s -- removing any special files "
-                              "for %s\n", cmd, vg_name);
+                              "for \"%s\"\n", cmd, vg_name);
          vg_remove_dir_and_group_and_nodes ( vg_name);
    
          if ( ( ret = vg_create_dir_and_group_and_nodes ( &vg, opt_v)) < 0) {
             fprintf ( stderr, "%s -- %d errors creating volume group "
                               "directory and special files\n\n",
                               cmd, -ret);
-            return 1;
+            return LVM_EVGCFGRESTORE_VG_REMOVE_DIR;
          }
    
-         if ( opt_v > 0) printf ( "%s -- inserting %s in %s\n",
+         if ( opt_v > 0) printf ( "%s -- inserting volume group \"%s\" "
+                                  "into \"%s\"\n",
                                   cmd, vg_name, LVMTAB);
          if ( ( ret = lvm_tab_vg_insert ( vg_name)) < 0 &&
               ret != -LVM_ELVM_TAB_VG_INSERT_VG_EXISTS) {
-            fprintf ( stderr, "%s -- ERROR %d inserting VG into %s\n\n",
+            fprintf ( stderr, "%s -- ERROR %d inserting volume group into "
+                              "\"%s\"\n\n",
                               cmd, ret, LVMTAB);
-            return 1;
+            return LVM_EVGCFGRESTORE_LVM_TAB_VG_INSERT;
          }
          
          if ( ( ret = vg_cfgbackup ( vg_name, LVMTAB_DIR,
                                      opt_v, &vg)) < 0) {
             fprintf ( stderr, "%s -- ERROR %d writing lvmtab\n\n",
                               cmd, ret);
-            return 1;
+            return LVM_EVGCFGRESTORE_VG_CFGBACKUP;
          }
 
-         printf ( "%s -- volume group %s descriptor area "
-                  "successfully restored to %s\n",
+         printf ( "%s -- VGDA for \"%s\" successfully restored "
+                  "to physical volume \"%s\"\n",
                   cmd, vg_name, pv_name);
          if ( opt_f > 0)
             printf ( "%s -- you may not have an actual backup of "
-                     "restored volume group %s\n", cmd, vg_name);
-      } else printf ( "%s -- test run for %s end\n", cmd, vg_name);
+                     "restored volume group \"%s\"\n", cmd, vg_name);
+      } else printf ( "%s -- test run for volume group \"%s\" end\n",
+                      cmd, vg_name);
    }
 
    printf ( "\n");
