@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgchange.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March-May,October 1997
- * May,June 1998
+ * May,June,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,8 @@
  *    05/09/1998 - implemented -x option
  *    05/16/1998 - added lvmtab checking
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/20/1998 - changed message in deativation volume group
+ *                 with open logical volumes
  *
  */
 
@@ -48,6 +52,7 @@ int main ( int argc, char **argv) {
    int ret = 0;
    int set_extendable = FALSE;
    int v = 0;
+   char *dir = NULL;
    char *vg_name = NULL;
 #ifdef DEBUG
    char *options = "A:a:dh?vx:";
@@ -74,23 +79,23 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_A++;
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'a':
             if ( opt_a > 0) {
-               fprintf ( stderr, "%s -- a option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- a option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_a++;
             if ( strcmp ( optarg, "y") == 0) {
@@ -102,14 +107,14 @@ int main ( int argc, char **argv) {
             }
             fprintf ( stderr, "%s -- a option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -135,8 +140,8 @@ int main ( int argc, char **argv) {
 
          case 'x':
             if ( opt_x > 0) {
-               fprintf ( stderr, "%s -- x option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- x option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_x++;
             if ( strcmp ( optarg, "y") == 0) {
@@ -148,13 +153,13 @@ int main ( int argc, char **argv) {
             }
             fprintf ( stderr, "%s -- a option argument \"%s\" invalid\n\n",
                       cmd, optarg);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -162,19 +167,21 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( opt_a == 0 && opt_x == 0) {
       fprintf ( stderr, "%s -- you have to give the -a or -x option\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
    
    if ( opt_a == 1 && opt_A > 1) {
       fprintf ( stderr, "%s -- -A option not neccessary with "
                         "-a option\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
    
    LVM_LOCK ( 0);
@@ -204,51 +211,62 @@ int main ( int argc, char **argv) {
    for ( ; optind < argc; optind++) {
       vg_name = argv[optind];
 
-      if ( opt_v > 0) printf ( "%s -- checking volume group name %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
                                cmd, vg_name);
       if ( vg_check_name ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- invalid volume group name %s\n",
+         fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n",
                    cmd, vg_name);
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking existence of volume group %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking existence of volume "
+                               "group \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) < 0) {
          if ( ret == -LVM_EPV_READ_PV_EXPORTED)
-            fprintf ( stderr, "%s -- can't access exported volume group %s\n",
+            fprintf ( stderr, "%s -- can't access exported volume "
+                              "group \"%s\"\n",
                               cmd, vg_name);
          else if ( ret == -LVM_EVG_CHECK_EXIST_PV_COUNT)
             fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
-                              "volume group %s online\n", cmd, vg_name);
+                              "volume group \"%s\" online\n", cmd, vg_name);
          else if ( ret == -LVM_EPV_READ_ALL_PV_OF_VG_NP)
-            fprintf ( stderr, "%s -- volume group %s doesn't exist\n",
+            fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
                       cmd, vg_name);
          else if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION)
-            fprintf ( stderr, "%s -- volume group %s has physical volumes "
+            fprintf ( stderr, "%s -- volume group \"%s\" has physical volumes "
                               "with invalid version\n",
                       cmd, vg_name);
          else
-            fprintf ( stderr, "%s -- ERROR %d checking existence of %s\n",
+            fprintf ( stderr, "%s -- ERROR %d checking existence of "
+                              "volume group \"%s\"\n",
                       cmd, ret, vg_name);
          vg_deactivate ( vg_name);
          continue;
       }
 
-      if ( opt_v > 0) printf ( "%s -- reading volume group data for %s "
+      if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
                                "from lvmtab\n", cmd, vg_name);
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: reading lvmtab data of %s\n",
-                   cmd, ret, vg_name);
-         vg_deactivate ( vg_name);
+         if ( ret == -LVM_EVG_CFGRESTORE_OPEN) {
+            fprintf ( stderr, "%s -- ERROR: volume group \"%s\" "
+                              "doesn't exist\n",
+                              cmd, vg_name);
+         } else {
+            fprintf ( stderr, "%s -- ERROR %d: reading lvmtab data of "
+                              "volume group \"%s\"\n",
+                              cmd, ret, vg_name);
+         }
+         if ( vg_check_active ( vg_name) == TRUE) vg_deactivate ( vg_name);
          continue;
       }
    
       if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
-                               " of %s\n", cmd, vg_name);
+                               " of \"%s\"\n", cmd, vg_name);
       if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: %s is inconsistent\n",
-                   cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" "
+                           " is inconsistent\n",
+                           cmd, ret, vg_name);
          vg_deactivate ( vg_name);
          continue;
       }
@@ -258,72 +276,91 @@ int main ( int argc, char **argv) {
       if ( opt_a > 0) {
          if ( opt_a_action == VG_CREATE) {
             if ( opt_v > 0) printf ( "%s -- checking if all physical volumes "
-                                     "of %s are available\n",
+                                     "of volume group \"%s\" are available\n",
                                      cmd, vg_name);
             if ( ( ret = vg_check_online_all_pv ( vg, &pv_offl,
                                                   &pv_incons)) < 0 &&
                  ret != -LVM_EPV_READ_MD_DEVICE) {
-               for ( p = 0; pv_offl != NULL && pv_offl[p] != NULL; p++) {
-                  fprintf ( stderr, "%s -- ERROR: physical volume %s of "
-                                    "volume group %s is offline\n",
+               if ( pv_offl != NULL) for ( p = 0; pv_offl[p] != NULL; p++) {
+                  fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" of "
+                                    "volume group \"%s\" is offline\n",
                                     cmd, pv_offl[p]->pv_name, vg_name);
                }
-               for ( p = 0; pv_incons != NULL && pv_incons[p] != NULL; p++) {
-                  fprintf ( stderr, "%s -- ERROR: physical volume %s of "
-                                    "volume group %s is inconsistent\n",
+               if ( pv_incons != NULL) for ( p = 0; pv_incons[p] != NULL; p++) {
+                  fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" of "
+                                    "volume group \"%s\" is inconsistent\n",
                                     cmd, pv_incons[p]->pv_name, vg_name);
                }
                printf ( "%s -- run vgscan\n", cmd);
                continue;
             }
-            if ( pv_offl != NULL) free ( pv_offl);
+            if ( pv_offl   != NULL) free ( pv_offl);
             if ( pv_incons != NULL) free ( pv_incons);
       
-            if ( opt_v > 0) printf ( "%s -- creating VGDA for %s in kernel\n",
+            if ( opt_v > 0) printf ( "%s -- creating VGDA for \"%s\" "
+                                     "in kernel\n",
                                      cmd, vg_name);
             if ( ( ret = vg_create ( vg_name, vg)) == 0)
-               printf ( "%s -- %s successfully activated\n", cmd, vg_name);
+               printf ( "%s -- volume group \"%s\" successfully activated\n",
+                        cmd, vg_name);
             else {
    #ifdef DEBUG
                debug ( "%s -- vg_create returned: %d\n", cmd, ret);
    #endif
                if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
                   fprintf ( stderr, "%s -- can't open logical volume manager "
-                                    "special to activate %s\n", cmd, vg_name);
+                                    "to activate volume group \"%s\"\n",
+                                    cmd, vg_name);
                } else if ( ret == -EPERM) {
-                  fprintf ( stderr, "%s -- volume group %s is already active\n",
+                  fprintf ( stderr, "%s -- volume group \"%s\" "
+                                    "already active\n",
                                     cmd, vg_name);
                } else if ( ret == -ENOMEM) {
-                  fprintf ( stderr, "%s -- memory error activating %s\n",
+                  fprintf ( stderr, "%s -- memory error activating "
+                                    "volume group \"%s\"\n",
                                     cmd, vg_name);
                } else {
-                  fprintf ( stderr, "%s -- ERROR %d activating %s\n",
-                  cmd, ret, vg_name);
+                  fprintf ( stderr, "%s -- ERROR %d activating "
+                                    "volume group \"%s\"\n",
+                                    cmd, ret, vg_name);
                }
                continue;
             }
          } else {
-            if ( opt_v > 0) printf ( "%s -- removing VGDA for %s from kernel\n",
+            if ( opt_v > 0) printf ( "%s -- removing VGDA for volume "
+                                     "group \"%s\" from kernel\n",
                                      cmd, vg_name);
             if ( ( ret = vg_remove ( vg_name)) == 0)
-               printf ( "%s -- %s successfully deactivated\n", cmd, vg_name);
+               printf ( "%s -- volume group \"%s\" successfully deactivated\n",
+                        cmd, vg_name);
             else {
    #ifdef DEBUG
                debug ( "%s -- vg_remove returned: %d\n", cmd, ret);
    #endif
                if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
                   fprintf ( stderr, "%s -- can't open logical volume manager "
-                                    "special to deactivate %s\n?\n", cmd, vg_name);
+                                    "to deactivate \"%s\"?\n", cmd, vg_name);
                } else if ( ret == -EPERM) {
-                   fprintf ( stderr, "%s -- can't deactivate volume group %s "
-                                     "with open logical volume(s)\n",
-                                     cmd, vg_name);
+                  if ( opt_v > 0)
+                     printf ( "%s -- getting volume group status from "
+                              "VGDA in kernel\n", cmd);
+                  if ( vg_status ( vg_name, &vg) < 0) {
+                     fprintf ( stderr, "%s -- can't get status of volume group "
+                                       "\"%s\" from kernel\n",
+                                       cmd, vg_name);
+                  } else {
+                     fprintf ( stderr, "%s -- can't deactivate volume group "
+                                       "\"%s\" with %d open logical volume%s\n",
+                                       cmd, vg_name, vg->lv_open,
+                                       vg->lv_open > 1 ? "s" : "");
+                  }
                } else if ( ret == -ENXIO) {
-                   fprintf ( stderr, "%s -- volume group %s isn't active\n",
-                             cmd, vg_name);
+                   fprintf ( stderr, "%s -- volume group \"%s\" isn't active\n",
+                                     cmd, vg_name);
                } else {
-                   fprintf ( stderr, "%s -- ERROR %d DEactivating %s\n",
-                             cmd, ret, vg_name);
+                   fprintf ( stderr, "%s -- ERROR %d DEactivating volume "
+                                     "group \"%s\"\n",
+                                     cmd, ret, vg_name);
                }
                continue;
             }
@@ -334,7 +371,7 @@ int main ( int argc, char **argv) {
       if ( opt_x > 0) {
          if ( opt_x_action == VG_EXTENDABLE) {
             if ( vg->vg_status & VG_EXTENDABLE) {
-               printf ( "%s -- volume group %s ist yet extendable\n",
+               printf ( "%s -- volume group \"%s\" is extendable\n",
                         cmd, vg->vg_name);
             } else {
                vg->vg_status |= VG_EXTENDABLE;
@@ -343,7 +380,7 @@ int main ( int argc, char **argv) {
             }
          } else {
             if ( ! ( vg->vg_status & VG_EXTENDABLE)) {
-               printf ( "%s -- volume group %s ist allready not extendable\n",
+               printf ( "%s -- volume group \"%s\" is not extendable\n",
                         cmd, vg_name);
             } else {
                vg->vg_status &= ~VG_EXTENDABLE;
@@ -357,8 +394,8 @@ int main ( int argc, char **argv) {
                                      "%s on disk(s)\n", cmd, vg_name);
             if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
                fprintf ( stderr, "%s -- ERROR %d storing volume group data "
-                                 "of %s on disk(s)\n\n", cmd, ret, vg_name);
-               return 1;
+                                 "of \"%s\" on disk(s)\n\n", cmd, ret, vg_name);
+               return LVM_EVGCHANGE_VG_WRITE;
             }
       
             if ( opt_v > 0) printf ( "%s -- changing VGDA in kernel\n", cmd);
@@ -374,23 +411,27 @@ int main ( int argc, char **argv) {
                } else {
                    fprintf ( stderr, "dis");
               }
-              fprintf ( stderr, "abling extension of %s\n\n", vg_name);
-              return 1;
+              fprintf ( stderr, "abling extension of volume group \"%s\"\n\n",
+                                vg_name);
+              return LVM_EVGCHANGE_VG_EXTEND;
             }
 
             if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
-            if ( ( ret = vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg)) == 0 &&
+            dir = LVMTAB_DIR;
+            if ( ( ret = vg_cfgbackup ( vg_name, dir, opt_v, vg)) == 0 &&
                  opt_A > 0) {
-               printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
-               ret = vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+               printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+                        cmd, vg_name);
+               dir = VG_BACKUP_DIR;
+               ret = vg_cfgbackup ( vg_name, dir, opt_v, vg);
             }
             if ( ret == 0) {
-               printf ( "%s -- volume group %s successfully changed\n\n",
+               printf ( "%s -- volume group \"%s\" successfully changed\n\n",
                         cmd, vg_name);
             } else {
                fprintf ( stderr, "%s -- ERROR %d storing volume group backup "
-                                 "of %s on disk(s)\n\n", cmd, ret, vg_name);
-               return 1;
+                                 "of \"%s\" in %s\n\n", cmd, ret, vg_name, dir);
+               return LVM_EVGCHANGE_VG_CFG_BACKUP;
             }
          }
       }
@@ -401,6 +442,5 @@ int main ( int argc, char **argv) {
 
    LVM_UNLOCK ( 0);
 
-   if ( ret == 0) return 0;
-   else           return 1;
+   return 0;
 }
