@@ -1,8 +1,11 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgcreate.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March-June 1997
- * May,June 1998
+ * May,June,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +38,8 @@
  *    06/12/1998 - enhanced checking numbers in option arguments
  *    06/27/1998 - changed lvm_tab_* calling convention
  *               - changed to new vg_setup_for_create() calling convention
+ *    09/05/1998 - corrected some messages
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -89,23 +94,23 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -134,8 +139,8 @@ int main ( int argc, char **argv) {
 
          case 'l':
             if ( opt_l > 0) {
-               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- l option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( lvm_check_number ( optarg) == TRUE) {
                max_lv = atoi ( optarg);
@@ -143,14 +148,14 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 'p':
             if ( opt_p > 0) {
-               fprintf ( stderr, "%s -- p option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- p option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( lvm_check_number ( optarg) == TRUE) {
                max_pv = atoi ( optarg);
@@ -158,37 +163,37 @@ int main ( int argc, char **argv) {
             } else {
                fprintf ( stderr, "%s -- ERROR option p argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
          case 's':
             if ( opt_s > 0) {
-               fprintf ( stderr, "%s -- s option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- s option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( lvm_check_number ( optarg) == TRUE) {
                pe_size = atoi ( optarg) * 2;
                opt_s++;
                if ( vg_check_pe_size ( pe_size) < 0) {
-                  fprintf ( stderr, "%s -- invalid physical extend size %lu\n"
+                  fprintf ( stderr, "%s -- invalid physical extent size %lu\n"
                                     "%s -- must be power of 2 and between "
                                     "%lu and %lu\n\n",
                                     cmd, pe_size / 2, cmd,
                                     LVM_MIN_PE_SIZE, LVM_MAX_PE_SIZE);
-                  return 1;
+                  return LVM_EINVALID_CMD_LINE;
                }
             } else {
                fprintf ( stderr, "%s -- ERROR option s argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
  
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -196,41 +201,43 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name"
                         " and physical volumes\n\n", cmd);
-      return 1;
+      return LVM_EVGCREATE_VG_AND_PV_MISSING;
    }
    vg_name = argv[optind];
    optind++;
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter physical volume names\n\n", cmd);
-      return 1;
+      return LVM_EVGCREATE_PV_MISSING;
    }
 
    /* valid VG name? */
    if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
    if ( ( ret = vg_check_name ( vg_name)) < 0) {
-      fprintf ( stderr, "%s -- invalid volume group name %s\n\n",
+      fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n\n",
                 cmd, vg_name);
-      return 1;
+      return LVM_EVGCREATE_VG_CHECK_NAME;
    } 
 
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
 
    /* does VG exist? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group %s existence\n",
+   if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" existence\n",
                             cmd, vg_name);
    if ( lvm_tab_vg_check_exist ( vg_name, NULL) == TRUE) {
-      fprintf ( stderr, "%s -- volume group %s allready exists\n\n",
+      fprintf ( stderr, "%s -- volume group \"%s\" already exists\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_EVGCREATE_VG_CHECK_EXIST;
    } 
 
    if ( opt_v > 0) printf ( "%s -- counting all existing volume groups\n", cmd);
@@ -241,7 +248,7 @@ int main ( int argc, char **argv) {
    if ( vg_count >= MAX_VG) {
       fprintf ( stderr, "%s -- maximum volume group count of %d reached\n\n",
                         cmd, MAX_VG);
-      return 1;
+      return LVM_EVGCREATE_MAX_VG;
    } 
 
    /* read all PVs */
@@ -250,7 +257,7 @@ int main ( int argc, char **argv) {
    if ( ( ret = pv_read_all_pv ( &pv, FALSE)) < 0) {
 	  fprintf ( stderr, "%s -- ERROR %d reading physical volumes\n\n",
                 cmd, ret);
-	  return 1;
+	  return LVM_EVGCREATE_PV_READ_ALL_PV;
    }
 
    /* check, if PVs are all defined and new */
@@ -260,65 +267,66 @@ int main ( int argc, char **argv) {
    np = 0;
    for ( ; pv != NULL && optind < argc ; optind++) {
       pv_name = argv[optind];
+      if ( opt_v > 0) printf ( "%s -- checking physical volumes name \"%s\"\n",
+                               cmd, pv_name);
       if ( pv_check_name ( pv_name) < 0) {
-         fprintf ( stderr, "%s -- physical volume path %s is invalid\n\n",
+         fprintf ( stderr, "%s -- physical volume name \"%s\" is invalid\n\n",
                            cmd, pv_name);
-         return 1;
+         return LVM_EVGCREATE_PV_CHECK_NAME;
       }
       for ( p = 0; pv[p] != NULL; p++) {
          if ( strcmp ( pv_name, pv[p]->pv_name) == 0) {
-            if ( opt_v > 0) printf ( "%s -- checking %s\n"
+            if ( opt_v > 0) printf ( "%s -- checking physical volume \"%s\"\n"
                                      "%s -- getting "
-                                     "size of physical volume %s\n",
+                                     "size of physical volume \"%s\"\n",
                                      cmd, pv_name,
                                      cmd, pv_name);
-	        if ( ( size = pv_get_size ( pv[p]->pv_name, NULL)) <= 0) {
-	           fprintf ( stderr, "%s -- ERROR %d getting size of "
-                                 "physical volume %s\n\n",
+	    if ( ( size = pv_get_size ( pv[p]->pv_name, NULL)) <= 0) {
+	       fprintf ( stderr, "%s -- ERROR %d getting size of "
+                                 "physical volume \"%s\"\n\n",
                                  cmd, size, pv_name);
-               return 1;
+               return LVM_EVGCREATE_PV_GET_SIZE;
             }
-            if ( opt_v > 0) printf ( "%s -- size of %s is %d sectors\n",
+            if ( opt_v > 0) printf ( "%s -- size of physical volume \"%s\" "
+                                     " is %d sectors\n",
                                      cmd, pv_name, size);
 
             if ( opt_v > 0) printf ( "%s -- checking for new physical "
-                                     "volume %s\n", cmd, pv_name);
+                                     "volume \"%s\"\n", cmd, pv_name);
             if ( pv_check_new ( pv[p]) == FALSE) {
-               fprintf ( stderr, "%s -- %s is no new physical volume\n",
-				                 cmd, pv_name);
+               fprintf ( stderr, "%s -- \"%s\" is no new physical volume\n",
+				 cmd, pv_name);
                if ( opt_v > 0) printf ( "%s -- checking volume group name "
-                                        "of physical volume %s\n",
+                                        "of physical volume \"%s\"\n",
                                         cmd, pv_name);
                if ( vg_check_name ( pv[p]->vg_name) == 0) {
-                  fprintf ( stderr,
-                            "%s -- %s allready belongs to volume group %s\n",
-                            cmd, pv_name, pv[p]->vg_name);
+                  fprintf ( stderr, "%s -- physical volume \"%s\" already "
+                                    "belongs to volume group \"%s\"\n",
+                                    cmd, pv_name, pv[p]->vg_name);
                } else {
-                  fprintf ( stderr,
-                            "%s -- %s is an invalid physical volume\n",
-                            cmd, pv_name);
+                  fprintf ( stderr, "%s -- ERROR: \"%s\" is an invalid "
+                                    "physical volume\n",
+                                    cmd, pv_name);
                }
                fprintf ( stderr, "\n");
-               return 1;
+               return LVM_EVGCREATE_PV_CHECK_NEW;
             }
             if ( opt_v > 0) printf ( "%s -- checking for identical physical "
                                      "volumes on command line\n", cmd);
             for ( p1 = 0; pvp != NULL && pvp[p1] != NULL; p1++) {
                if ( strcmp ( pv_name, pvp[p1]->pv_name) == 0) {
-                  fprintf ( stderr, "%s -- ERROR: physical volume %s occurs "
-                                    "multiple times\n\n",
-				                    cmd, pv_name);
-	              return 1;
+                  fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" "
+                                    "occurs multiple times\n\n",
+				    cmd, pv_name);
+	              return LVM_EVGCREATE_PV_MULTIPLE;
                }
             }
 
             if ( ( pvp = realloc ( pvp, ( np + 2) * sizeof ( pv_t*))) == NULL) {
-         	   fprintf ( stderr, "%s -- alloc error %d byte "
-                                 "in file %s [line %d]\n\n",
-                                 cmd,
-                                 ( np + 2) * sizeof ( pv_t*),
-                                 __FILE__, __LINE__);
-         	   return 1;
+               fprintf ( stderr, "%s -- realloc error in file "
+                                 "\"%s\" [line %d]\n\n",
+                                 cmd, __FILE__, __LINE__);
+               return LVM_EVGCREATE_REALLOC;
             }
 
             pvp[np] = pv[p];
@@ -338,19 +346,19 @@ int main ( int argc, char **argv) {
    if ( np == 0) {
       fprintf ( stderr, "%s -- no valid physical volumes in command line\n\n",
                         cmd);
-      return 1;
+      return LVM_EVGCREATE_NO_VALID_PV;
    }
 
    if ( np != count_sav) {
       fprintf ( stderr,
                 "%s -- some invalid physical volumes in command line\n\n", cmd);
-      return 1;
+      return LVM_EVGCREATE_SOME_INVALID_PV;
    }
 
    if ( opt_v > 0) {
-      printf ( "%s -- %d physical volume", cmd, np);
-      if ( np > 1) printf ( "s");
-      printf ( " will be inserted into %s\n", vg_name);
+      printf ( "%s -- %d physical volume%s will be inserted into "
+               "volume group \"%s\"\n",
+               cmd, np, np > 1 ? "s" : "", vg_name);
    }
 
    /* load volume group */
@@ -362,7 +370,7 @@ int main ( int argc, char **argv) {
    if ( max_pv < 0 || max_pv <= np || max_pv > ABS_MAX_PV) {
       fprintf ( stderr, "%s -- invalid maximum physical volumes -p %d\n\n",
                         cmd, max_pv);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( opt_v > 0) printf ( "%s -- INFO: maximum of %d logical volumes\n",
@@ -370,7 +378,7 @@ int main ( int argc, char **argv) {
    if ( max_lv < 0 || max_lv > ABS_MAX_LV ) {
       fprintf ( stderr, "%s -- invalid maximum logical volumes -l %d\n\n",
                         cmd, max_lv);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    size = ( LVM_PV_DISK_SIZE +
@@ -384,7 +392,7 @@ int main ( int argc, char **argv) {
                         size,
                         pvp[min_pv_index]->pv_name,
                         pvp[min_pv_index]->pv_size / 2);
-      return 1;
+      return LVM_EVGCREATE_PV_TOO_SMALL;
    }
 
    if ( opt_v > 0) printf ( "%s -- setting up volume data for creation\n",
@@ -392,28 +400,27 @@ int main ( int argc, char **argv) {
    if ( ( ret = vg_setup_for_create ( vg_name, &vg, pvp, pe_size,
                                       max_pv, max_lv)) < 0) {
       if ( ret == -LVM_EVG_SETUP_FOR_CREATE_PV_SIZE) {
-         fprintf ( stderr, "%s -- ERROR: physical volume(s) too small "
-                           "for physical extend size of %s\n"
+         fprintf ( stderr, "%s -- ERROR: %d physical volume%s too small "
+                           "for physical extent size of %s\n"
                            "%s -- minimum physical volume size "
                            "must be %s\n\n",
-                           cmd, vg_name, cmd,
+                           cmd, count_sav, count_sav > 1 ? "s" : "",
+                           vg_name, cmd,
                            ( dummy = lvm_show_size ( vg.pe_size /
                                                  2 * LVM_PE_SIZE_PV_SIZE_REL,
                                                  SHORT)));
                            free ( dummy); dummy = NULL;
       } else {
-         fprintf ( stderr, "%s -- ERROR %d setting volume group %s up "
+         fprintf ( stderr, "%s -- ERROR %d setting volume group \"%s\" up "
                            "for create\n\n",
                            cmd, ret, vg_name);
       }
-      return 1;
+      return LVM_EVGCREATE_VG_SETUP;
    }
 
    if ( opt_s == 0) {
-      printf ( "%s -- INFO: using default physical extend "
-               "size %s\n",
-               cmd,
-               ( dummy = lvm_show_size ( pe_size / 2, SHORT)));
+      printf ( "%s -- INFO: using default physical extent size %s\n",
+               cmd, ( dummy = lvm_show_size ( pe_size / 2, SHORT)));
       free ( dummy); dummy = NULL;
    }
    printf ( "%s -- INFO: maximum logical volume size is %s\n",
@@ -426,78 +433,92 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- storing volume group data on disk(s)\n",
                             cmd);
    if ( ( ret = vg_write_with_pv_and_lv ( &vg)) < 0) {
-      fprintf ( stderr,
-                "%s -- ERROR %d storing volume group data "
-                "for %s on disk(s)\n\n",
-                cmd, ret, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
+                        "\"%s\" on disk(s)\n\n",
+                        cmd, ret, vg_name);
+      return LVM_EVGCREATE_VG_WRITE;
    }
 
    if ( opt_v) printf ( "%s -- removing any invalid special files "
-                        "for %s\n", cmd, vg_name);
+                        "of volume group \"%s\"\n", cmd, vg_name);
    vg_remove_dir_and_group_and_nodes ( vg_name);
 
-   if ( opt_v > 0) printf ( "%s -- creating volume group directory /dev/%s\n",
+   if ( opt_v > 0) printf ( "%s -- creating volume group directory "
+                            LVM_DIR_PREFIX
+                            "%s\n",
                             cmd, vg_name);
    if ( ( ret = vg_create_dir_and_group ( &vg)) < 0) {
       if ( ret  == -LVM_EVG_CREATE_DIR_AND_GROUP_MKDIR) {
          fprintf ( stderr, "%s -- problem creating volume group "
-                           "directory /dev/%s\n",
-                   cmd, vg_name);
+                           "directory "
+                           LVM_DIR_PREFIX
+                           "%s\n",
+                           cmd, vg_name);
       } else if ( ret == -LVM_EVG_CREATE_DIR_AND_GROUP_CHMOD_DIR) {
          fprintf ( stderr, "%s -- problem changing permission "
-                           "for volume group directory /dev/%s\n",
-                   cmd, vg_name);
+                           "for volume group directory "
+                           LVM_DIR_PREFIX
+                           "%s\n",
+                           cmd, vg_name);
       } else if ( ret == -LVM_EVG_CREATE_DIR_AND_GROUP_CHMOD_GROUP) {
          fprintf ( stderr, "%s -- problem changing permission "
-                           "for volume group file /dev/%s/group\n",
-                   cmd, vg_name);
+                           "for volume group file "
+                           LVM_DIR_PREFIX
+                           "%s/group\n",
+                           cmd, vg_name);
       } else {
          fprintf ( stderr, "%s -- ERROR %d creating volume group "
-                           "directory /dev/%s\n",
-                   cmd, ret, vg_name);
+                           "directory "
+                           LVM_DIR_PREFIX
+                           "%s\n",
+                           cmd, ret, vg_name);
       }
    }
 
    
    /* create VGDA in kernel */
-   if ( opt_v > 0) printf ( "%s -- creating VGDA for %s in kernel\n",
+   if ( opt_v > 0) printf ( "%s -- creating VGDA for volume group \"%s\" "
+                            "in kernel\n",
                             cmd, vg_name);
    if ( ( ret = vg_create ( vg_name, &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating VGDA for %s in kernel\n",
+      fprintf ( stderr, "%s -- ERROR %d creating VGDA for volume group \"%s\" "
+                        "in kernel\n",
                         cmd, ret, vg_name);
       if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN)
          fprintf ( stderr, "%s -- LVM not in kernel?\n", cmd);
       fprintf ( stderr, "\n");
-      return 1;
+      return LVM_EVGCREATE_VG_CREATE;
    }
 
-   if ( opt_v > 0) printf ( "%s -- inserting %s into lvmtab\n",
-                            cmd, vg_name );
+   if ( opt_v > 0) printf ( "%s -- inserting volume group \"%s\" into \"%s\"\n",
+                            cmd, vg_name, LVMTAB);
    if ( ( ret = lvm_tab_vg_insert ( vg_name)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d inserting %s into %s\n\n",
+      fprintf ( stderr, "%s -- ERROR %d inserting volume group \"%s\" "
+                        "into \"%s\"\n\n",
                         cmd, ret, vg_name, LVMTAB);
-      return 1;
+      return LVM_EVGCREATE_VG_INSERT;
    }
 
    if ( ( ret = vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, &vg)) == 0) {
       if ( opt_A > 0) {
-         printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+         printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+                  cmd, vg_name);
          if ( ( ret = vg_cfgbackup ( vg_name, VG_BACKUP_DIR,
                                      opt_v, &vg)) < 0) {
             fprintf ( stderr, "%s -- ERROR %d writing VG backup\n\n", cmd, ret);
-            return 1;
+            return LVM_EVGCREATE_VG_CFGBACKUP;
          }
       }
    } else {
-      fprintf ( stderr, "%s -- ERROR %d writing %s\n\n", cmd, ret, LVMTAB);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d writing \"%s\"\n\n",
+                        cmd, ret, LVMTAB);
+      return LVM_EVGCREATE_VG_CFGBACKUP_LVMTAB;
    }
 
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
-   printf ( "%s -- volume group %s successfully created and activated\n\n",
+   printf ( "%s -- volume group \"%s\" successfully created and activated\n\n",
             cmd, vg_name);
    return 0;
 }
