@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgdisplay.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * March-July,November 1997
- * January,May,June 1998
+ * January,May,June,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,8 @@
  *    05/16/1998 - added lvmtab check
  *    01/18/1998 - added output of free volume group size to option s output
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    07/08/1998 - use vg_red() instead of lvm_tab_vg_read() with -D
+ *    09/02/1998 - corrected some messages
  *
  */
 
@@ -68,8 +72,8 @@ int main ( int argc, char **argv)
       switch ( c) {
          case 'A':
             if ( opt_A > 0) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_A++;
             break;
@@ -77,8 +81,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -86,8 +90,8 @@ int main ( int argc, char **argv)
 
          case 'D':
             if ( opt_D > 0) {
-               fprintf ( stderr, "%s -- D option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- D option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_D++;
             break;
@@ -113,16 +117,16 @@ int main ( int argc, char **argv)
 
          case 's':
             if ( opt_s > 0) {
-               fprintf ( stderr, "%s -- s option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- s option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_s++;
             break;
 
          case 'v':
             if ( opt_v > 1) {
-               fprintf ( stderr, "%s -- v option yet given two times\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given two times\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -130,18 +134,20 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( opt_s > 0 && opt_v > 0) {
       fprintf ( stderr, "%s -- don't give options s and v together\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( opt_A > 0 && opt_D > 0) {
       fprintf ( stderr, "%s -- option D is not allowed with option A\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    }
 
    if ( optind < argc) {
@@ -150,7 +156,7 @@ int main ( int argc, char **argv)
       if ( opt_A > 0) {
          fprintf ( stderr, "%s -- option A is not allowed with volume group "
                            "names\n\n", cmd);
-         return 1;
+         return LVM_EINVALID_CMD_LINE;
       }
    } else {
       if ( opt_v > 1) {
@@ -168,7 +174,7 @@ int main ( int argc, char **argv)
       printf ( "%s -- no ", cmd);
       if ( opt_A > 0) printf ( "active ");
       printf ( "volume groups found\n\n");
-      return VGDISPLAY_NOVG;
+      return LVM_EVGDISPLAY_NO_VG;
    }
 
    LVM_LOCK ( 1);
@@ -177,24 +183,24 @@ int main ( int argc, char **argv)
    for ( ; optind < argc; optind++) {
       vg_name = argv[optind];
 
-      if ( opt_v > 1) printf ( "%s -- checking volume group name %s\n",
+      if ( opt_v > 1) printf ( "%s -- checking volume group name \"%s\"\n",
                                cmd, vg_name);
       if ( vg_check_name ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- %s is an invalid volume group name\n",
+         fprintf ( stderr, "%s -- \"%s\" is an invalid volume group name\n",
                    cmd, vg_name);
          continue;
       }
 
-      if ( opt_v > 1) printf ( "%s -- checking volume group %s existence\n",
+      if ( opt_v > 1) printf ( "%s -- checking volume group \"%s\" existence\n",
                                cmd, vg_name);
       if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) == TRUE ||
            ret == -LVM_EVG_READ_VG_EXPORTED ||
            ret == -LVM_EVG_READ_LVM_STRUCT_VERSION) {
          if ( ret == -LVM_EVG_READ_VG_EXPORTED)
-            printf ( "%s -- WARNING: volume group %s is exported\n",
+            printf ( "%s -- WARNING: volume group \"%s\" is exported\n",
                      cmd, vg_name);
          if ( opt_s > 0) {
-            printf ( "%s -- %s (", cmd, vg_name);
+            printf ( "%s -- \"%s\" (", cmd, vg_name);
             if ( vg_check_active ( vg_name) != TRUE) printf ( "inactive)\n");
             else  {
                printf ( "active");
@@ -215,15 +221,15 @@ int main ( int argc, char **argv)
          }
          if ( opt_D == 0) {
             if ( opt_v > 1) printf ( "%s -- checking volume group "
-                                     "%s activity\n", cmd, vg_name);
+                                     "\"%s\" activity\n", cmd, vg_name);
             if ( vg_check_active ( vg_name) == TRUE) {
-               if ( opt_v > 1) printf ( "%s -- getting VGDA of %s from "
+               if ( opt_v > 1) printf ( "%s -- getting VGDA of \"%s\" from "
                                         "kernel\n", cmd, vg_name);
                if ( opt_v > 0) {
                   if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg,
                                                           TRUE)) < 0) {
                      fprintf ( stderr,
-                               "%s -- ERROR %d getting status of %s\n",
+                               "%s -- ERROR %d getting status of \"%s\"\n",
                                 cmd, ret, vg_name);
                      continue;
                   }
@@ -239,26 +245,26 @@ int main ( int argc, char **argv)
                   printf ( "\n");
                }
             } else {
-               printf ( "%s -- %s is NOT active; try -D\n\n",
+               printf ( "%s -- \"%s\" is NOT active; try -D\n",
                         cmd, vg_name);
             }
          } else { /* display from disks */
             if ( opt_v > 1) printf ( "%s -- reading volume group data of "
-                                     "%s from disk(s)\n", cmd, vg_name);
+                                     "\"%s\" from disk(s)\n", cmd, vg_name);
             if ( opt_v > 0) {
                if ( ( ret = vg_read_with_pv_and_lv ( vg_name,
                                                      &vg)) < 0 &&
                       ret != -LVM_EVG_READ_VG_EXPORTED) {
-                  exit_code = 1;
+                  exit_code = LVM_EVGDISPLAY_VG_READ;
                   if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION) {
                      fprintf ( stderr,
-                               "%s -- can't show volume group %s with "
+                               "%s -- can't show volume group \"%s\" with "
                                "invalid physical volume version\n",
                                 cmd, vg_name);
                   } else {
                      fprintf ( stderr,
-                               "%s -- ERROR %d getting data of %s form "
-                               "kernel\n",
+                               "%s -- ERROR %d getting data of \"%s\" from "
+                               "disk(s)\n",
                                cmd, ret, vg_name);
                   }
                   continue;
@@ -269,16 +275,21 @@ int main ( int argc, char **argv)
                printf ( "\n");
                pv_show_all_pv_of_vg_short ( vg);
             } else {
-               if ( ( ret = lvm_tab_vg_read ( vg_name, &vg)) < 0) {
-                  exit_code = 1;
+               if ( ( ret = vg_read ( vg_name, &vg)) < 0) {
+                  exit_code = LVM_EVGDISPLAY_VG_READ;
                   if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION) {
                      fprintf ( stderr,
-                               "%s -- can't show volume group %s with "
+                               "%s -- can't show volume group \"%s\" with "
                                "invalid physical volume version\n",
                                 cmd, vg_name);
+                  } else if ( -LVM_EPV_READ_ALL_PV_OF_VG_NP) {
+                     fprintf ( stderr,
+                               "%s -- ERROR: not all physical volumes of \"%s\" "
+                               "online\n",
+                               cmd, vg_name);
                   } else {
                      fprintf ( stderr,
-                               "%s -- ERROR %d reading data of %s "
+                               "%s -- ERROR %d reading data of \"%s\" "
                                "from disk(s)\n",
                                cmd, ret, vg_name);
                   }
@@ -291,17 +302,17 @@ int main ( int argc, char **argv)
             }
          }
       } else if ( ret == -LVM_EPV_READ_ALL_PV_OF_VG_NP) {
-         fprintf ( stderr, "%s -- volume group %s doesn't exist\n",
+         fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
                    cmd, vg_name);
-         exit_code = 1;
+         exit_code = LVM_EVGDISPLAY_VG_EEXIST;
       } else if ( ret == -LVM_EVG_CHECK_EXIST_PV_COUNT) {
          fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
-                           "volume group %s online\n", cmd, vg_name);
-         exit_code = 1;
+                           "volume group \"%s\" online\n", cmd, vg_name);
+         exit_code = LVM_EVGDISPLAY_PV_COUNT;
       } else {
-         fprintf ( stderr, "%s -- volume group %s not found\n",
+         fprintf ( stderr, "%s -- volume group \"%s\" not found\n",
                            cmd, vg_name);
-         exit_code = 1;
+         exit_code = LVM_EVGDISPLAY_VG_NOT_FOUND;
       }
    }
 
