@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgexport.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * May 1997
- * May,June 1998
+ * May,June,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +33,10 @@
  *    05/16/1998 - added lvmtab checking
  *    06/10/1998 - added -a option
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/05/1998 - corrected some messages
+ *               - used lvm_tab_vg_check_exist() instead of vg_check_exist()
+ *               - checked for volume group activity first for
+ *                 better performance
  *
  */
 
@@ -67,8 +73,8 @@ int main ( int argc, char **argv) {
 
          case 'a':
             if ( opt_a > 0) {
-               fprintf ( stderr, "%s -- a option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- a option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_a++;
             break;
@@ -76,8 +82,8 @@ int main ( int argc, char **argv) {
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -102,8 +108,8 @@ int main ( int argc, char **argv) {
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -111,10 +117,12 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( opt_a > 0) {
       argv = lvm_tab_vg_check_exist_all_vg ();
       argc = optind = 0;
@@ -122,11 +130,11 @@ int main ( int argc, char **argv) {
    
       if ( optind == argc) {
          printf ( "%s -- no volume groups found\n\n", cmd);
-         return 1;
+         return LVM_EVGEXPORT_NO_VG;
       }
    } else if ( optind == argc) {
       fprintf ( stderr, "%s -- please give a volume group name\n\n", cmd);
-      return 1;
+      return LVM_EVGEXPORT_VG_MISSING;
    }
 
 
@@ -138,14 +146,21 @@ int main ( int argc, char **argv) {
 
       if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
       if ( vg_check_name ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- invalid volume group name %s\n",
+         fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n",
                    cmd, vg_name);
          continue;
       }
    
+      if ( opt_v > 0) printf ( "%s -- checking volume group activity\n", cmd);
+      if ( vg_check_active ( vg_name) == TRUE) {
+         fprintf ( stderr, "%s -- can't export active volume group \"%s\"\n",
+                   cmd, vg_name);
+         continue;
+      }
+
       if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
-      if ( ( ret = vg_check_exist ( vg_name)) != TRUE) {
-         fprintf ( stderr, "%s -- volume group %s doesn't exist\n",
+      if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) != TRUE) {
+         fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
                    cmd, vg_name);
          continue;
       }
@@ -153,46 +168,42 @@ int main ( int argc, char **argv) {
       sprintf ( vg_name_exp, "%s%s%c", vg_name, EXPORTED, 0);
       if ( ( ret = vg_check_exist ( vg_name_exp)) != TRUE) {
          if ( ret == -LVM_EVG_READ_VG_EXPORTED) {
-            fprintf ( stderr, "%s -- a volume group named %s is "
-                              "allready exported\n",
+            fprintf ( stderr, "%s -- a volume group named \"%s\" is "
+                              "already exported\n",
                               cmd, vg_name);
             continue;
          }
       }
 
-      if ( opt_v > 0) printf ( "%s -- checking volume group activity\n", cmd);
-      if ( vg_check_active ( vg_name) == TRUE) {
-         fprintf ( stderr, "%s -- can't export active volume group %s\n",
-                   cmd, vg_name);
-         continue;
-      }
-
-      if ( opt_v > 0) printf ( "%s -- reading volume group data for %s "
+      if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
                                "from disk(s)\n", cmd, vg_name);
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d reading data of %s\n\n",
-                   cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR %d reading data of volume "
+                           "group \"%s\"\n\n",
+                           cmd, ret, vg_name);
          continue;
       }
 
       if ( opt_v > 0) printf ( "%s -- checking for physical volumes in "
-                               "volume group %s\n", cmd, vg_name);
+                               "volume group \"%s\"\n", cmd, vg_name);
       if ( vg->pv_cur == 0) {
-         fprintf ( stderr, "%s -- no physical volumes in volume group %s\n",
+         fprintf ( stderr, "%s -- no physical volumes in volume group \"%s\"\n",
                    cmd, vg_name);
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
-                               "of %s\n", cmd, vg_name);
+      if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" "
+                               "consistency\n", cmd, vg_name);
       if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d: %s is inconsistent\n\n",
-                   cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" is "
+                           "inconsistent\n\n",
+                           cmd, ret, vg_name);
          continue;
       }
 
       /* set my export marker */
-      if ( opt_v > 0) printf ( "%s -- setting up volume group %s for export\n",
+      if ( opt_v > 0) printf ( "%s -- setting up volume group \"%s\" "
+                               "for export\n",
                                cmd, vg_name);
       system_id_set_exported ( vg->pv[0]->system_id);
       strcat ( vg->pv[0]->vg_name, EXPORTED);
@@ -206,8 +217,8 @@ int main ( int argc, char **argv) {
       vg->vg_status |= VG_EXPORTED;
       strcpy ( vg->vg_name, vg->pv[0]->vg_name);
 
-      if ( opt_v > 0) printf ( "%s -- exporting %s with %u physical "
-                               "and %u logical volumes\n",
+      if ( opt_v > 0) printf ( "%s -- exporting volume group \"%s\" with "
+                               "%u physical and %u logical volumes\n",
                                cmd, vg_name, vg->pv_cur, vg->lv_cur);
    
       lvm_dont_interrupt ( 0);
@@ -217,21 +228,22 @@ int main ( int argc, char **argv) {
                            "data on disk(s)\n", cmd, ret);
       }
 
-      if ( opt_v > 0) printf ( "%s -- removing special files for %s\n",
+      if ( opt_v > 0) printf ( "%s -- removing special files of volume "
+                               "group \"%s\"\n",
                                cmd, vg_name);
       vg_remove_dir_and_group_and_nodes ( vg_name);
    
       if ( ( ret = lvm_tab_vg_remove ( vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing %s from %s\n",
+         fprintf ( stderr, "%s -- ERROR %d removing volume group \"%s\" "
+                           "from \"%s\"\n",
                            cmd, ret, vg_name, LVMTAB);
          continue;
       }
 
       lvm_interrupt ();
 
-      printf ( "%s -- %s", cmd, vg_name);
-      if ( ret < 0) printf ( " NOT");
-      printf ( " sucessfully exported\n");
+      printf ( "%s -- volume group \"%s\" %s  sucessfully exported\n",
+               cmd, vg_name, ret < 0 ? " NOT" : "");
       if ( ret < 0) continue;
 
    }
