@@ -1,8 +1,11 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgextend.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * April 1997
- * May,June 1998
+ * May,June,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +32,8 @@
  *    05/16/1998 - added lvmtab checking
  *    06/08/1998 - checked PV to PE size relation
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/05/1998 - corrected some messages
+ *    01/15/1999 - avoided unecessary volume group exixtence check
  *
  */
 
@@ -75,23 +80,23 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -117,8 +122,8 @@ int main ( int argc, char **argv) {
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -126,82 +131,77 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name"
                         " and a physical volume path\n\n", cmd);
-      return 1;
+      return LVM_EVGEXTEND_VG_MISSING;
    }
    vg_name = argv[optind];
 
    if ( pv_create_kdev_t ( vg_name) != 0) {
       fprintf ( stderr, "%s -- please enter a volume group name first\n\n",
                         cmd);
-      return 1;
+      return LVM_EVGEXTEND_NO_VG_NAME;
    }
 
    optind++;
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a physical volume path\n\n", cmd);
-      return 1;
+      return LVM_EVGEXTEND_PV_MISSING;
    }
 
    /* valid VG name? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group name %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
                             cmd, vg_name);
    if ( vg_check_name ( vg_name) < 0) {
-	  fprintf ( stderr, "%s -- invalid volume group name %s\n\n",
-                cmd, vg_name);
-	  return 1;
+      fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_EVGEXTEND_VG_CHECK_NAME;
    } 
 
-   if ( lvm_tab_vg_check_exist ( vg_name, &vg) != TRUE) {
-      fprintf ( stderr, "%s -- volume group %s doesn't exist\n\n",
+   LVM_LOCK ( 0);
+   LVM_CHECK_IOP;
+
+   if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" existence\n",
+                            cmd, vg_name);
+   if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
+      fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_EVGEXTEND_VG_CHECK_EXIST;
    }
 
    if ( opt_v > 0) printf ( "%s -- checking for inactivity of volume group\n",
                             cmd);
    if ( vg_check_active ( vg_name) != TRUE) {
-      fprintf ( stderr, "%s -- volume group %s must be active for "
+      fprintf ( stderr, "%s -- ERROR: volume group \"%s\" must be active for "
                         "extension\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_EVGEXTEND_VG_CHECK_ACTIVE;
    }
 
-
-   LVM_LOCK ( 0);
-   LVM_CHECK_IOP;
-
-   /* does VG exist? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group %s existence\n",
-                            cmd, vg_name);
-   if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
-	  fprintf ( stderr, "%s -- can't extend: %s doesn't exist\n\n",
-                cmd, vg_name);
-	  return 1;
-   } 
-
    /* read complete VGDA */
-   if ( opt_v > 0) printf ( "%s -- reading volume group %s data from disk(s)\n",
+   if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
+                            "from lvmtab\n",
                             cmd, vg_name);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
       fprintf ( stderr, "%s -- ERROR %d: can't extend;"
-                        " couldn't get volume group data of %s\n\n",
+                        " couldn't get data of volume group \"%s\"\n\n",
                         cmd, ret, vg_name);
-      return 1;
+      return LVM_EVGEXTEND_VG_READ;
    }
 
    if ( ! ( vg->vg_status & VG_EXTENDABLE)) {
-      fprintf ( stderr, "%s -- volume group %s can't be extended "
-                        "(see vgchange(8))\n\n",
+      fprintf ( stderr, "%s -- ERROR: volume group \"%s\" is "
+                        "not extendable\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_EVGEXTEND_NOT_EXTENDABLE;
    }
 
    printf ( "%s -- INFO: maximum logical volume size is %s\n",
@@ -212,7 +212,7 @@ int main ( int argc, char **argv) {
    if ( vg->pv_cur >= vg->pv_max) {
       fprintf ( stderr, "%s -- maximum physical volume count exceeded\n\n",
                  cmd);
-      return 1;
+      return LVM_EVGEXTEND_PV_MAX;
    }
 
    /* read all PVs */
@@ -222,66 +222,72 @@ int main ( int argc, char **argv) {
       fprintf ( stderr, "%s -- ERROR %d: can't extend;"
                         " couldn't read physical volume data\n\n",
                 cmd, ret);
-      return 1;
+      return LVM_EVGEXTEND_PV_READ_ALL_PV;
    }
 
 
-   /* check, if PVs are all defined and new
+   /* check, if PVs are all defined and new,
       and extend them in VG structures */
-   if ( opt_v > 0) printf ( "%s -- extending VGDA structures of %s\n",
+   if ( opt_v > 0) printf ( "%s -- extending VGDA structures of volume "
+                            "group \"%s\"\n",
                             cmd, vg_name);
    /* do the VGDA structure extension */
    if ( ( ret = vg_setup_for_extend ( &argv[optind], argc - optind,
                                       pv, vg, &error_pv_name)) < 0) {
       if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_CHECK_NAME) {
-         fprintf ( stderr, "%s -- physical volume path %s is invalid\n\n",
+         fprintf ( stderr, "%s -- physical volume name \"%s\" is invalid\n\n",
                    cmd, argv[optind]);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_MAX_PV) {
-            fprintf ( stderr, "%s -- maximum physical volume count of "
-                              "volume group %s exceeded\n\n", cmd, vg_name);
+            fprintf ( stderr, "%s -- ERROR: maximum physical volume count of "
+                              "volume group \"%s\" exceeded\n\n",
+                              cmd, vg_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_GET_SIZE) {
             fprintf ( stderr, "%s -- ERROR: getting size of "
-                              "physical volume %s\n\n", cmd, error_pv_name);
+                              "physical volume \"%s\"\n\n",
+                              cmd, error_pv_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_ALLREADY) {
-            fprintf ( stderr, "%s -- ERROR: physical volume %s "
-                              "allready belongs to %s\n\n",
+            fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" "
+                              "already belongs to volume group \"%s\"\n\n",
                               cmd, error_pv_name, vg_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_CHECK_NEW) {
-            fprintf ( stderr, "%s -- no new physical volume %s\n\n",
+            fprintf ( stderr, "%s -- ERROR: \"%s\" is no new physical "
+                              "volume\n\n",
                               cmd, error_pv_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_SIZE_REL) {
-            fprintf ( stderr, "%s -- ERROR: physical volume %s too small "
-                              "for physical extend size of %s\n\n",
+            fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" too small "
+                              "for physical extent size of volume "
+                              "group \"%s\"\n\n",
                                cmd, error_pv_name, vg_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_NO_PV) {
             fprintf ( stderr, "%s -- ERROR: no physical volumes usable to "
-                              "extend %s by\n\n",
+                              "extend volume group \"%s\"\n\n",
                                cmd, vg_name);
       } else {
          fprintf ( stderr, "%s -- ERROR %d setting up volume group "
                            "%s for extend\n\n", cmd, ret, vg_name);
       }
-      return 1;
+      return LVM_EVGEXTEND_VG_SETUP;
    }
 
    /* ret > 0 */
-   if ( opt_v > 0) printf ( "%s -- volume group %s will be extended with "
+   if ( opt_v > 0) printf ( "%s -- volume group \"%s\" will be extended by "
                             "%d new physical volumes\n",
-			                cmd, vg_name, vg->pv_cur - ret);
+                            cmd, vg_name, vg->pv_cur - ret);
 
    lvm_dont_interrupt ( 0);
 
    /* extend vg */
    np = np_sav = ret;
    for ( ; vg->pv[np] != NULL; np++) {
-      if ( opt_v > 0) printf ( "%s -- extending volume group %s with "
-                               "%s in kernel\n",
+      if ( opt_v > 0) printf ( "%s -- extending volume group \"%s\" by "
+                               "physical volume \"%s\" in kernel\n",
                                cmd, vg_name, vg->pv[np]->pv_name);
       if ( ( ret = vg_extend ( vg_name, vg->pv[np], vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d extending %s with %s in kernel\n",
+         fprintf ( stderr, "%s -- ERROR %d extending volume group \"%s\" by "
+                           "physical volume \"%s\" in kernel\n",
                            cmd, ret, vg_name, vg->pv[np]->pv_name);
          for ( p = np_sav; p < np; p++) vg_reduce ( vg_name, vg->pv[p], vg);
-         return 1;
+         return LVM_EVGEXTEND_VG_EXTEND;
       }
    }
 
@@ -289,21 +295,23 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- storing volume group data of "
                             "%s on disk(s)\n", cmd, vg_name);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing volume group data "
-                        "of %s on disk(s)\n\n", cmd, ret, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
+                        "\"%s\" on disk(s)\n\n", cmd, ret, vg_name);
+      return LVM_EVGEXTEND_VG_WRITE;
    }
 
    if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
    if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
         opt_A > 0) {
-      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+      printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+               cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
    }
-   printf ( "%s -- volume group %s successfully extended\n", cmd, vg_name);
+   printf ( "%s -- volume group \"%s\" successfully extended\n", cmd, vg_name);
 
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
+
    if ( opt_v == 0) printf ( "\n");
 
    return 0;
