@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgimport.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * May 1997
- * April-June 1998
+ * April-June,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +35,7 @@
  *    05/16/1998 - added lvmtab checking
  *    05/17/1998 - obsoleted physical volume name correction (now in pv_read())
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/05/1998 - corrected some messages
  *
  */
 
@@ -76,8 +79,8 @@ int main ( int argc, char **argv) {
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -102,8 +105,8 @@ int main ( int argc, char **argv) {
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -111,40 +114,43 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please give a volume group name\n\n", cmd);
-      return 1;
+      return LVM_EVGIMPORT_VG_MISSING;
    }
 
    vg_name = argv[optind++];
    if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
    if ( vg_check_name ( vg_name) < 0) {
-      fprintf ( stderr, "%s -- invalid volume group name %s\n\n",
+      fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_EVGIMPORT_VG_CHECK_NAME;
    }
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a physical volume path\n\n", cmd);
-      return 1;
+      return LVM_EVGIMPORT_PV_MISSING;
    }
 
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
 
-   if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
+   if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" existence\n",
+                            cmd, vg_name);
    if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) == TRUE) {
-      fprintf ( stderr, "%s -- a volume group named %s allready exists\n\n",
+      fprintf ( stderr, "%s -- a volume group named \"%s\" already exists\n\n",
                 cmd, vg_name);
-      return 1;
+      return LVM_EVGIMPORT_VG_CHECK_EXIST;
    }
 
    /* get the physical volume data */
-   if ( opt_v > 0) printf ( "%s -- trying to read the physical volumes\n", cmd);
+   if ( opt_v > 0) printf ( "%s -- trying to read physical volumes\n", cmd);
    p = 0;
    for ( ; optind < argc; optind++) {
       pv_name = argv[optind];
@@ -153,60 +159,61 @@ int main ( int argc, char **argv) {
                                "physical volumes\n", cmd);
       for ( p1 = 0; pv_this != NULL && pv_this[p1] != NULL; p1++) {
          if ( strcmp ( pv_name, pv_this[p1]->pv_name) == 0) {
-            fprintf ( stderr,
-                      "%s -- ERROR: physical volume %s occurs "
-                      "multiple times\n\n",
-                      cmd, pv_name);
-            return 1;
+            fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" occurs "
+                              "multiple times\n\n",
+                               cmd, pv_name);
+            return LVM_EVGIMPORT_PV_MULTIPLE;
          }
       }
 
-      if ( opt_v > 0) printf ( "%s -- checking physical volume path %s\n",
+      if ( opt_v > 0) printf ( "%s -- checking physical volume name \"%s\"\n",
                                cmd, pv_name);
       if ( pv_check_name ( pv_name) < 0) {
-         fprintf ( stderr, "%s -- invalid physical volume path %s\n\n",
+         fprintf ( stderr, "%s -- invalid physical volume name \"%s\"\n\n",
                    cmd, pv_name);
-         return 1;
+         return LVM_EVGIMPORT_PV_CHECK_NAME;
       }
 
-      if ( opt_v > 0) printf ( "%s -- reading physical volume data of %s "
+      if ( opt_v > 0) printf ( "%s -- reading data of physical volume \"%s\" "
                                "from disk\n", cmd, pv_name);
       if ( ( ret = pv_read ( pv_name, &pv, NULL)) < 0 &&
            ret != -LVM_EPV_READ_PV_EXPORTED) {
-         fprintf ( stderr, "%s -- ERROR %d reading physical volume %s\n\n",
-                   cmd, ret, pv_name);
-         return 1;
+         fprintf ( stderr, "%s -- ERROR %d reading data of physical "
+                           "volume \"%s\"\n\n",
+                           cmd, ret, pv_name);
+         return LVM_EVGIMPORT_PV_READ;
       }
 
       if ( opt_v > 0) printf ( "%s -- checking for exported "
-                               "physical volume %s\n", cmd, pv_name);
+                               "physical volume \"%s\"\n",
+                               cmd, pv_name);
       if ( system_id_check_exported ( pv->system_id) == FALSE) {
-         fprintf ( stderr, "%s -- %s doesn't belong to an exported "
-                           "volume group\n\n", cmd, pv_name);
-         return 1;
+         fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" doesn't belong "
+                           "to an exported volume group\n\n",
+                           cmd, pv_name);
+         return LVM_EVGIMPORT_CHECK_EXPORTED;
       }
 
-      if ( opt_v > 0) printf ( "%s -- (re)allocating memory\n", cmd);
-      if ( pv_this == NULL) pv_this = malloc  ( 2 * sizeof ( pv_t*));
+      if ( opt_v > 0) printf ( "%s -- reallocating memory\n", cmd);
       if ( ( pv_this = realloc ( pv_this,
                                  ( p + 2) * sizeof ( pv_t*))) == NULL) {
-         fprintf ( stderr, "%s -- alloc error in file %s [line %d]\n\n",
+         fprintf ( stderr, "%s -- realloc error in file \"%s\" [line %d]\n\n",
                            cmd, __FILE__, __LINE__);
-         return 1;
+         return LVM_EVGIMPORT_REALLOC;
       }
 
       if ( ( pv_this[p] = malloc ( sizeof ( pv_t))) == NULL) {
-         fprintf ( stderr, "%s -- malloc error in file %s [line %d]\n\n",
+         fprintf ( stderr, "%s -- malloc error in file \"%s\" [line %d]\n\n",
                            cmd, __FILE__, __LINE__);
-         return 1;
+         return LVM_EVGIMPORT_MALLOC;
       }
-      bcopy ( pv, pv_this[p], sizeof ( pv_t));
+      memcpy ( pv_this[p], pv, sizeof ( pv_t));
 
       if ( opt_v > 0) printf ( "%s -- checking consistency of physical "
-                               "volume %s\n", cmd, pv_name);
+                               "volume \"%s\"\n", cmd, pv_name);
       if ( ( ret = pv_check_consistency ( pv_this[p])) < 0) {
          fprintf ( stderr, "%s -- ERROR %d checking physical "
-                           "volume consistency of %s\n",
+                           "volume consistency of \"%s\"\n",
                    cmd, ret, pv_name);
          continue;
       }
@@ -217,9 +224,10 @@ int main ( int argc, char **argv) {
    pv_number_this = p;
 
    if ( p == 0) {
-      fprintf ( stderr, "%s -- no valid physical volumes found for "
-                        "import of %s\n\n", cmd, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR: no valid physical volumes found to "
+                        "import volume group \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_EVGIMPORT_NO_PV_FOUND;
    }
 
    /* check volume group consistency of found physical volumes */
@@ -227,10 +235,10 @@ int main ( int argc, char **argv) {
    for ( p = 1; pv_this[p] != NULL; p++) {
       if ( strcmp ( pv_this[0]->vg_name, pv_this[p]->vg_name) != 0 ||
            strcmp ( pv_this[0]->system_id, pv_this[p]->system_id) != 0) {
-         fprintf ( stderr, "%s -- %s doesn't belong to the same exported "
-                           "volume group as %s\n\n",
+         fprintf ( stderr, "%s -- physical volume \"%s\" doesn't belong to "
+                           "the same exported volume group as \"%s\"\n\n",
                    cmd, pv_this[p]->pv_name, pv_this[0]->pv_name);
-         return 1;
+         return LVM_EVGIMPORT_VG_DIFF;
       }
       /* get the highest pv_number */
       if ( pv_number < pv_this[p]->pv_number) pv_number = pv_this[p]->pv_number;
@@ -238,15 +246,16 @@ int main ( int argc, char **argv) {
 
    if ( pv_number != pv_number_this) {
       fprintf ( stderr, "%s -- ERROR: wrong number of physical volumes "
-                        "given to import %s\n\n", cmd, vg_name);
-      return 1;
+                        "to import volume group \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_EVGIMPORT_PV_COUNT;
    }
 
    /* get the physical volume numbers counter array */
    if ( ( pv_number_count = malloc ( pv_number * sizeof ( int))) == NULL) {
-      fprintf ( stderr, "%s -- malloc error in file %s [line %d]\n\n",
+      fprintf ( stderr, "%s -- malloc error in file \"%s\" [line %d]\n\n",
                         cmd, __FILE__, __LINE__);
-      return 1;
+      return LVM_EVGIMPORT_MALLOC;
    }
    memset ( pv_number_count, 0, pv_number * sizeof ( int));
 
@@ -256,11 +265,11 @@ int main ( int argc, char **argv) {
 
    for ( p = 0; p < pv_number; p++) {
       if ( pv_number_count[p] != 1) {
-         fprintf ( stderr, "%s -- physical volume number %u "
-                           "of %s occurs %d times\n\n",
+         fprintf ( stderr, "%s -- ERROR: physical volume number %u "
+                           "of \"%s\" occurs %d times\n\n",
                             cmd, pv_this[p]->pv_number,
                             pv_this[p]->pv_name, pv_number_count[p]);
-         return 1;
+         return LVM_EVGIMPORT_PV_MULTIPLE;
       }
    }
    free ( pv_number_count);
@@ -271,56 +280,59 @@ int main ( int argc, char **argv) {
    strncpy ( vg_name_this,
              pv_this[0]->vg_name,
              strlen ( pv_this[0]->vg_name) - strlen ( EXPORTED));
-   if ( opt_v > 0) printf ( "%s -- reading the exported volume group data "
-                            " of %s from disk(s)\n", cmd, vg_name_this);
+   if ( opt_v > 0) printf ( "%s -- reading exported volume group data "
+                            " of \"%s\" from disk(s)\n", cmd, vg_name_this);
    if ( ( ret = vg_read_with_pv_and_lv ( pv_this[0]->vg_name, &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d reading volume group data of "
-                        "exported volume group %s\n\n",
-                cmd, ret, vg_name_this);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d reading data of "
+                        "exported volume group \"%s\"\n\n",
+                        cmd, ret, vg_name_this);
+      return LVM_EVGIMPORT_VG_READ;
    }
 
    if ( pv_number_this != vg->pv_cur) {
       fprintf ( stderr, "%s -- ERROR: only %d physical volumes of %u "
-                        "given for volume group %s\n\n",
+                        "given for volume group \"%s\"\n\n",
                         cmd, pv_number_this, vg->pv_cur, vg_name);
-      return 1;
+      return LVM_EVGIMPORT_PV_COUNT;
    }
 
    /* continue checking consistency in red exported volume group */
    if ( opt_v > 0) printf ( "%s -- checking for correct system ids on "
-                            "physical volumes of %s\n", cmd, vg->vg_name);
+                            "physical volumes of volume group \"%s\"\n",
+                            cmd, vg->vg_name);
    for ( p = 1; vg->pv[p] != NULL; p++) {
       if ( strcmp ( vg->pv[0]->vg_name, vg->pv[p]->vg_name) != 0 ||
            strcmp ( vg->pv[0]->system_id, vg->pv[p]->system_id) != 0) {
-         fprintf ( stderr, "%s -- %s has a different system_id than %s\n\n",
-                   cmd, vg->pv[p]->pv_name, vg->pv[0]->pv_name);
-         return 1;
+         fprintf ( stderr, "%s -- physical volume \"%s\" has a different "
+                           "system_id than \"%s\"\n\n",
+                           cmd, vg->pv[p]->pv_name, vg->pv[0]->pv_name);
+         return LVM_EVGIMPORT_VG_DIFF;
       }
    }
 
    if ( opt_v > 0) printf ( "%s -- checking exported volume group "
-                            "%s consistency\n", cmd, vg->vg_name);
+                            "\"%s\" consistency\n", cmd, vg->vg_name);
    if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0 &&
         ret != -LVM_EVG_CHECK_CONSISTENCY_VG_STATUS) {
-      fprintf ( stderr, "%s -- ERROR %d: exported volume group %s "
+      fprintf ( stderr, "%s -- ERROR %d: exported volume group \"%s\" "
                         "is inconsistent\n"
                         "%s -- it's system id is \"%s\"\n\n",
                         cmd, ret, vg_name_this, cmd, vg->pv[0]->system_id);
-      return 1;
+      return LVM_EVGIMPORT_VG_CHECK_CONSISTENCY;
    }
 
    if ( vg->lv_cur > MAX_LV || vg->lv_max > MAX_LV) {
       fprintf ( stderr, "%s -- too many logical volumes to import in "
-                        "exported volume group %s\n\n",
+                        "exported volume group \"%s\"\n\n",
                         cmd, vg_name_this);
-      return 1;
+      return LVM_EVGIMPORT_MAX_LV;
    }
 
    /* correct the volume group name */
    strcpy ( vg->vg_name, vg_name);
 
-   if ( opt_v > 0) printf ( "%s -- importing physical volumes of %s\n",
+   if ( opt_v > 0) printf ( "%s -- importing physical volumes of volume "
+                            "group \"%s\"\n",
                             cmd, vg->vg_name);
    for ( p = 0; vg->pv[p] != NULL; p++) {
       system_id_set_imported ( vg->pv[p]->system_id);
@@ -329,16 +341,18 @@ int main ( int argc, char **argv) {
 
    lvm_dont_interrupt ( 0);
 
-   if ( opt_v > 0) printf ( "%s -- getting free volume group number for %s\n\n",
+   if ( opt_v > 0) printf ( "%s -- getting free number for volume "
+                            "group \"%s\"\n\n",
                             cmd, vg_name);
    if ( ( vg->vg_number = lvm_tab_get_free_vg_number ()) < 0) {
-      fprintf ( stderr, "%s -- ERROR getting free volume group "
-                        "number for %s\n\n",
-                cmd, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR getting free number for volume "
+                        "group \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_EVGIMPORT_GET_FREE_VG_NUMBER;
    }
 
-   if ( opt_v > 0) printf ( "%s -- importing logical volumes of %s\n",
+   if ( opt_v > 0) printf ( "%s -- importing logical volumes of volume "
+                            "group \"%s\"\n",
                             cmd, vg->vg_name);
    for ( l = 0; l < vg->lv_max; l++) {
       if ( vg->lv[l] != NULL) {
@@ -351,22 +365,14 @@ int main ( int argc, char **argv) {
 
    vg->vg_status &= ~VG_EXPORTED;
 
-/* TEST without path information ( lv_create_kdev_t() won't work...)
-   for ( l = 0; l < vg->lv_max && vg->lv[l] != NULL; l++) {
-      if ( ( lv_name_ptr = strrchr ( vg->lv[l]->lv_name, '/')) != NULL) {
-         lv_name_ptr++;
-         strcpy ( vg->lv[l]->lv_name, lv_name_ptr);
-      }
-   }
-*/
-
-   if ( opt_v > 0) printf ( "%s -- storing imported volume group data of %s "
-                            "on disk(s)\n", cmd, vg_name);
+   if ( opt_v > 0) printf ( "%s -- storing data of imported volume group "
+                            "\"%s\" " "on disk(s)\n",
+                            cmd, vg_name);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing imported volume group data "
-                        "of %s on disks\n\n",
-                cmd, ret, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d storing data of imported volume "
+                        "\"%s\" on disks\n\n",
+                        cmd, ret, vg_name);
+      return LVM_EVGIMPORT_VG_WRITE;
    }
 
    if ( ( ret = vg_create_dir_and_group_and_nodes ( vg, opt_v)) < 0)
@@ -376,39 +382,41 @@ int main ( int argc, char **argv) {
    /* create VGDA in kernel */
    if ( opt_v > 0) printf ( "%s -- creating VGDA in kernel\n", cmd);
    if ( ( ret = vg_create ( vg_name, vg)) < 0) {
-      fprintf ( stderr, "%s -- couldn't create VGDA for imported volume "
-                        "group %s in kernel\n", cmd, vg_name);
+      fprintf ( stderr, "%s -- ERROR: couldn't create VGDA for imported volume "
+                        "group \"%s\" in kernel\n", cmd, vg_name);
       if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN)
          fprintf ( stderr, "%s -- LVM not in kernel?\n", cmd);
       fprintf ( stderr, "\n");
-      return 1;
+      return LVM_EVGIMPORT_VG_CREATE;
    }
 
-   if ( opt_v > 0) printf ( "%s -- inserting %s into %s\n",
+   if ( opt_v > 0) printf ( "%s -- inserting volume group \"%s\" into \"%s\"\n",
                             cmd, vg_name, LVMTAB);
    if ( ( ret = lvm_tab_vg_insert ( vg_name)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d inserting %s into %s\n\n",
+      fprintf ( stderr, "%s -- ERROR %d inserting volume group \"%s\" "
+                        "into \"%s\"\n\n",
                         cmd, ret, vg_name, LVMTAB);
-      return 1;
+      return LVM_EVGIMPORT_VG_INSERT;
    }
 
    if ( ( ret = vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg)) == 0) {
-      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+      printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+               cmd, vg_name);
       if ( ( ret = vg_cfgbackup ( vg_name, VG_BACKUP_DIR,
                                   opt_v, vg)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d writing VG backup\n\n", cmd, ret);
-         return 1;
+         return LVM_EVGIMPORT_VG_CFGBACKUP;
       }
    } else {
       fprintf ( stderr, "%s -- ERROR %d writing lvmtab\n\n", cmd, ret);
-      return 1;
+      return LVM_EVGIMPORT_VG_CFGBACKUP_LVMTAB;
    }
 
 
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
-   printf ( "%s -- volume group %s successfully imported and activated\n\n",
+   printf ( "%s -- volume group \"%s\" successfully imported and activated\n\n",
             cmd, vg_name);
 
    return 0;
