@@ -1,7 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgmknodes.c
  *
- * May-June 1998
+ * Copyright (C) 1997 - 1998  Heinz Mauelshagen, Germany
+ *
+ * May-June,September 1998
+ * January 1999
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +27,8 @@
  * Changelog
  *
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/05/1998 - corrected some messages
+ *    01/26/1999 - made volume group directory prefix a preprocessor option
  *
  */
 
@@ -38,6 +43,7 @@ int opt_d = 0;
 int main ( int argc, char **argv)
 {
    int c = 0;
+   int length = 0;
    int opt_v = 0;
    int ret = 0;
 #ifdef DEBUG
@@ -58,7 +64,7 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
                return 1;
             }
             opt_d++;
@@ -78,12 +84,12 @@ int main ( int argc, char **argv)
                      "\t[-v]\n"
                      "\t[VolumeGroupName...]\n\n",
                      lvm_version, cmd, cmd);
-            return 0;
+            return LVM_EINVALID_CMD_LINE;
             break;
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
                return 1;
             }
             opt_v++;
@@ -92,57 +98,65 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
   
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       argv = lvm_tab_vg_check_exist_all_vg ();
       argc = optind = 0;
       if ( argv != NULL) while ( argv[argc] != NULL) argc++;
+      if ( argc == 0) {
+         printf ( "%s -- no volume groups found to create nodes for\n\n", cmd);
+         return 0;
+      }
    }
 
-   if ( argc == 0) {
-      printf ( "%s -- no volume groups to create nodes for\n\n", cmd);
-      return 0;
-   }
 
    for ( ; optind < argc; optind++) {
       vg_name = argv[optind];
    
-      if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
+      if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
+                               cmd, vg_name);
       if ( vg_check_name ( vg_name) < 0) { 
-         fprintf ( stderr, "%s -- invalid volume group name %s\n\n",
+         fprintf ( stderr, "%s -- ERROR: invalid volume group name \"%s\"\n\n",
                            cmd, vg_name);
-         return 1;
+         return LVM_EVGMKNODES_VG_CHECK_NAME;
       }
-      if ( strncmp ( vg_name, "/dev/", 5) == 0) vg_name += 5;
+      length = strlen ( LVM_DIR_PREFIX);
+      if ( strncmp ( vg_name, LVM_DIR_PREFIX, length) == 0) vg_name += length;
    
-      if ( lvm_tab_vg_check_exist ( vg_name, &vg) != TRUE) {
-         fprintf ( stderr, "%s -- volume group %s doesn't exist\n\n",
+      if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" existence\n",
+                               cmd, vg_name);
+      if ( lvm_tab_vg_check_exist ( vg_name, &vg) == FALSE) {
+         fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n\n",
                            cmd, vg_name);
-         return 1;
+         return LVM_EVGMKNODES_VG_CHECK_EXIST;
       }
    
-      if ( opt_v > 0) printf ( "%s -- removing directory and nodes of %s\n",
+      if ( opt_v > 0) printf ( "%s -- removing directory and nodes of "
+                               "volume group \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = vg_remove_dir_and_group_and_nodes ( vg_name)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d: removing volume group "
-                           "nodes of %s\n\n",
+                           "nodes of \"%s\"\n\n",
                            cmd, ret, vg_name);
-         return 1;
+         return LVM_EVGMKNODES_VG_REMOVE_DIR_AND_GROUP_NODES;
       }
    
-      if ( opt_v > 0) printf ( "%s -- creating directory and nodes of %s\n",
+      if ( opt_v > 0) printf ( "%s -- creating directory and nodes of "
+                               "volume group \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = vg_create_dir_and_group_and_nodes ( vg, opt_v)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d: creating volume group "
-                           "nodes of %s\n\n",
+                           "nodes of \"%s\"\n\n",
                            cmd, ret, vg_name);
-         return 1;
+         return LVM_EVGMKNODES_VG_CREATE_DIR_AND_GROUP_NODES;
       }
    
-      printf ( "%s -- successfully made all nodes of volume group %s\n",
+      printf ( "%s -- successfully made all nodes of volume group \"%s\"\n",
                cmd, vg_name);
    }
 
