@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgreduce.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * April,October 1997
- * May,June 1998
+ * May,June,September 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +31,7 @@
  *    05/09/1998 - check for volume group beeing reducable
  *    05/16/1998 - added lvmtab checking
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/05/1998 - corrected some messages
  *
  */
 
@@ -44,7 +47,6 @@ int main ( int argc, char **argv) {
    int error = 0;
    int p = 0;
    int np = 0;
-   int np_reduced = 0;
    int opt_a = 0;
    int opt_A = 1;
    int opt_v = 0;
@@ -70,31 +72,31 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'a':
             if ( opt_a > 0) {
-               fprintf ( stderr, "%s -- a option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- a option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_a++;
             break;
 
          case 'A':
             if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
+               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
                                  cmd, optarg);
-               return 1;
+               return LVM_EINVALID_CMD_LINE;
             }
             break;
 
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -121,8 +123,8 @@ int main ( int argc, char **argv) {
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -130,86 +132,87 @@ int main ( int argc, char **argv) {
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name"
                          " and physical volume paths\n\n", cmd);
-      return 1;
+      return LVM_EVGREDUCE_VG_PV_NAMES;
    }
 
    vg_name = argv[optind++];
 
    /* valid VG name? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group name %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
                             cmd, vg_name);
    if ( vg_check_name ( vg_name) < 0) {
-	  fprintf ( stderr, "%s -- invalid volume group name %s\n\n",
-                cmd, vg_name);
-	  return 1;
+      fprintf ( stderr, "%s -- ERROR: invalid volume group name \"%s\"\n\n",
+                        cmd, vg_name);
+      return LVM_EVGREDUCE_VG_CHECK_NAME;
    } 
 
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
 
    /* does VG exist? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group name %s\n",
+   if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" existence\n",
                             cmd, vg_name);
    if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
-	  fprintf ( stderr, "%s -- volume group %s doesn't exist\n\n",
-                cmd, vg_name);
-	  return 1;
+      fprintf ( stderr, "%s -- ERROR: volume group \"%s\" doesn't exist\n\n",
+                        cmd, vg_name);
+      return LVM_EVGREDUCE_VG_CHECK_EXIST;
    } 
 
    /* is VG active? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group %s activity\n",
+   if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" activity\n",
                             cmd, vg_name);
-   if ( vg_check_active ( vg_name) != TRUE) {
-	  fprintf ( stderr, "%s -- volume group %s is not active\n\n",
-                cmd, vg_name);
-	  return 1;
+   if ( vg_check_active ( vg_name) == FALSE) {
+      fprintf ( stderr, "%s -- ERROR: volume group \"%s\" is not active\n\n",
+                        cmd, vg_name);
+      return LVM_EVGREDUCE_VG_CHECK_ACTIVE;
    } 
 
    /* read complete VGDA */
-   if ( opt_v > 0) printf ( "%s -- reading volume group data of %s "
+   if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
                             "from disk(s)\n", cmd, vg_name);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: can't reduce;"
-                        " couldn't get volume group data of %s\n\n",
+      fprintf ( stderr, "%s -- ERROR %d: couldn't get data of volume "
+                        "group \"%s\"\n\n",
                         cmd, ret, vg_name);
-      return 1;
+      return LVM_EVGREDUCE_VG_READ;
    }
 
    if ( ! ( vg->vg_status & VG_EXTENDABLE)) {
-      fprintf ( stderr, "%s -- volume group %s can't be reduced "
-                        "(see vgchange(8))\n\n",
+      fprintf ( stderr, "%s -- ERROR: volume group \"%s\" is not reducable\n\n",
                         cmd, vg_name);
-      return 1;
+      return LVM_EVGREDUCE_VG_NOT_REDUCABLE;
    }
 
    if ( optind == argc) {
       if ( opt_a == 0) {
          fprintf ( stderr, "%s -- please enter physical volume paths "
                            "or option -a\n\n", cmd);
-         return 1;
+         return LVM_EVGREDUCE_PV_NAME;
       } else {
          np = 0;
          pv_names = NULL;
          for ( p = 0; p < vg->pv_max; p++) {
             if ( vg->pv[p] != NULL && vg->pv[p]->pe_allocated == 0) {
-               pv_names = realloc ( pv_names, sizeof ( char*) * ( np + 2));
-               if ( pv_names == NULL) {
-                  fprintf ( stderr, "%s -- realloc error in file %s "
+               if ( ( pv_names = realloc ( pv_names,
+                                           sizeof ( char*) *
+                                           ( np + 2))) == NULL) {
+                  fprintf ( stderr, "%s -- realloc error in file \"%s\" "
                                     "[line %d]\n\n", cmd, __FILE__, __LINE__);
-                  return 1;
+                  return LVM_EVGREDUCE_REALLOC;
                }
-               pv_names[np] = malloc ( NAME_LEN);
-               if ( pv_names[np] == NULL) {
-                  fprintf ( stderr, "%s -- malloc error in file %s "
+               if ( ( pv_names[np] = malloc ( NAME_LEN)) == NULL) {
+                  fprintf ( stderr, "%s -- malloc error in file \"%s\" "
                                     "[line %d]\n\n", cmd, __FILE__, __LINE__);
-                  return 1;
+                  return LVM_EVGREDUCE_MALLOC;
                }
                strcpy ( pv_names[np], vg->pv[p]->pv_name);
                np++;
@@ -217,9 +220,10 @@ int main ( int argc, char **argv) {
             }
          }
          if ( np == 0) {
-            fprintf ( stderr, "%s -- no empty physical volumes found "
-                              "in volume group %s\n\n", cmd, vg_name);
-            return 1;
+            fprintf ( stderr, "%s -- ERROR: no empty physical volumes found "
+                              "in volume group \"%s\"\n\n",
+                              cmd, vg_name);
+            return LVM_EVGREDUCE_NO_EMPTY_PV;
          }
          if ( np == vg->pv_cur) {
             np--;
@@ -228,7 +232,9 @@ int main ( int argc, char **argv) {
          }
          if ( opt_v > 0) {
             for ( p = 0; p < np; p++) {
-               printf ( "%s -- reducing %s by %s\n", cmd, vg_name, pv_names[p]);
+               printf ( "%s -- reducing volume group \"%s\" by physical "
+                        "volume \"%s\"\n",
+                        cmd, vg_name, pv_names[p]);
             }
          }
          optind = 0; argc = np;
@@ -236,52 +242,53 @@ int main ( int argc, char **argv) {
    } else if ( opt_a != 0) {
       fprintf ( stderr, "%s -- you can't give option -a AND enter "
                         "physical volume paths\n\n", cmd);
-      return 1;
+      return LVM_EINVALID_CMD_LINE;
    } else {
       pv_names = argv + optind;
       argc -= optind;
    }
 
+   np = vg->pv_cur;
+
    /* check, if PVs are all defined and in this volume group
       and reduce them in VG structures */
-   if ( opt_v > 0) printf ( "%s -- checking physical volumes "
-                            "against volume group %s\n",
+   if ( opt_v > 0) printf ( "%s -- reducing VGDA structures of volume "
+                            "group \"%s\" \n",
                             cmd, vg_name);
    if ( ( ret = vg_setup_for_reduce ( pv_names, argc,
                                       vg, &pv_this, &error_pv_name)) < 0) {
       if ( ret == -LVM_EVG_SETUP_FOR_REDUCE_PV_INVALID) {
-         fprintf ( stderr, "%s -- invalid physical volume name %s\n\n",
+         fprintf ( stderr, "%s -- ERROR: invalid physical volume "
+                           "name \"%s\"\n\n",
                            cmd, error_pv_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_REDUCE_LV) {
-         fprintf ( stderr, "%s -- can't reduce %s by "
-                           "%s with logical volume(s)\n\n",
+         fprintf ( stderr, "%s -- ERROR: can't reduce volume group \"%s\" by "
+                           "used physical volume \"%s\"\n\n",
                            cmd, vg_name, error_pv_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_REDUCE_LAST_PV) {
-         fprintf ( stderr, "%s -- can't reduce volume group %s "
+         fprintf ( stderr, "%s -- can't reduce volume group \"%s\" "
                            "to zero physical volumes\n\n", cmd, vg_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_REDUCE_LAST_PV_NOT_IN_VG) {
-         fprintf ( stderr,
-                   "%s -- physical volume %s doesn't belong to volume "
-                   "group %s\n",
-                   cmd, error_pv_name, vg_name);
+         fprintf ( stderr, "%s -- physical volume \"%s\" doesn't belong "
+                           "to volume group \"%s\"\n",
+                           cmd, error_pv_name, vg_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_REDUCE_NO_PV_TO_REDUCE) {
          fprintf ( stderr, "%s -- no physical volumes found to "
-                           "reduce %s by\n\n",
+                           "reduce \"%s\"\n\n",
                            cmd, vg_name);
       } else {
-         fprintf ( stderr,
-                   "%s -- ERROR %d reducing VGDA structures for volume "
-                   "group %s\n",
-                   cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR %d reducing VGDA structures of volume "
+                           "group \"%s\"\n",
+                           cmd, ret, vg_name);
       }
-      return 1;
+      return LVM_EVGREDUCE_VG_SETUP;
    }
 
-   np_reduced = ret;
+   np -= ret;
    if ( opt_v > 0) {
-      printf ( "%s -- volume group %s will be reduced by %d physical volume",
-               cmd, vg_name, np_reduced);
-      if ( np_reduced > 1) putchar ( 's');
+      printf ( "%s -- volume group \"%s\" will be reduced by "
+               "%d physical volume%s\n",
+               cmd, vg_name, np, np > 1 ? "s" : "");
       printf ( "\n");
    }
 
@@ -292,53 +299,55 @@ int main ( int argc, char **argv) {
                             "in kernel\n", cmd);
    for ( p = 0; pv_this[p] != NULL; p++) {
       if ( ( ret = vg_reduce ( vg_name, pv_this[p], vg)) < 0) {
-         fprintf ( stderr,
-                   "%s -- ERROR %d reducing %s by %s in kernel\n",
-                   cmd, ret, vg_name, pv_this[p]->pv_name);
-         return 1;
+         fprintf ( stderr, "%s -- ERROR %d reducing volume group \"%s\" "
+                           "by physical volume \"%s\" in kernel\n",
+                           cmd, ret, vg_name, pv_this[p]->pv_name);
+         return LVM_EVGREDUCE_VG_REDUCE;
       }
    }
 
    /* store vg on disk(s) */
-   if ( opt_v > 0) printf ( "%s -- storing volume group data of %s "
+   if ( opt_v > 0) printf ( "%s -- storing data of volume group \"%s\" "
                             "on disk(s)\n", cmd, vg_name);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-      fprintf ( stderr,
-                "%s -- ERROR %d storing volume group data "
-                "for %s on disk(s)\n\n",
-                cmd, ret, vg_name);
-      return 1;
+      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
+                        "\"%s\" on disk(s)\n\n",
+                        cmd, ret, vg_name);
+      return LVM_EVGREDUCE_VG_WRITE;
    }
 
    /* setup freed PVs and store them */
    error = 0;
    for ( p = 0; pv_this[p] != NULL; p++) {
+      if ( opt_v > 0) printf ( "%s -- initializing reduced physical "
+                               "volume \"%s\"\n",
+                               cmd, pv_this[p]->pv_name);
       pv_setup_for_create ( pv_this[p]->pv_name, pv_this[p],
                             pv_this[p]->pv_size);
       if ( ( ret = pv_write ( pv_this[p]->pv_name, pv_this[p])) < 0) {
-         fprintf ( stderr,
-                   "%s -- ERROR %d storing physical volume %s\n",
-                   cmd, ret, pv_this[p]->pv_name);
+         fprintf ( stderr, "%s -- ERROR %d storing physical volume \"%s\"\n",
+                           cmd, ret, pv_this[p]->pv_name);
          error++;
       }
    }
 
-   if ( error > 0) fprintf ( stderr, "%s -- %d errors storing physical "
-                                     "volume data for %s\n",
+   if ( error > 0) fprintf ( stderr, "%s -- %d errors initializing reduced "
+                                     "physical volume data of \"%s\"\n",
                                      cmd, error, vg_name);
+
    if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
    if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
         opt_A > 0) {
-      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
+      printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
+               cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
    }
 
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
-   printf ( "%s -- %s ", cmd, vg_name);
-   if ( error > 0) printf ( "NOT ");
-   printf ( "successfully reduced by ");
+   printf ( "%s -- volume group \"%s\" %ssuccessfully reduced by ",
+            cmd, vg_name, error > 0 ? "NOT " : "");
    for ( p = 0; pv_this[p] != NULL; p++) {
       printf ( "%s", pv_this[p]->pv_name);
       if ( pv_this[p+1] != NULL) printf ( ", ");
