@@ -1,8 +1,10 @@
 /*
- * Author: Heinz Mauelshagen, Germany
+ * tools/vgremove.c
+ *
+ * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
  *
  * February, October 1997
- * May-June 1998
+ * May-June,September 1998
  *
  * vgremove is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,7 @@
  *    06/13/1998 - change to use lvm_tab_vg_read_with_pv_and_lv()
  *                 instead of vg_read() and pv_read_all_pv_of_vg()
  *    06/27/1998 - changed lvm_tab_* calling convention
+ *    09/05/1998 - corrected some messages
  *
  */
 
@@ -66,8 +69,8 @@ int main ( int argc, char **argv)
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_d++;
             break;
@@ -91,8 +94,8 @@ int main ( int argc, char **argv)
 
          case 'v':
             if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
-               return 1;
+               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
             }
             opt_v++;
             break;
@@ -100,13 +103,15 @@ int main ( int argc, char **argv)
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return 1;
+            return LVM_EINVALID_CMD_LINE;
       }
    }
 
+   CMD_MINUS_CHK;
+
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name\n\n", cmd);
-      return 1;
+      return LVM_EVGREMOVE_VG_NAME;
    }
 
    LVM_LOCK ( 0);
@@ -114,97 +119,109 @@ int main ( int argc, char **argv)
 
    for ( ; optind < argc; optind++) {
       vg_name = argv[optind];
-
-      if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
+      if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
+                              cmd, vg_name);
       if ( vg_check_name ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- no valid volume group name %s\n",
+         fprintf ( stderr, "%s -- ERROR: invalid volume group name \"%s\"\n",
+                           cmd, vg_name);
+         continue;
+      }
+   
+      if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" activity\n",
+                               cmd, vg_name);
+      if ( vg_check_active ( vg_name) == TRUE) {
+         fprintf ( stderr, "%s -- ERROR: can't remove active volume "
+                           "group \"%s\"\n",
                            cmd, vg_name);
          continue;
       }
    
       /* does VG exist? */
-      if ( opt_v > 0) printf ( "%s -- checking volume group %s existence\n",
+      if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" existence\n",
                                cmd, vg_name);
       if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) != TRUE) {
          if ( ret == -LVM_EVG_CHECK_EXIST_PV_COUNT) {
-            fprintf ( stderr, "%s -- not all physical volumes of volume group "
-                              "%s are online\n", cmd, vg_name);
+            fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
+                              "volume group \"%s\" are online\n",
+                              cmd, vg_name);
          } else if ( ret == FALSE) {
-            fprintf ( stderr, "%s -- volume group %s doesn't exist\n",
+            fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
                               cmd, vg_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d: volume group %s doesn't exist\n",
+            fprintf ( stderr, "%s -- ERROR %d: checking existence of "
+                              "volume group \"%s\"\n",
                               cmd, ret, vg_name);
          }
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking volume group %s activity\n",
-                               cmd, vg_name);
-      if ( vg_check_active ( vg_name) == TRUE) {
-         fprintf ( stderr, "%s -- can't remove active volume group %s\n",
-                           cmd, vg_name);
-         continue;
-      }
-   
-      if ( opt_v > 0) printf ( "%s -- reading data of volume group %s\n",
-                               cmd, vg_name);
+      if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
+                               "from \"%s\"\n",
+                               cmd, vg_name, LVMTAB);
       if ( lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: can't read volume group descriptor "
-                           " area of volume group %s from lvmtab\n",
-                           cmd, ret, vg_name);
-         continue;
-      }
-   
-      if ( opt_v > 0) printf ( "%s -- checking for logical volumes in "
-                               "volume group %s\n",
-                               cmd, vg_name);
-      if ( vg->lv_cur > 0) {
-         fprintf ( stderr, "%s -- can't remove volume group %s with"
-                           " %u logical volume", cmd, vg_name, vg->lv_cur);
-         if ( vg->lv_cur > 1) fprintf ( stderr, "s\n");
-         continue;
-      }
-   
-      /* init physical volumes */
-      if ( opt_v > 0) printf ( "%s -- giving free physical volumes of "
-                               "volume group %s\n",
-                               cmd, vg_name);
-      for ( p = 0; vg->pv[p] != NULL; p++) {
-         if ( opt_v > 0) printf ( "%s -- freeing physical volume %s\n",
-                                  cmd, pv[p]->pv_name);
-         pv_setup_for_create ( vg->pv[p]->pv_name, vg->pv[p],
-                               vg->pv[p]->pv_size);
-      }
-   
-      if ( opt_v > 0) printf ( "%s -- storing physical volume data of "
-                               "volume group %s on disk(s)\n",
-                               cmd, vg_name);
-   
-      lvm_dont_interrupt ( 0);
-   
-      if ( ( ret = pv_write_all_pv_of_vg ( vg)) != 0) {
-         fprintf ( stderr, "%s -- can't remove all %u physical volumes "
-                           "of volume group %s\n\n",
-                           cmd, vg->pv_cur, vg_name);
-         return 1;
-      }
-   
-      if ( opt_v > 0) printf ( "%s -- removing special files of "
-                               "volume group %s\n", cmd, vg_name);
-      vg_remove_dir_and_group_and_nodes ( vg_name);
-   
-   
-      if ( opt_v > 0) printf ( "%s -- removing lvmtab of volume group %s",
-                               cmd, vg_name);
-      if ( ( ret = lvm_tab_vg_remove ( vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing %s from %s\n",
+         fprintf ( stderr, "%s -- ERROR %d: can't get data of volume group "
+                           "\"%s\" from \"%s\"\n",
                            cmd, ret, vg_name, LVMTAB);
          continue;
       }
    
-      printf ( "%s -- volume group %s successfully removed\n", cmd, vg_name);
-
+      if ( opt_v > 0) printf ( "%s -- checking for logical volumes in "
+                               "volume group \"%s\"\n",
+                               cmd, vg_name);
+      if ( vg->lv_cur > 0) {
+         fprintf ( stderr, "%s -- ERROR: can't remove volume group \"%s\" with"
+                           " %u logical volume%s\n",
+                           cmd, vg_name, vg->lv_cur, vg->lv_cur > 1 ? "s" : "");
+         continue;
+      }
+   
+      /* init physical volumes */
+      if ( opt_v > 0) printf ( "%s -- initializing all physical volumes of "
+                               "volume group \"%s\"\n",
+                               cmd, vg_name);
+      for ( p = 0; vg->pv[p] != NULL; p++) {
+         if ( opt_v > 0) printf ( "%s -- initializing physical volume \"%s\"\n",
+                                  cmd, pv[p]->pv_name);
+         if ( ( ret = pv_setup_for_create ( vg->pv[p]->pv_name, vg->pv[p],
+                                            vg->pv[p]->pv_size)) < 0) {
+            fprintf ( stderr, "%s -- ERROR %d initializing physical "
+                              "volume \"%s\"\n\n",
+                              cmd, ret,  vg->pv[p]->pv_name);
+            return LVM_EVGREMOVE_PV_SETUP;
+         }
+      }
+   
+      lvm_dont_interrupt ( 0);
+      if ( opt_v > 0) printf ( "%s -- storing all initialized physical "
+                               "volumes of volume group \"%s\" on disk(s)\n",
+                               cmd, vg_name);
+      if ( ( ret = pv_write_all_pv_of_vg ( vg)) < 0) {
+         fprintf ( stderr, "%s -- ERROR %d storing all %u initilized physical "
+                           "volumes of volume group \"%s\"\n\n",
+                           cmd, ret, vg->pv_cur, vg_name);
+         return LVM_EVGREMOVE_PV_WRITE_ALL_PV_OF_VG;
+      }
+   
+      if ( opt_v > 0) printf ( "%s -- removing special files of "
+                               "volume group \"%s\"\n", cmd, vg_name);
+      if ( vg_remove_dir_and_group_and_nodes ( vg_name) < 0) {
+         fprintf ( stderr, "%s -- ERROR remving special files of volume "
+                           "group \"%s\"\n",
+                           cmd, vg_name);
+      }
+   
+      if ( opt_v > 0) printf ( "%s -- removing \"%s\" entry of volume "
+                               "group \"%s\"\n",
+                               cmd, LVMTAB, vg_name);
+      if ( ( ret = lvm_tab_vg_remove ( vg_name)) < 0) {
+         fprintf ( stderr, "%s -- ERROR %d removing volume group \"%s\" "
+                           "entry from \"%s\"\n",
+                           cmd, ret, vg_name, LVMTAB);
+         return LVM_EVGREMOVE_VG_REMOVE;
+      }
+   
+      printf ( "%s -- volume group \"%s\" successfully removed\n",
+               cmd, vg_name);
       lvm_interrupt ();
    }
 
